@@ -16,18 +16,20 @@ import (
 	"go/token"
 	"go/types"
 	"log"
+	"path/filepath"
 	"regexp"
 	"strconv"
 	"strings"
 
 	"golang.org/x/tools/go/packages"
+	"golang.org/x/tools/gopls/internal/bug"
+	"golang.org/x/tools/gopls/internal/file"
 	"golang.org/x/tools/gopls/internal/lsp/command"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/safetoken"
 	"golang.org/x/tools/gopls/internal/lsp/source"
-	"golang.org/x/tools/gopls/internal/span"
+	"golang.org/x/tools/gopls/internal/settings"
 	"golang.org/x/tools/internal/analysisinternal"
-	"golang.org/x/tools/internal/bug"
 	"golang.org/x/tools/internal/typesinternal"
 )
 
@@ -35,32 +37,39 @@ import (
 // diagnostic, using the provided metadata and filesource.
 //
 // The slice of diagnostics may be empty.
-func goPackagesErrorDiagnostics(ctx context.Context, e packages.Error, m *source.Metadata, fs source.FileSource) ([]*source.Diagnostic, error) {
+func goPackagesErrorDiagnostics(ctx context.Context, e packages.Error, m *Metadata, fs file.Source) ([]*source.Diagnostic, error) {
 	if diag, err := parseGoListImportCycleError(ctx, e, m, fs); err != nil {
 		return nil, err
 	} else if diag != nil {
 		return []*source.Diagnostic{diag}, nil
 	}
 
-	var spn span.Span
-	if e.Pos == "" {
-		spn = parseGoListError(e.Msg, m.LoadDir)
-		// We may not have been able to parse a valid span. Apply the errors to all files.
-		if _, err := spanToRange(ctx, fs, spn); err != nil {
-			var diags []*source.Diagnostic
-			for _, uri := range m.CompiledGoFiles {
-				diags = append(diags, &source.Diagnostic{
-					URI:      uri,
-					Severity: protocol.SeverityError,
-					Source:   source.ListError,
-					Message:  e.Msg,
-				})
-			}
-			return diags, nil
+	// Parse error location and attempt to convert to protocol form.
+	loc, err := func() (protocol.Location, error) {
+		filename, line, col8 := parseGoListError(e, m.LoadDir)
+		uri := protocol.URIFromPath(filename)
+
+		fh, err := fs.ReadFile(ctx, uri)
+		if err != nil {
+			return protocol.Location{}, err
 		}
-	} else {
-		spn = span.ParseInDir(e.Pos, m.LoadDir)
-	}
+		content, err := fh.Content()
+		if err != nil {
+			return protocol.Location{}, err
+		}
+		mapper := protocol.NewMapper(uri, content)
+		posn, err := mapper.LineCol8Position(line, col8)
+		if err != nil {
+			return protocol.Location{}, err
+		}
+		return protocol.Location{
+			URI: uri,
+			Range: protocol.Range{
+				Start: posn,
+				End:   posn,
+			},
+		}, nil
+	}()
 
 	// TODO(rfindley): in some cases the go command outputs invalid spans, for
 	// example (from TestGoListErrors):
@@ -73,13 +82,23 @@ func goPackagesErrorDiagnostics(ctx context.Context, e packages.Error, m *source
 	// likely because *token.File lacks information about newline termination.
 	//
 	// We could do better here by handling that case.
-	rng, err := spanToRange(ctx, fs, spn)
 	if err != nil {
-		return nil, err
+		// Unable to parse a valid position.
+		// Apply the error to all files to be safe.
+		var diags []*source.Diagnostic
+		for _, uri := range m.CompiledGoFiles {
+			diags = append(diags, &source.Diagnostic{
+				URI:      uri,
+				Severity: protocol.SeverityError,
+				Source:   source.ListError,
+				Message:  e.Msg,
+			})
+		}
+		return diags, nil
 	}
 	return []*source.Diagnostic{{
-		URI:      spn.URI(),
-		Range:    rng,
+		URI:      loc.URI,
+		Range:    loc.Range,
 		Severity: protocol.SeverityError,
 		Source:   source.ListError,
 		Message:  e.Msg,
@@ -92,7 +111,7 @@ func parseErrorDiagnostics(pkg *syntaxPackage, errList scanner.ErrorList) ([]*so
 		return nil, fmt.Errorf("no errors in %v", errList)
 	}
 	e := errList[0]
-	pgf, err := pkg.File(span.URIFromPath(e.Pos.Filename))
+	pgf, err := pkg.File(protocol.URIFromPath(e.Pos.Filename))
 	if err != nil {
 		return nil, err
 	}
@@ -118,7 +137,7 @@ func typeErrorDiagnostics(moduleMode bool, linkTarget string, pkg *syntaxPackage
 		return nil, err
 	}
 	diag := &source.Diagnostic{
-		URI:      loc.URI.SpanURI(),
+		URI:      loc.URI,
 		Range:    loc.Range,
 		Severity: protocol.SeverityError,
 		Source:   source.TypeError,
@@ -151,13 +170,13 @@ func typeErrorDiagnostics(moduleMode bool, linkTarget string, pkg *syntaxPackage
 	}
 
 	if match := importErrorRe.FindStringSubmatch(e.primary.Msg); match != nil {
-		diag.SuggestedFixes, err = goGetQuickFixes(moduleMode, loc.URI.SpanURI(), match[1])
+		diag.SuggestedFixes, err = goGetQuickFixes(moduleMode, loc.URI, match[1])
 		if err != nil {
 			return nil, err
 		}
 	}
 	if match := unsupportedFeatureRe.FindStringSubmatch(e.primary.Msg); match != nil {
-		diag.SuggestedFixes, err = editGoDirectiveQuickFix(moduleMode, loc.URI.SpanURI(), match[1])
+		diag.SuggestedFixes, err = editGoDirectiveQuickFix(moduleMode, loc.URI, match[1])
 		if err != nil {
 			return nil, err
 		}
@@ -165,37 +184,37 @@ func typeErrorDiagnostics(moduleMode bool, linkTarget string, pkg *syntaxPackage
 	return []*source.Diagnostic{diag}, nil
 }
 
-func goGetQuickFixes(moduleMode bool, uri span.URI, pkg string) ([]source.SuggestedFix, error) {
+func goGetQuickFixes(moduleMode bool, uri protocol.DocumentURI, pkg string) ([]source.SuggestedFix, error) {
 	// Go get only supports module mode for now.
 	if !moduleMode {
 		return nil, nil
 	}
 	title := fmt.Sprintf("go get package %v", pkg)
 	cmd, err := command.NewGoGetPackageCommand(title, command.GoGetPackageArgs{
-		URI:        protocol.URIFromSpanURI(uri),
+		URI:        uri,
 		AddRequire: true,
 		Pkg:        pkg,
 	})
 	if err != nil {
 		return nil, err
 	}
-	return []source.SuggestedFix{source.SuggestedFixFromCommand(cmd, protocol.QuickFix)}, nil
+	return []source.SuggestedFix{SuggestedFixFromCommand(cmd, protocol.QuickFix)}, nil
 }
 
-func editGoDirectiveQuickFix(moduleMode bool, uri span.URI, version string) ([]source.SuggestedFix, error) {
+func editGoDirectiveQuickFix(moduleMode bool, uri protocol.DocumentURI, version string) ([]source.SuggestedFix, error) {
 	// Go mod edit only supports module mode.
 	if !moduleMode {
 		return nil, nil
 	}
 	title := fmt.Sprintf("go mod edit -go=%s", version)
 	cmd, err := command.NewEditGoDirectiveCommand(title, command.EditGoDirectiveArgs{
-		URI:     protocol.URIFromSpanURI(uri),
+		URI:     uri,
 		Version: version,
 	})
 	if err != nil {
 		return nil, err
 	}
-	return []source.SuggestedFix{source.SuggestedFixFromCommand(cmd, protocol.QuickFix)}, nil
+	return []source.SuggestedFix{SuggestedFixFromCommand(cmd, protocol.QuickFix)}, nil
 }
 
 // encodeDiagnostics gob-encodes the given diagnostics.
@@ -212,7 +231,7 @@ func encodeDiagnostics(srcDiags []*source.Diagnostic) []byte {
 				for _, srcEdit := range srcEdits {
 					gobFix.TextEdits = append(gobFix.TextEdits, gobTextEdit{
 						Location: protocol.Location{
-							URI:   protocol.URIFromSpanURI(uri),
+							URI:   uri,
 							Range: srcEdit.Range,
 						},
 						NewText: []byte(srcEdit.NewText),
@@ -235,7 +254,7 @@ func encodeDiagnostics(srcDiags []*source.Diagnostic) []byte {
 		}
 		gobDiag := gobDiagnostic{
 			Location: protocol.Location{
-				URI:   protocol.URIFromSpanURI(srcDiag.URI),
+				URI:   srcDiag.URI,
 				Range: srcDiag.Range,
 			},
 			Severity:       srcDiag.Severity,
@@ -249,13 +268,13 @@ func encodeDiagnostics(srcDiags []*source.Diagnostic) []byte {
 		}
 		gobDiags = append(gobDiags, gobDiag)
 	}
-	return mustJSONEncode(gobDiags)
+	return diagnosticsCodec.Encode(gobDiags)
 }
 
 // decodeDiagnostics decodes the given gob-encoded diagnostics.
 func decodeDiagnostics(data []byte) []*source.Diagnostic {
 	var gobDiags []gobDiagnostic
-	mustJSONDecode(data, &gobDiags)
+	diagnosticsCodec.Decode(data, &gobDiags)
 	var srcDiags []*source.Diagnostic
 	for _, gobDiag := range gobDiags {
 		var srcFixes []source.SuggestedFix
@@ -266,13 +285,13 @@ func decodeDiagnostics(data []byte) []*source.Diagnostic {
 			}
 			for _, gobEdit := range gobFix.TextEdits {
 				if srcFix.Edits == nil {
-					srcFix.Edits = make(map[span.URI][]protocol.TextEdit)
+					srcFix.Edits = make(map[protocol.DocumentURI][]protocol.TextEdit)
 				}
 				srcEdit := protocol.TextEdit{
 					Range:   gobEdit.Location.Range,
 					NewText: string(gobEdit.NewText),
 				}
-				uri := gobEdit.Location.URI.SpanURI()
+				uri := gobEdit.Location.URI
 				srcFix.Edits[uri] = append(srcFix.Edits[uri], srcEdit)
 			}
 			if gobCmd := gobFix.Command; gobCmd != nil {
@@ -290,7 +309,7 @@ func decodeDiagnostics(data []byte) []*source.Diagnostic {
 			srcRelated = append(srcRelated, srcRel)
 		}
 		srcDiag := &source.Diagnostic{
-			URI:            gobDiag.Location.URI.SpanURI(),
+			URI:            gobDiag.Location.URI,
 			Range:          gobDiag.Location.Range,
 			Severity:       gobDiag.Severity,
 			Code:           gobDiag.Code,
@@ -307,7 +326,7 @@ func decodeDiagnostics(data []byte) []*source.Diagnostic {
 }
 
 // toSourceDiagnostic converts a gobDiagnostic to "source" form.
-func toSourceDiagnostic(srcAnalyzer *source.Analyzer, gobDiag *gobDiagnostic) *source.Diagnostic {
+func toSourceDiagnostic(srcAnalyzer *settings.Analyzer, gobDiag *gobDiagnostic) *source.Diagnostic {
 	var related []protocol.DiagnosticRelatedInformation
 	for _, gobRelated := range gobDiag.Related {
 		related = append(related, protocol.DiagnosticRelatedInformation(gobRelated))
@@ -317,21 +336,6 @@ func toSourceDiagnostic(srcAnalyzer *source.Analyzer, gobDiag *gobDiagnostic) *s
 	if len(srcAnalyzer.ActionKind) == 0 {
 		kinds = append(kinds, protocol.QuickFix)
 	}
-	fixes := suggestedAnalysisFixes(gobDiag, kinds)
-	if srcAnalyzer.Fix != "" {
-		cmd, err := command.NewApplyFixCommand(gobDiag.Message, command.ApplyFixArgs{
-			URI:   gobDiag.Location.URI,
-			Range: gobDiag.Location.Range,
-			Fix:   srcAnalyzer.Fix,
-		})
-		if err != nil {
-			// JSON marshalling of these argument values cannot fail.
-			log.Fatalf("internal error in NewApplyFixCommand: %v", err)
-		}
-		for _, kind := range kinds {
-			fixes = append(fixes, source.SuggestedFixFromCommand(cmd, kind))
-		}
-	}
 
 	severity := srcAnalyzer.Severity
 	if severity == 0 {
@@ -339,22 +343,44 @@ func toSourceDiagnostic(srcAnalyzer *source.Analyzer, gobDiag *gobDiagnostic) *s
 	}
 
 	diag := &source.Diagnostic{
-		URI:            gobDiag.Location.URI.SpanURI(),
-		Range:          gobDiag.Location.Range,
-		Severity:       severity,
-		Source:         source.AnalyzerErrorKind(gobDiag.Source),
-		Message:        gobDiag.Message,
-		Related:        related,
-		SuggestedFixes: fixes,
+		URI:      gobDiag.Location.URI,
+		Range:    gobDiag.Location.Range,
+		Severity: severity,
+		Code:     gobDiag.Code,
+		CodeHref: gobDiag.CodeHref,
+		Source:   source.AnalyzerErrorKind(gobDiag.Source),
+		Message:  gobDiag.Message,
+		Related:  related,
+		Tags:     srcAnalyzer.Tag,
 	}
+	if source.CanFix(srcAnalyzer, diag) {
+		fixes := suggestedAnalysisFixes(gobDiag, kinds)
+		if srcAnalyzer.Fix != "" {
+			cmd, err := command.NewApplyFixCommand(gobDiag.Message, command.ApplyFixArgs{
+				URI:   gobDiag.Location.URI,
+				Range: gobDiag.Location.Range,
+				Fix:   string(srcAnalyzer.Fix),
+			})
+			if err != nil {
+				// JSON marshalling of these argument values cannot fail.
+				log.Fatalf("internal error in NewApplyFixCommand: %v", err)
+			}
+			for _, kind := range kinds {
+				fixes = append(fixes, SuggestedFixFromCommand(cmd, kind))
+			}
+		}
+		diag.SuggestedFixes = fixes
+	}
+
 	// If the fixes only delete code, assume that the diagnostic is reporting dead code.
-	if onlyDeletions(fixes) {
-		diag.Tags = []protocol.DiagnosticTag{protocol.Unnecessary}
+	if onlyDeletions(diag.SuggestedFixes) {
+		diag.Tags = append(diag.Tags, protocol.Unnecessary)
 	}
 	return diag
 }
 
-// onlyDeletions returns true if all of the suggested fixes are deletions.
+// onlyDeletions returns true if fixes is non-empty and all of the suggested
+// fixes are deletions.
 func onlyDeletions(fixes []source.SuggestedFix) bool {
 	for _, fix := range fixes {
 		if fix.Command != nil {
@@ -381,9 +407,9 @@ func typesCodeHref(linkTarget string, code typesinternal.ErrorCode) string {
 func suggestedAnalysisFixes(diag *gobDiagnostic, kinds []protocol.CodeActionKind) []source.SuggestedFix {
 	var fixes []source.SuggestedFix
 	for _, fix := range diag.SuggestedFixes {
-		edits := make(map[span.URI][]protocol.TextEdit)
+		edits := make(map[protocol.DocumentURI][]protocol.TextEdit)
 		for _, e := range fix.TextEdits {
-			uri := span.URI(e.Location.URI)
+			uri := e.Location.URI
 			edits[uri] = append(edits[uri], protocol.TextEdit{
 				Range:   e.Location.Range,
 				NewText: string(e.NewText),
@@ -422,7 +448,7 @@ func typeErrorData(pkg *syntaxPackage, terr types.Error) (typesinternal.ErrorCod
 	if !posn.IsValid() {
 		return 0, protocol.Location{}, fmt.Errorf("position %d of type error %q (code %q) not found in FileSet", start, start, terr)
 	}
-	pgf, err := pkg.File(span.URIFromPath(posn.Filename))
+	pgf, err := pkg.File(protocol.URIFromPath(posn.Filename))
 	if err != nil {
 		return 0, protocol.Location{}, err
 	}
@@ -433,43 +459,60 @@ func typeErrorData(pkg *syntaxPackage, terr types.Error) (typesinternal.ErrorCod
 	return ecode, loc, err
 }
 
-// spanToRange converts a span.Span to a protocol.Range, by mapping content
-// contained in the provided FileSource.
-func spanToRange(ctx context.Context, fs source.FileSource, spn span.Span) (protocol.Range, error) {
-	uri := spn.URI()
-	fh, err := fs.ReadFile(ctx, uri)
-	if err != nil {
-		return protocol.Range{}, err
+func parseGoListError(e packages.Error, dir string) (filename string, line, col8 int) {
+	input := e.Pos
+	if input == "" {
+		// No position. Attempt to parse one out of a
+		// go list error of the form "file:line:col:
+		// message" by stripping off the message.
+		input = strings.TrimSpace(e.Msg)
+		if i := strings.Index(input, ": "); i >= 0 {
+			input = input[:i]
+		}
 	}
-	content, err := fh.Content()
-	if err != nil {
-		return protocol.Range{}, err
+
+	filename, line, col8 = splitFileLineCol(input)
+	if !filepath.IsAbs(filename) {
+		filename = filepath.Join(dir, filename)
 	}
-	mapper := protocol.NewMapper(uri, content)
-	return mapper.SpanRange(spn)
+	return filename, line, col8
 }
 
-// parseGoListError attempts to parse a standard `go list` error message
-// by stripping off the trailing error message.
-//
-// It works only on errors whose message is prefixed by colon,
-// followed by a space (": "). For example:
-//
-//	attributes.go:13:1: expected 'package', found 'type'
-func parseGoListError(input, wd string) span.Span {
-	input = strings.TrimSpace(input)
-	msgIndex := strings.Index(input, ": ")
-	if msgIndex < 0 {
-		return span.Parse(input)
+// splitFileLineCol splits s into "filename:line:col",
+// where line and col consist of decimal digits.
+func splitFileLineCol(s string) (file string, line, col8 int) {
+	// Beware that the filename may contain colon on Windows.
+
+	// stripColonDigits removes a ":%d" suffix, if any.
+	stripColonDigits := func(s string) (rest string, num int) {
+		if i := strings.LastIndex(s, ":"); i >= 0 {
+			if v, err := strconv.ParseInt(s[i+1:], 10, 32); err == nil {
+				return s[:i], int(v)
+			}
+		}
+		return s, -1
 	}
-	return span.ParseInDir(input[:msgIndex], wd)
+
+	// strip col ":%d"
+	s, n1 := stripColonDigits(s)
+	if n1 < 0 {
+		return s, 0, 0 // "filename"
+	}
+
+	// strip line ":%d"
+	s, n2 := stripColonDigits(s)
+	if n2 < 0 {
+		return s, n1, 0 // "filename:line"
+	}
+
+	return s, n2, n1 // "filename:line:col"
 }
 
 // parseGoListImportCycleError attempts to parse the given go/packages error as
 // an import cycle, returning a diagnostic if successful.
 //
 // If the error is not detected as an import cycle error, it returns nil, nil.
-func parseGoListImportCycleError(ctx context.Context, e packages.Error, m *source.Metadata, fs source.FileSource) (*source.Diagnostic, error) {
+func parseGoListImportCycleError(ctx context.Context, e packages.Error, m *Metadata, fs file.Source) (*source.Diagnostic, error) {
 	re := regexp.MustCompile(`(.*): import stack: \[(.+)\]`)
 	matches := re.FindStringSubmatch(strings.TrimSpace(e.Msg))
 	if len(matches) < 3 {
@@ -516,19 +559,21 @@ func parseGoListImportCycleError(ctx context.Context, e packages.Error, m *sourc
 // to use in a list of file of a package, for example.
 //
 // It returns an error if the file could not be read.
-func parseGoURI(ctx context.Context, fs source.FileSource, uri span.URI, mode parser.Mode) (*source.ParsedGoFile, error) {
+//
+// TODO(rfindley): eliminate this helper.
+func parseGoURI(ctx context.Context, fs file.Source, uri protocol.DocumentURI, mode parser.Mode) (*source.ParsedGoFile, error) {
 	fh, err := fs.ReadFile(ctx, uri)
 	if err != nil {
 		return nil, err
 	}
-	return parseGoImpl(ctx, token.NewFileSet(), fh, mode)
+	return parseGoImpl(ctx, token.NewFileSet(), fh, mode, false)
 }
 
 // parseModURI is a helper to parse the Mod file at the given URI from the file
 // source fs.
 //
 // It returns an error if the file could not be read.
-func parseModURI(ctx context.Context, fs source.FileSource, uri span.URI) (*source.ParsedModule, error) {
+func parseModURI(ctx context.Context, fs file.Source, uri protocol.DocumentURI) (*source.ParsedModule, error) {
 	fh, err := fs.ReadFile(ctx, uri)
 	if err != nil {
 		return nil, err
