@@ -17,9 +17,10 @@ import (
 
 	"golang.org/x/tools/go/packages"
 	"golang.org/x/tools/gopls/internal/bug"
+	"golang.org/x/tools/gopls/internal/file"
+	"golang.org/x/tools/gopls/internal/immutable"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
-	"golang.org/x/tools/gopls/internal/lsp/source"
-	"golang.org/x/tools/gopls/internal/span"
+	"golang.org/x/tools/gopls/internal/pathutil"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/event/tag"
 	"golang.org/x/tools/internal/gocommand"
@@ -38,7 +39,7 @@ var errNoPackages = errors.New("no packages returned")
 // errors associated with specific modules.
 //
 // If scopes contains a file scope there must be exactly one scope.
-func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadScope) (err error) {
+func (s *Snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadScope) (err error) {
 	id := atomic.AddUint64(&loadID, 1)
 	eventName := fmt.Sprintf("go/packages.Load #%d", id) // unique name for logging
 
@@ -62,12 +63,12 @@ func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadSc
 			// information. For example go/packages returns at most one command-line
 			// arguments package, and does not handle a combination of standalone
 			// files and packages.
-			uri := span.URI(scope)
+			uri := protocol.DocumentURI(scope)
 			if len(scopes) > 1 {
 				panic(fmt.Sprintf("internal error: load called with multiple scopes when a file scope is present (file: %s)", uri))
 			}
 			fh := s.FindFile(uri)
-			if fh == nil || s.FileKind(fh) != source.Go {
+			if fh == nil || s.FileKind(fh) != file.Go {
 				// Don't try to load a file that doesn't exist, or isn't a go file.
 				continue
 			}
@@ -75,17 +76,17 @@ func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadSc
 			if err != nil {
 				continue
 			}
-			if isStandaloneFile(contents, s.options.StandaloneTags) {
+			if isStandaloneFile(contents, s.Options().StandaloneTags) {
 				standalone = true
-				query = append(query, uri.Filename())
+				query = append(query, uri.Path())
 			} else {
-				query = append(query, fmt.Sprintf("file=%s", uri.Filename()))
+				query = append(query, fmt.Sprintf("file=%s", uri.Path()))
 			}
 
 		case moduleLoadScope:
 			modQuery := fmt.Sprintf("%s%c...", scope.dir, filepath.Separator)
 			query = append(query, modQuery)
-			moduleQueries[modQuery] = string(scope.modulePath)
+			moduleQueries[modQuery] = scope.modulePath
 
 		case viewLoadScope:
 			// If we are outside of GOPATH, a module, or some other known
@@ -112,12 +113,12 @@ func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadSc
 	ctx, done := event.Start(ctx, "cache.snapshot.load", tag.Query.Of(query))
 	defer done()
 
-	flags := source.LoadWorkspace
+	flags := LoadWorkspace
 	if allowNetwork {
-		flags |= source.AllowNetwork
+		flags |= AllowNetwork
 	}
 	_, inv, cleanup, err := s.goCommandInvocation(ctx, flags, &gocommand.Invocation{
-		WorkingDir: s.view.goCommandDir.Filename(),
+		WorkingDir: s.view.goCommandDir.Path(),
 	})
 	if err != nil {
 		return err
@@ -141,7 +142,7 @@ func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadSc
 	}
 
 	// This log message is sought for by TestReloadOnlyOnce.
-	labels := append(source.SnapshotLabels(s), tag.Query.Of(query), tag.PackageCount.Of(len(pkgs)))
+	labels := append(s.Labels(), tag.Query.Of(query), tag.PackageCount.Of(len(pkgs)))
 	if err != nil {
 		event.Error(ctx, eventName, err, labels...)
 	} else {
@@ -160,8 +161,8 @@ func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadSc
 	}
 
 	moduleErrs := make(map[string][]packages.Error) // module path -> errors
-	filterFunc := s.filterFunc()
-	newMetadata := make(map[PackageID]*source.Metadata)
+	filterFunc := s.view.filterFunc()
+	newMetadata := make(map[PackageID]*Metadata)
 	for _, pkg := range pkgs {
 		// The Go command returns synthetic list results for module queries that
 		// encountered module errors.
@@ -178,9 +179,9 @@ func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadSc
 			continue
 		}
 
-		if !containsDir || s.options.VerboseOutput {
+		if !containsDir || s.Options().VerboseOutput {
 			event.Log(ctx, eventName, append(
-				source.SnapshotLabels(s),
+				s.Labels(),
 				tag.Package.Of(pkg.ID),
 				tag.Files.Of(pkg.CompiledGoFiles))...)
 		}
@@ -225,9 +226,9 @@ func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadSc
 
 	// Compute the minimal metadata updates (for Clone)
 	// required to preserve the above invariant.
-	var files []span.URI // files to preload
-	seenFiles := make(map[span.URI]bool)
-	updates := make(map[PackageID]*source.Metadata)
+	var files []protocol.DocumentURI // files to preload
+	seenFiles := make(map[protocol.DocumentURI]bool)
+	updates := make(map[PackageID]*Metadata)
 	for _, m := range newMetadata {
 		if existing := s.meta.metadata[m.ID]; existing == nil {
 			// Record any new files we should pre-load.
@@ -238,7 +239,7 @@ func (s *snapshot) load(ctx context.Context, allowNetwork bool, scopes ...loadSc
 				}
 			}
 			updates[m.ID] = m
-			delete(s.shouldLoad, m.ID)
+			s.shouldLoad.Delete(m.ID)
 		}
 	}
 
@@ -310,7 +311,7 @@ func (m *moduleErrorMap) Error() string {
 //
 // TODO(rfindley): separate workspace diagnostics from critical workspace
 // errors.
-func (s *snapshot) workspaceLayoutError(ctx context.Context) (error, []*source.Diagnostic) {
+func (s *Snapshot) workspaceLayoutError(ctx context.Context) (error, []*Diagnostic) {
 	// TODO(rfindley): both of the checks below should be delegated to the workspace.
 
 	if s.view.effectiveGO111MODULE() == off {
@@ -354,31 +355,31 @@ https://github.com/golang/tools/blob/master/gopls/doc/workspace.md.`
 	return nil, nil
 }
 
-func (s *snapshot) applyCriticalErrorToFiles(ctx context.Context, msg string, files []*Overlay) []*source.Diagnostic {
-	var srcDiags []*source.Diagnostic
+func (s *Snapshot) applyCriticalErrorToFiles(ctx context.Context, msg string, files []*Overlay) []*Diagnostic {
+	var srcDiags []*Diagnostic
 	for _, fh := range files {
 		// Place the diagnostics on the package or module declarations.
 		var rng protocol.Range
 		switch s.FileKind(fh) {
-		case source.Go:
-			if pgf, err := s.ParseGo(ctx, fh, source.ParseHeader); err == nil {
+		case file.Go:
+			if pgf, err := s.ParseGo(ctx, fh, ParseHeader); err == nil {
 				// Check that we have a valid `package foo` range to use for positioning the error.
 				if pgf.File.Package.IsValid() && pgf.File.Name != nil && pgf.File.Name.End().IsValid() {
 					rng, _ = pgf.PosRange(pgf.File.Package, pgf.File.Name.End())
 				}
 			}
-		case source.Mod:
+		case file.Mod:
 			if pmf, err := s.ParseMod(ctx, fh); err == nil {
 				if mod := pmf.File.Module; mod != nil && mod.Syntax != nil {
 					rng, _ = pmf.Mapper.OffsetRange(mod.Syntax.Start.Byte, mod.Syntax.End.Byte)
 				}
 			}
 		}
-		srcDiags = append(srcDiags, &source.Diagnostic{
+		srcDiags = append(srcDiags, &Diagnostic{
 			URI:      fh.URI(),
 			Range:    rng,
 			Severity: protocol.SeverityError,
-			Source:   source.ListError,
+			Source:   ListError,
 			Message:  msg,
 		})
 	}
@@ -388,12 +389,12 @@ func (s *snapshot) applyCriticalErrorToFiles(ctx context.Context, msg string, fi
 // buildMetadata populates the updates map with metadata updates to
 // apply, based on the given pkg. It recurs through pkg.Imports to ensure that
 // metadata exists for all dependencies.
-func buildMetadata(updates map[PackageID]*source.Metadata, pkg *packages.Package, loadDir string, standalone bool) {
+func buildMetadata(updates map[PackageID]*Metadata, pkg *packages.Package, loadDir string, standalone bool) {
 	// Allow for multiple ad-hoc packages in the workspace (see #47584).
 	pkgPath := PackagePath(pkg.PkgPath)
 	id := PackageID(pkg.ID)
 
-	if source.IsCommandLineArguments(id) {
+	if IsCommandLineArguments(id) {
 		if len(pkg.CompiledGoFiles) != 1 {
 			bug.Reportf("unexpected files in command-line-arguments package: %v", pkg.CompiledGoFiles)
 			return
@@ -413,8 +414,12 @@ func buildMetadata(updates map[PackageID]*source.Metadata, pkg *packages.Package
 		return
 	}
 
+	if pkg.TypesSizes == nil {
+		panic(id + ".TypeSizes is nil")
+	}
+
 	// Recreate the metadata rather than reusing it to avoid locking.
-	m := &source.Metadata{
+	m := &Metadata{
 		ID:         id,
 		PkgPath:    pkgPath,
 		Name:       PackageName(pkg.Name),
@@ -430,15 +435,15 @@ func buildMetadata(updates map[PackageID]*source.Metadata, pkg *packages.Package
 	updates[id] = m
 
 	for _, filename := range pkg.CompiledGoFiles {
-		uri := span.URIFromPath(filename)
+		uri := protocol.URIFromPath(filename)
 		m.CompiledGoFiles = append(m.CompiledGoFiles, uri)
 	}
 	for _, filename := range pkg.GoFiles {
-		uri := span.URIFromPath(filename)
+		uri := protocol.URIFromPath(filename)
 		m.GoFiles = append(m.GoFiles, uri)
 	}
 	for _, filename := range pkg.IgnoredFiles {
-		uri := span.URIFromPath(filename)
+		uri := protocol.URIFromPath(filename)
 		m.IgnoredFiles = append(m.IgnoredFiles, uri)
 	}
 
@@ -553,7 +558,7 @@ func buildMetadata(updates map[PackageID]*source.Metadata, pkg *packages.Package
 // computeLoadDiagnostics computes and sets m.Diagnostics for the given metadata m.
 //
 // It should only be called during metadata construction in snapshot.load.
-func computeLoadDiagnostics(ctx context.Context, m *source.Metadata, meta *metadataGraph, fs source.FileSource, workspacePackages map[PackageID]PackagePath) {
+func computeLoadDiagnostics(ctx context.Context, m *Metadata, meta *metadataGraph, fs file.Source, workspacePackages immutable.Map[PackageID, PackagePath]) {
 	for _, packagesErr := range m.Errors {
 		// Filter out parse errors from go list. We'll get them when we
 		// actually parse, and buggy overlay support may generate spurious
@@ -589,7 +594,7 @@ func computeLoadDiagnostics(ctx context.Context, m *source.Metadata, meta *metad
 // snapshot s.
 //
 // s.mu must be held while calling this function.
-func containsPackageLocked(s *snapshot, m *source.Metadata) bool {
+func containsPackageLocked(s *Snapshot, m *Metadata) bool {
 	// In legacy workspace mode, or if a package does not have an associated
 	// module, a package is considered inside the workspace if any of its files
 	// are under the workspace root (and not excluded).
@@ -602,13 +607,13 @@ func containsPackageLocked(s *snapshot, m *source.Metadata) bool {
 	// gowork != "". It should suffice to consider workspace mod files (also, we
 	// will hopefully eliminate the concept of a workspace package soon).
 	if m.Module != nil && s.view.gowork != "" {
-		modURI := span.URIFromPath(m.Module.GoMod)
-		_, ok := s.workspaceModFiles[modURI]
+		modURI := protocol.URIFromPath(m.Module.GoMod)
+		_, ok := s.view.workspaceModFiles[modURI]
 		if !ok {
 			return false
 		}
 
-		uris := map[span.URI]struct{}{}
+		uris := map[protocol.DocumentURI]struct{}{}
 		for _, uri := range m.CompiledGoFiles {
 			uris[uri] = struct{}{}
 		}
@@ -616,7 +621,7 @@ func containsPackageLocked(s *snapshot, m *source.Metadata) bool {
 			uris[uri] = struct{}{}
 		}
 
-		filterFunc := s.filterFunc()
+		filterFunc := s.view.filterFunc()
 		for uri := range uris {
 			// Don't use view.contains here. go.work files may include modules
 			// outside of the workspace folder.
@@ -627,15 +632,15 @@ func containsPackageLocked(s *snapshot, m *source.Metadata) bool {
 		return false
 	}
 
-	return containsFileInWorkspaceLocked(s, m)
+	return containsFileInWorkspaceLocked(s.view, m)
 }
 
 // containsOpenFileLocked reports whether any file referenced by m is open in
 // the snapshot s.
 //
 // s.mu must be held while calling this function.
-func containsOpenFileLocked(s *snapshot, m *source.Metadata) bool {
-	uris := map[span.URI]struct{}{}
+func containsOpenFileLocked(s *Snapshot, m *Metadata) bool {
+	uris := map[protocol.DocumentURI]struct{}{}
 	for _, uri := range m.CompiledGoFiles {
 		uris[uri] = struct{}{}
 	}
@@ -656,8 +661,8 @@ func containsOpenFileLocked(s *snapshot, m *source.Metadata) bool {
 // workspace of the snapshot s.
 //
 // s.mu must be held while calling this function.
-func containsFileInWorkspaceLocked(s *snapshot, m *source.Metadata) bool {
-	uris := map[span.URI]struct{}{}
+func containsFileInWorkspaceLocked(v *View, m *Metadata) bool {
+	uris := map[protocol.DocumentURI]struct{}{}
 	for _, uri := range m.CompiledGoFiles {
 		uris[uri] = struct{}{}
 	}
@@ -671,7 +676,7 @@ func containsFileInWorkspaceLocked(s *snapshot, m *source.Metadata) bool {
 
 		// The package's files are in this view. It may be a workspace package.
 		// Vendored packages are not likely to be interesting to the user.
-		if !strings.Contains(string(uri), "/vendor/") && s.contains(uri) {
+		if !strings.Contains(string(uri), "/vendor/") && v.contains(uri) {
 			return true
 		}
 	}
@@ -683,14 +688,14 @@ func containsFileInWorkspaceLocked(s *snapshot, m *source.Metadata) bool {
 // contain intermediate test variants.
 //
 // s.mu must be held while calling this function.
-func computeWorkspacePackagesLocked(s *snapshot, meta *metadataGraph) map[PackageID]PackagePath {
+func computeWorkspacePackagesLocked(s *Snapshot, meta *metadataGraph) immutable.Map[PackageID, PackagePath] {
 	workspacePackages := make(map[PackageID]PackagePath)
 	for _, m := range meta.metadata {
 		if !containsPackageLocked(s, m) {
 			continue
 		}
 
-		if source.IsCommandLineArguments(m.ID) {
+		if IsCommandLineArguments(m.ID) {
 			// If all the files contained in m have a real package, we don't need to
 			// keep m as a workspace package.
 			if allFilesHaveRealPackages(meta, m) {
@@ -718,7 +723,7 @@ func computeWorkspacePackagesLocked(s *snapshot, meta *metadataGraph) map[Packag
 			workspacePackages[m.ID] = m.ForTest
 		}
 	}
-	return workspacePackages
+	return immutable.MapOf(workspacePackages)
 }
 
 // allFilesHaveRealPackages reports whether all files referenced by m are
@@ -728,12 +733,12 @@ func computeWorkspacePackagesLocked(s *snapshot, meta *metadataGraph) map[Packag
 // function returns false.
 //
 // If m is not a command-line-arguments package, this is trivially true.
-func allFilesHaveRealPackages(g *metadataGraph, m *source.Metadata) bool {
+func allFilesHaveRealPackages(g *metadataGraph, m *Metadata) bool {
 	n := len(m.CompiledGoFiles)
 checkURIs:
 	for _, uri := range append(m.CompiledGoFiles[0:n:n], m.GoFiles...) {
 		for _, id := range g.ids[uri] {
-			if !source.IsCommandLineArguments(id) {
+			if !IsCommandLineArguments(id) {
 				continue checkURIs
 			}
 		}
@@ -755,7 +760,7 @@ func isTestMain(pkg *packages.Package, gocache string) bool {
 	if len(pkg.GoFiles) > 1 {
 		return false
 	}
-	if !source.InDir(gocache, pkg.GoFiles[0]) {
+	if !pathutil.InDir(gocache, pkg.GoFiles[0]) {
 		return false
 	}
 	return true
