@@ -10,9 +10,10 @@ import (
 	"go/token"
 	"math/bits"
 	"testing"
+	"time"
 
-	"golang.org/x/tools/gopls/internal/lsp/source"
-	"golang.org/x/tools/gopls/internal/span"
+	"golang.org/x/tools/gopls/internal/file"
+	"golang.org/x/tools/gopls/internal/lsp/protocol"
 )
 
 func skipIfNoParseCache(t *testing.T) {
@@ -25,17 +26,17 @@ func TestParseCache(t *testing.T) {
 	skipIfNoParseCache(t)
 
 	ctx := context.Background()
-	uri := span.URI("file:///myfile")
+	uri := protocol.DocumentURI("file:///myfile")
 	fh := makeFakeFileHandle(uri, []byte("package p\n\nconst _ = \"foo\""))
 	fset := token.NewFileSet()
 
-	var cache parseCache
-	pgfs1, err := cache.parseFiles(ctx, fset, source.ParseFull, fh)
+	cache := newParseCache(0)
+	pgfs1, err := cache.parseFiles(ctx, fset, ParseFull, false, fh)
 	if err != nil {
 		t.Fatal(err)
 	}
 	pgf1 := pgfs1[0]
-	pgfs2, err := cache.parseFiles(ctx, fset, source.ParseFull, fh)
+	pgfs2, err := cache.parseFiles(ctx, fset, ParseFull, false, fh)
 	pgf2 := pgfs2[0]
 	if err != nil {
 		t.Fatal(err)
@@ -45,9 +46,14 @@ func TestParseCache(t *testing.T) {
 	}
 
 	// Fill up the cache with other files, but don't evict the file above.
-	files := []source.FileHandle{fh}
-	files = append(files, dummyFileHandles(parseCacheMaxFiles-1)...)
-	pgfs3, err := cache.parseFiles(ctx, fset, source.ParseFull, files...)
+	cache.gcOnce()
+	files := []file.Handle{fh}
+	files = append(files, dummyFileHandles(parseCacheMinFiles-1)...)
+
+	pgfs3, err := cache.parseFiles(ctx, fset, ParseFull, false, files...)
+	if err != nil {
+		t.Fatal(err)
+	}
 	pgf3 := pgfs3[0]
 	if pgf3 != pgf1 {
 		t.Errorf("parseFiles(%q, ...): unexpected cache miss", uri)
@@ -60,12 +66,15 @@ func TestParseCache(t *testing.T) {
 	}
 
 	// Now overwrite the cache, after which we should get new results.
-	files = dummyFileHandles(parseCacheMaxFiles)
-	_, err = cache.parseFiles(ctx, fset, source.ParseFull, files...)
+	cache.gcOnce()
+	files = dummyFileHandles(parseCacheMinFiles)
+	_, err = cache.parseFiles(ctx, fset, ParseFull, false, files...)
 	if err != nil {
 		t.Fatal(err)
 	}
-	pgfs4, err := cache.parseFiles(ctx, fset, source.ParseFull, fh)
+	// force a GC, which should collect the recently parsed files
+	cache.gcOnce()
+	pgfs4, err := cache.parseFiles(ctx, fset, ParseFull, false, fh)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -82,14 +91,14 @@ func TestParseCache_Reparsing(t *testing.T) {
 	}(parsePadding)
 	parsePadding = 0
 
-	files := dummyFileHandles(parseCacheMaxFiles)
+	files := dummyFileHandles(parseCacheMinFiles)
 	danglingSelector := []byte("package p\nfunc _() {\n\tx.\n}")
 	files = append(files, makeFakeFileHandle("file:///bad1", danglingSelector))
 	files = append(files, makeFakeFileHandle("file:///bad2", danglingSelector))
 
 	// Parsing should succeed even though we overflow the padding.
-	var cache parseCache
-	_, err := cache.parseFiles(context.Background(), token.NewFileSet(), source.ParseFull, files...)
+	cache := newParseCache(0)
+	_, err := cache.parseFiles(context.Background(), token.NewFileSet(), ParseFull, false, files...)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -105,25 +114,76 @@ func TestParseCache_Issue59097(t *testing.T) {
 	parsePadding = 0
 
 	danglingSelector := []byte("package p\nfunc _() {\n\tx.\n}")
-	files := []source.FileHandle{makeFakeFileHandle("file:///bad", danglingSelector)}
+	files := []file.Handle{makeFakeFileHandle("file:///bad", danglingSelector)}
 
 	// Parsing should succeed even though we overflow the padding.
-	var cache parseCache
-	_, err := cache.parseFiles(context.Background(), token.NewFileSet(), source.ParseFull, files...)
+	cache := newParseCache(0)
+	_, err := cache.parseFiles(context.Background(), token.NewFileSet(), ParseFull, false, files...)
 	if err != nil {
 		t.Fatal(err)
 	}
 }
 
+func TestParseCache_TimeEviction(t *testing.T) {
+	skipIfNoParseCache(t)
+
+	ctx := context.Background()
+	fset := token.NewFileSet()
+	uri := protocol.DocumentURI("file:///myfile")
+	fh := makeFakeFileHandle(uri, []byte("package p\n\nconst _ = \"foo\""))
+
+	const gcDuration = 10 * time.Millisecond
+	cache := newParseCache(gcDuration)
+	cache.stop() // we'll manage GC manually, for testing.
+
+	pgfs0, err := cache.parseFiles(ctx, fset, ParseFull, false, fh, fh)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	files := dummyFileHandles(parseCacheMinFiles)
+	_, err = cache.parseFiles(ctx, fset, ParseFull, false, files...)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	// Even after filling up the 'min' files, we get a cache hit for our original file.
+	pgfs1, err := cache.parseFiles(ctx, fset, ParseFull, false, fh, fh)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	if pgfs0[0] != pgfs1[0] {
+		t.Errorf("before GC, got unexpected cache miss")
+	}
+
+	// But after GC, we get a cache miss.
+	_, err = cache.parseFiles(ctx, fset, ParseFull, false, files...) // mark dummy files as newer
+	if err != nil {
+		t.Fatal(err)
+	}
+	time.Sleep(gcDuration)
+	cache.gcOnce()
+
+	pgfs2, err := cache.parseFiles(ctx, fset, ParseFull, false, fh, fh)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	if pgfs0[0] == pgfs2[0] {
+		t.Errorf("after GC, got unexpected cache hit for %s", pgfs0[0].URI)
+	}
+}
+
 func TestParseCache_Duplicates(t *testing.T) {
 	skipIfNoParseCache(t)
 
 	ctx := context.Background()
-	uri := span.URI("file:///myfile")
+	uri := protocol.DocumentURI("file:///myfile")
 	fh := makeFakeFileHandle(uri, []byte("package p\n\nconst _ = \"foo\""))
 
-	var cache parseCache
-	pgfs, err := cache.parseFiles(ctx, token.NewFileSet(), source.ParseFull, fh, fh)
+	cache := newParseCache(0)
+	pgfs, err := cache.parseFiles(ctx, token.NewFileSet(), ParseFull, false, fh, fh)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -132,32 +192,32 @@ func TestParseCache_Duplicates(t *testing.T) {
 	}
 }
 
-func dummyFileHandles(n int) []source.FileHandle {
-	var fhs []source.FileHandle
+func dummyFileHandles(n int) []file.Handle {
+	var fhs []file.Handle
 	for i := 0; i < n; i++ {
-		uri := span.URI(fmt.Sprintf("file:///_%d", i))
+		uri := protocol.DocumentURI(fmt.Sprintf("file:///_%d", i))
 		src := []byte(fmt.Sprintf("package p\nvar _ = %d", i))
 		fhs = append(fhs, makeFakeFileHandle(uri, src))
 	}
 	return fhs
 }
 
-func makeFakeFileHandle(uri span.URI, src []byte) fakeFileHandle {
+func makeFakeFileHandle(uri protocol.DocumentURI, src []byte) fakeFileHandle {
 	return fakeFileHandle{
 		uri:  uri,
 		data: src,
-		hash: source.HashOf(src),
+		hash: file.HashOf(src),
 	}
 }
 
 type fakeFileHandle struct {
-	source.FileHandle
-	uri  span.URI
+	file.Handle
+	uri  protocol.DocumentURI
 	data []byte
-	hash source.Hash
+	hash file.Hash
 }
 
-func (h fakeFileHandle) URI() span.URI {
+func (h fakeFileHandle) URI() protocol.DocumentURI {
 	return h.uri
 }
 
@@ -165,8 +225,8 @@ func (h fakeFileHandle) Content() ([]byte, error) {
 	return h.data, nil
 }
 
-func (h fakeFileHandle) FileIdentity() source.FileIdentity {
-	return source.FileIdentity{
+func (h fakeFileHandle) Identity() file.Identity {
+	return file.Identity{
 		URI:  h.uri,
 		Hash: h.hash,
 	}
