@@ -6,20 +6,25 @@ package cache
 
 import (
 	"context"
+	"errors"
 	"fmt"
+	"os"
+	"path/filepath"
 	"strconv"
 	"strings"
 	"sync"
 	"sync/atomic"
 
-	"golang.org/x/tools/gopls/internal/govulncheck"
-	"golang.org/x/tools/gopls/internal/lsp/source"
-	"golang.org/x/tools/gopls/internal/span"
-	"golang.org/x/tools/internal/bug"
+	"golang.org/x/tools/gopls/internal/bug"
+	"golang.org/x/tools/gopls/internal/file"
+	"golang.org/x/tools/gopls/internal/lsp/protocol"
+	"golang.org/x/tools/gopls/internal/lsp/source/typerefs"
+	"golang.org/x/tools/gopls/internal/persistent"
+	"golang.org/x/tools/gopls/internal/vulncheck"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/gocommand"
 	"golang.org/x/tools/internal/imports"
-	"golang.org/x/tools/internal/persistent"
+	"golang.org/x/tools/internal/memoize"
 	"golang.org/x/tools/internal/xcontext"
 )
 
@@ -31,12 +36,11 @@ type Session struct {
 	cache       *Cache            // shared cache
 	gocmdRunner *gocommand.Runner // limits go command concurrency
 
-	optionsMu sync.Mutex
-	options   *source.Options
-
 	viewMu  sync.Mutex
 	views   []*View
-	viewMap map[span.URI]*View // map of URI->best view
+	viewMap map[protocol.DocumentURI]*View // file->best view
+
+	parseCache *parseCache
 
 	*overlayFS
 }
@@ -45,18 +49,9 @@ type Session struct {
 func (s *Session) ID() string     { return s.id }
 func (s *Session) String() string { return s.id }
 
-// Options returns a copy of the SessionOptions for this session.
-func (s *Session) Options() *source.Options {
-	s.optionsMu.Lock()
-	defer s.optionsMu.Unlock()
-	return s.options
-}
-
-// SetOptions sets the options of this session to new values.
-func (s *Session) SetOptions(options *source.Options) {
-	s.optionsMu.Lock()
-	defer s.optionsMu.Unlock()
-	s.options = options
+// GoCommandRunner returns the gocommand Runner for this session.
+func (s *Session) GoCommandRunner() *gocommand.Runner {
+	return s.gocmdRunner
 }
 
 // Shutdown the session and all views it has created.
@@ -70,6 +65,7 @@ func (s *Session) Shutdown(ctx context.Context) {
 	for _, view := range views {
 		view.shutdown()
 	}
+	s.parseCache.stop()
 	event.Log(ctx, "Shutdown session", KeyShutdownSession.Of(s))
 }
 
@@ -78,42 +74,49 @@ func (s *Session) Cache() *Cache {
 	return s.cache
 }
 
+// TODO(rfindley): is the logic surrounding this error actually necessary?
+var ErrViewExists = errors.New("view already exists for session")
+
 // NewView creates a new View, returning it and its first snapshot. If a
 // non-empty tempWorkspace directory is provided, the View will record a copy
 // of its gopls workspace module in that directory, so that client tooling
 // can execute in the same main module.  On success it also returns a release
 // function that must be called when the Snapshot is no longer needed.
-func (s *Session) NewView(ctx context.Context, name string, folder span.URI, options *source.Options) (*View, source.Snapshot, func(), error) {
+func (s *Session) NewView(ctx context.Context, folder *Folder) (*View, *Snapshot, func(), error) {
 	s.viewMu.Lock()
 	defer s.viewMu.Unlock()
-	for _, view := range s.views {
-		if span.SameExistingFile(view.folder, folder) {
-			return nil, nil, nil, source.ErrViewExists
+
+	// Querying the file system to check whether
+	// two folders denote the same existing directory.
+	if inode1, err := os.Stat(filepath.FromSlash(folder.Dir.Path())); err == nil {
+		for _, view := range s.views {
+			inode2, err := os.Stat(filepath.FromSlash(view.folder.Dir.Path()))
+			if err == nil && os.SameFile(inode1, inode2) {
+				return nil, nil, nil, ErrViewExists
+			}
 		}
 	}
-	view, snapshot, release, err := s.createView(ctx, name, folder, options, 0)
+
+	def, err := getViewDefinition(ctx, s.gocmdRunner, s, folder)
+	if err != nil {
+		return nil, nil, nil, err
+	}
+	view, snapshot, release, err := s.createView(ctx, def, folder, 0)
 	if err != nil {
 		return nil, nil, nil, err
 	}
 	s.views = append(s.views, view)
 	// we always need to drop the view map
-	s.viewMap = make(map[span.URI]*View)
+	s.viewMap = make(map[protocol.DocumentURI]*View)
 	return view, snapshot, release, nil
 }
 
 // TODO(rfindley): clarify that createView can never be cancelled (with the
 // possible exception of server shutdown).
-func (s *Session) createView(ctx context.Context, name string, folder span.URI, options *source.Options, seqID uint64) (*View, *snapshot, func(), error) {
+// On success, the caller becomes responsible for calling the release function once.
+func (s *Session) createView(ctx context.Context, def *viewDefinition, folder *Folder, seqID uint64) (*View, *Snapshot, func(), error) {
 	index := atomic.AddInt64(&viewIndex, 1)
 
-	// Get immutable workspace information.
-	info, err := s.getWorkspaceInformation(ctx, folder, options)
-	if err != nil {
-		return nil, nil, func() {}, err
-	}
-
-	wsModFiles, wsModFilesErr := computeWorkspaceModFiles(ctx, info.gomod, info.effectiveGOWORK(), info.effectiveGO111MODULE(), s)
-
 	// We want a true background context and not a detached context here
 	// the spans need to be unrelated and no tag values should pollute it.
 	baseCtx := event.Detach(xcontext.Detach(ctx))
@@ -122,58 +125,53 @@ func (s *Session) createView(ctx context.Context, name string, folder span.URI,
 	v := &View{
 		id:                   strconv.FormatInt(index, 10),
 		gocmdRunner:          s.gocmdRunner,
+		folder:               folder,
 		initialWorkspaceLoad: make(chan struct{}),
 		initializationSema:   make(chan struct{}, 1),
-		options:              options,
 		baseCtx:              baseCtx,
-		name:                 name,
-		folder:               folder,
-		moduleUpgrades:       map[span.URI]map[string]string{},
-		vulns:                map[span.URI]*govulncheck.Result{},
+		parseCache:           s.parseCache,
 		fs:                   s.overlayFS,
-		workspaceInformation: info,
+		viewDefinition:       def,
 	}
 	v.importsState = &importsState{
 		ctx: backgroundCtx,
 		processEnv: &imports.ProcessEnv{
 			GocmdRunner: s.gocmdRunner,
 			SkipPathInScan: func(dir string) bool {
-				prefix := strings.TrimSuffix(string(v.folder), "/") + "/"
-				uri := strings.TrimSuffix(string(span.URIFromPath(dir)), "/")
+				prefix := strings.TrimSuffix(string(v.folder.Dir), "/") + "/"
+				uri := strings.TrimSuffix(string(protocol.URIFromPath(dir)), "/")
 				if !strings.HasPrefix(uri+"/", prefix) {
 					return false
 				}
-				filterer := source.NewFilterer(options.DirectoryFilters)
+				filterer := NewFilterer(folder.Options.DirectoryFilters)
 				rel := strings.TrimPrefix(uri, prefix)
 				disallow := filterer.Disallow(rel)
 				return disallow
 			},
 		},
 	}
-	v.snapshot = &snapshot{
-		sequenceID:           seqID,
-		globalID:             nextSnapshotID(),
-		view:                 v,
-		backgroundCtx:        backgroundCtx,
-		cancel:               cancel,
-		store:                s.cache.store,
-		packages:             persistent.NewMap(packageIDLessInterface),
-		meta:                 new(metadataGraph),
-		files:                newFilesMap(),
-		parseCache:           new(parseCache),
-		activePackages:       persistent.NewMap(packageIDLessInterface),
-		symbolizeHandles:     persistent.NewMap(uriLessInterface),
-		analyses:             persistent.NewMap(analysisKeyLessInterface),
-		workspacePackages:    make(map[PackageID]PackagePath),
-		unloadableFiles:      make(map[span.URI]struct{}),
-		parseModHandles:      persistent.NewMap(uriLessInterface),
-		parseWorkHandles:     persistent.NewMap(uriLessInterface),
-		modTidyHandles:       persistent.NewMap(uriLessInterface),
-		modVulnHandles:       persistent.NewMap(uriLessInterface),
-		modWhyHandles:        persistent.NewMap(uriLessInterface),
-		knownSubdirs:         newKnownDirsSet(),
-		workspaceModFiles:    wsModFiles,
-		workspaceModFilesErr: wsModFilesErr,
+	v.snapshot = &Snapshot{
+		sequenceID:       seqID,
+		globalID:         nextSnapshotID(),
+		view:             v,
+		backgroundCtx:    backgroundCtx,
+		cancel:           cancel,
+		store:            s.cache.store,
+		packages:         new(persistent.Map[PackageID, *packageHandle]),
+		meta:             new(metadataGraph),
+		files:            newFileMap(),
+		activePackages:   new(persistent.Map[PackageID, *Package]),
+		symbolizeHandles: new(persistent.Map[protocol.DocumentURI, *memoize.Promise]),
+		shouldLoad:       new(persistent.Map[PackageID, []PackagePath]),
+		unloadableFiles:  new(persistent.Set[protocol.DocumentURI]),
+		parseModHandles:  new(persistent.Map[protocol.DocumentURI, *memoize.Promise]),
+		parseWorkHandles: new(persistent.Map[protocol.DocumentURI, *memoize.Promise]),
+		modTidyHandles:   new(persistent.Map[protocol.DocumentURI, *memoize.Promise]),
+		modVulnHandles:   new(persistent.Map[protocol.DocumentURI, *memoize.Promise]),
+		modWhyHandles:    new(persistent.Map[protocol.DocumentURI, *memoize.Promise]),
+		pkgIndex:         typerefs.NewPackageIndex(),
+		moduleUpgrades:   new(persistent.Map[protocol.DocumentURI, map[string]string]),
+		vulns:            new(persistent.Map[protocol.DocumentURI, *vulncheck.Result]),
 	}
 	// Save one reference in the view.
 	v.releaseSnapshot = v.snapshot.Acquire()
@@ -197,8 +195,8 @@ func (s *Session) createView(ctx context.Context, name string, folder span.URI,
 	return v, snapshot, snapshot.Acquire(), nil
 }
 
-// View returns a view with a matching name, if the session has one.
-func (s *Session) View(name string) *View {
+// ViewByName returns a view with a matching name, if the session has one.
+func (s *Session) ViewByName(name string) *View {
 	s.viewMu.Lock()
 	defer s.viewMu.Unlock()
 	for _, view := range s.views {
@@ -209,16 +207,28 @@ func (s *Session) View(name string) *View {
 	return nil
 }
 
+// View returns the view with a matching id, if present.
+func (s *Session) View(id string) (*View, error) {
+	s.viewMu.Lock()
+	defer s.viewMu.Unlock()
+	for _, view := range s.views {
+		if view.ID() == id {
+			return view, nil
+		}
+	}
+	return nil, fmt.Errorf("no view with ID %q", id)
+}
+
 // ViewOf returns a view corresponding to the given URI.
 // If the file is not already associated with a view, pick one using some heuristics.
-func (s *Session) ViewOf(uri span.URI) (*View, error) {
+func (s *Session) ViewOf(uri protocol.DocumentURI) (*View, error) {
 	s.viewMu.Lock()
 	defer s.viewMu.Unlock()
 	return s.viewOfLocked(uri)
 }
 
 // Precondition: caller holds s.viewMu lock.
-func (s *Session) viewOfLocked(uri span.URI) (*View, error) {
+func (s *Session) viewOfLocked(uri protocol.DocumentURI) (*View, error) {
 	// Check if we already know this file.
 	if v, found := s.viewMap[uri]; found {
 		return v, nil
@@ -241,11 +251,11 @@ func (s *Session) Views() []*View {
 
 // bestViewForURI returns the most closely matching view for the given URI
 // out of the given set of views.
-func bestViewForURI(uri span.URI, views []*View) *View {
+func bestViewForURI(uri protocol.DocumentURI, views []*View) *View {
 	// we need to find the best view for this file
 	var longest *View
 	for _, view := range views {
-		if longest != nil && len(longest.Folder()) > len(view.Folder()) {
+		if longest != nil && len(longest.folder.Dir) > len(view.folder.Dir) {
 			continue
 		}
 		// TODO(rfindley): this should consider the workspace layout (i.e.
@@ -271,6 +281,7 @@ func bestViewForURI(uri span.URI, views []*View) *View {
 func (s *Session) RemoveView(view *View) {
 	s.viewMu.Lock()
 	defer s.viewMu.Unlock()
+
 	i := s.dropView(view)
 	if i == -1 { // error reported elsewhere
 		return
@@ -280,24 +291,18 @@ func (s *Session) RemoveView(view *View) {
 	s.views = removeElement(s.views, i)
 }
 
-// updateView recreates the view with the given options.
+// updateViewLocked recreates the view with the given options.
 //
 // If the resulting error is non-nil, the view may or may not have already been
 // dropped from the session.
-func (s *Session) updateView(ctx context.Context, view *View, options *source.Options) (*View, error) {
-	s.viewMu.Lock()
-	defer s.viewMu.Unlock()
-
-	return s.updateViewLocked(ctx, view, options)
-}
-
-func (s *Session) updateViewLocked(ctx context.Context, view *View, options *source.Options) (*View, error) {
+func (s *Session) updateViewLocked(ctx context.Context, view *View, def *viewDefinition, folder *Folder) (*View, error) {
 	// Preserve the snapshot ID if we are recreating the view.
 	view.snapshotMu.Lock()
 	if view.snapshot == nil {
 		view.snapshotMu.Unlock()
 		panic("updateView called after View was already shut down")
 	}
+	// TODO(rfindley): we should probably increment the sequence ID here.
 	seqID := view.snapshot.sequenceID // Preserve sequence IDs when updating a view in place.
 	view.snapshotMu.Unlock()
 
@@ -306,9 +311,12 @@ func (s *Session) updateViewLocked(ctx context.Context, view *View, options *sou
 		return nil, fmt.Errorf("view %q not found", view.id)
 	}
 
-	v, _, release, err := s.createView(ctx, view.name, view.folder, options, seqID)
-	release()
-
+	var (
+		snapshot *Snapshot
+		release  func()
+		err      error
+	)
+	view, snapshot, release, err = s.createView(ctx, def, folder, seqID)
 	if err != nil {
 		// we have dropped the old view, but could not create the new one
 		// this should not happen and is very bad, but we still need to clean
@@ -316,9 +324,21 @@ func (s *Session) updateViewLocked(ctx context.Context, view *View, options *sou
 		s.views = removeElement(s.views, i)
 		return nil, err
 	}
+	defer release()
+
+	// The new snapshot has lost the history of the previous view. As a result,
+	// it may not see open files that aren't in its build configuration (as it
+	// would have done via didOpen notifications). This can lead to inconsistent
+	// behavior when configuration is changed mid-session.
+	//
+	// Ensure the new snapshot observes all open files.
+	for _, o := range view.fs.Overlays() {
+		_, _ = snapshot.ReadFile(ctx, o.URI())
+	}
+
 	// substitute the new view into the array where the old view was
-	s.views[i] = v
-	return v, nil
+	s.views[i] = view
+	return view, nil
 }
 
 // removeElement removes the ith element from the slice replacing it with the last element.
@@ -335,7 +355,7 @@ func removeElement(slice []*View, index int) []*View {
 // not found. s.viewMu must be held while calling this function.
 func (s *Session) dropView(v *View) int {
 	// we always need to drop the view map
-	s.viewMap = make(map[span.URI]*View)
+	s.viewMap = make(map[protocol.DocumentURI]*View)
 	for i := range s.views {
 		if v == s.views[i] {
 			// we found the view, drop it and return the index it was found at
@@ -350,23 +370,12 @@ func (s *Session) dropView(v *View) int {
 	return -1
 }
 
-func (s *Session) ModifyFiles(ctx context.Context, changes []source.FileModification) error {
-	_, release, err := s.DidModifyFiles(ctx, changes)
-	release()
-	return err
-}
-
-// TODO(rfindley): fileChange seems redundant with source.FileModification.
-// De-dupe into a common representation for changes.
-type fileChange struct {
-	content    []byte
-	exists     bool
-	fileHandle source.FileHandle
-
-	// isUnchanged indicates whether the file action is one that does not
-	// change the actual contents of the file. Opens and closes should not
-	// be treated like other changes, since the file content doesn't change.
-	isUnchanged bool
+// ResetView resets the best view for the given URI.
+func (s *Session) ResetView(ctx context.Context, uri protocol.DocumentURI) (*View, error) {
+	s.viewMu.Lock()
+	defer s.viewMu.Unlock()
+	v := bestViewForURI(uri, s.views)
+	return s.updateViewLocked(ctx, v, v.viewDefinition, v.folder)
 }
 
 // DidModifyFiles reports a file modification to the session. It returns
@@ -378,7 +387,7 @@ type fileChange struct {
 // TODO(rfindley): what happens if this function fails? It must leave us in a
 // broken state, which we should surface to the user, probably as a request to
 // restart gopls.
-func (s *Session) DidModifyFiles(ctx context.Context, changes []source.FileModification) (map[source.Snapshot][]span.URI, func(), error) {
+func (s *Session) DidModifyFiles(ctx context.Context, changes []file.Modification) (map[*Snapshot][]protocol.DocumentURI, func(), error) {
 	s.viewMu.Lock()
 	defer s.viewMu.Unlock()
 
@@ -402,17 +411,21 @@ func (s *Session) DidModifyFiles(ctx context.Context, changes []source.FileModif
 	checkViews := false
 
 	for _, c := range changes {
-		if isGoMod(c.URI) || isGoWork(c.URI) {
-			// Change, InvalidateMetadata, and UnknownFileAction actions do not cause
-			// us to re-evaluate views.
-			redoViews := (c.Action != source.Change &&
-				c.Action != source.InvalidateMetadata &&
-				c.Action != source.UnknownFileAction)
-
-			if redoViews {
-				checkViews = true
-				break
-			}
+		// Any on-disk change to a go.work file causes a re-diagnosis.
+		//
+		// TODO(rfindley): go.work files need not be named "go.work" -- we need to
+		// check each view's source to handle the case of an explicit GOWORK value.
+		// Write a test that fails, and fix this.
+		if isGoWork(c.URI) && (c.Action == file.Save || c.OnDisk) {
+			checkViews = true
+			break
+		}
+		// Opening/Close/Create/Delete of go.mod files all trigger
+		// re-evaluation of Views. Changes do not as they can't affect the set of
+		// Views.
+		if isGoMod(c.URI) && c.Action != file.Change && c.Action != file.Save {
+			checkViews = true
+			break
 		}
 	}
 
@@ -422,7 +435,7 @@ func (s *Session) DidModifyFiles(ctx context.Context, changes []source.FileModif
 			// synchronously to change processing? Can we assume that the env did not
 			// change, and derive go.work using a combination of the configured
 			// GOWORK value and filesystem?
-			info, err := s.getWorkspaceInformation(ctx, view.folder, view.Options())
+			info, err := getViewDefinition(ctx, s.gocmdRunner, s, view.folder)
 			if err != nil {
 				// Catastrophic failure, equivalent to a failure of session
 				// initialization and therefore should almost never happen. One
@@ -433,11 +446,8 @@ func (s *Session) DidModifyFiles(ctx context.Context, changes []source.FileModif
 				// TODO(rfindley): consider surfacing this error more loudly. We
 				// could report a bug, but it's not really a bug.
 				event.Error(ctx, "fetching workspace information", err)
-			}
-
-			if info != view.workspaceInformation {
-				_, err := s.updateViewLocked(ctx, view, view.Options())
-				if err != nil {
+			} else if !viewDefinitionsEqual(view.viewDefinition, info) {
+				if _, err := s.updateViewLocked(ctx, view, info, view.folder); err != nil {
 					// More catastrophic failure. The view may or may not still exist.
 					// The best we can do is log and move on.
 					event.Error(ctx, "recreating view", err)
@@ -447,15 +457,9 @@ func (s *Session) DidModifyFiles(ctx context.Context, changes []source.FileModif
 	}
 
 	// Collect information about views affected by these changes.
-	views := make(map[*View]map[span.URI]*fileChange)
-	affectedViews := map[span.URI][]*View{}
-	// forceReloadMetadata records whether any change is the magic
-	// source.InvalidateMetadata action.
-	forceReloadMetadata := false
+	views := make(map[*View]map[protocol.DocumentURI]file.Handle)
+	affectedViews := map[protocol.DocumentURI][]*View{}
 	for _, c := range changes {
-		if c.Action == source.InvalidateMetadata {
-			forceReloadMetadata = true
-		}
 		// Build the list of affected views.
 		var changedViews []*View
 		for _, view := range s.views {
@@ -480,37 +484,22 @@ func (s *Session) DidModifyFiles(ctx context.Context, changes []source.FileModif
 		}
 		affectedViews[c.URI] = changedViews
 
-		isUnchanged := c.Action == source.Open || c.Action == source.Close
-
 		// Apply the changes to all affected views.
+		fh := mustReadFile(ctx, s, c.URI)
 		for _, view := range changedViews {
 			// Make sure that the file is added to the view's seenFiles set.
 			view.markKnown(c.URI)
 			if _, ok := views[view]; !ok {
-				views[view] = make(map[span.URI]*fileChange)
-			}
-			fh, err := s.ReadFile(ctx, c.URI)
-			if err != nil {
-				return nil, nil, err
-			}
-			content, err := fh.Content()
-			if err != nil {
-				// Ignore the error: the file may be deleted.
-				content = nil
-			}
-			views[view][c.URI] = &fileChange{
-				content:     content,
-				exists:      err == nil,
-				fileHandle:  fh,
-				isUnchanged: isUnchanged,
+				views[view] = make(map[protocol.DocumentURI]file.Handle)
 			}
+			views[view][c.URI] = fh
 		}
 	}
 
 	var releases []func()
-	viewToSnapshot := map[*View]*snapshot{}
+	viewToSnapshot := make(map[*View]*Snapshot)
 	for view, changed := range views {
-		snapshot, release := view.invalidateContent(ctx, changed, forceReloadMetadata)
+		snapshot, release := view.Invalidate(ctx, StateChange{Files: changed})
 		releases = append(releases, release)
 		viewToSnapshot[view] = snapshot
 	}
@@ -527,7 +516,7 @@ func (s *Session) DidModifyFiles(ctx context.Context, changes []source.FileModif
 	// it "most" belongs. We do this by picking the best view for each URI,
 	// and then aggregating the set of snapshots and their URIs (to avoid
 	// diagnosing the same snapshot multiple times).
-	snapshotURIs := map[source.Snapshot][]span.URI{}
+	snapshotURIs := map[*Snapshot][]protocol.DocumentURI{}
 	for _, mod := range changes {
 		viewSlice, ok := affectedViews[mod.URI]
 		if !ok || len(viewSlice) == 0 {
@@ -536,7 +525,7 @@ func (s *Session) DidModifyFiles(ctx context.Context, changes []source.FileModif
 		view := bestViewForURI(mod.URI, viewSlice)
 		snapshot, ok := viewToSnapshot[view]
 		if !ok {
-			panic(fmt.Sprintf("no snapshot for view %s", view.Folder()))
+			panic(fmt.Sprintf("no snapshot for view %s", view.folder.Dir))
 		}
 		snapshotURIs[snapshot] = append(snapshotURIs[snapshot], mod.URI)
 	}
@@ -547,11 +536,11 @@ func (s *Session) DidModifyFiles(ctx context.Context, changes []source.FileModif
 // ExpandModificationsToDirectories returns the set of changes with the
 // directory changes removed and expanded to include all of the files in
 // the directory.
-func (s *Session) ExpandModificationsToDirectories(ctx context.Context, changes []source.FileModification) []source.FileModification {
-	var snapshots []*snapshot
+func (s *Session) ExpandModificationsToDirectories(ctx context.Context, changes []file.Modification) []file.Modification {
+	var snapshots []*Snapshot
 	s.viewMu.Lock()
 	for _, v := range s.views {
-		snapshot, release, err := v.getSnapshot()
+		snapshot, release, err := v.Snapshot()
 		if err != nil {
 			continue // view is shut down; continue with others
 		}
@@ -560,73 +549,43 @@ func (s *Session) ExpandModificationsToDirectories(ctx context.Context, changes
 	}
 	s.viewMu.Unlock()
 
-	knownDirs := knownDirectories(ctx, snapshots)
-	defer knownDirs.Destroy()
-
-	var result []source.FileModification
+	// Expand the modification to any file we could care about, which we define
+	// to be any file observed by any of the snapshots.
+	//
+	// There may be other files in the directory, but if we haven't read them yet
+	// we don't need to invalidate them.
+	var result []file.Modification
 	for _, c := range changes {
-		if !knownDirs.Contains(c.URI) {
+		expanded := make(map[protocol.DocumentURI]bool)
+		for _, snapshot := range snapshots {
+			for _, uri := range snapshot.filesInDir(c.URI) {
+				expanded[uri] = true
+			}
+		}
+		if len(expanded) == 0 {
 			result = append(result, c)
-			continue
+		} else {
+			for uri := range expanded {
+				result = append(result, file.Modification{
+					URI:        uri,
+					Action:     c.Action,
+					LanguageID: "",
+					OnDisk:     c.OnDisk,
+					// changes to directories cannot include text or versions
+				})
+			}
 		}
-		affectedFiles := knownFilesInDir(ctx, snapshots, c.URI)
-		var fileChanges []source.FileModification
-		for uri := range affectedFiles {
-			fileChanges = append(fileChanges, source.FileModification{
-				URI:        uri,
-				Action:     c.Action,
-				LanguageID: "",
-				OnDisk:     c.OnDisk,
-				// changes to directories cannot include text or versions
-			})
-		}
-		result = append(result, fileChanges...)
 	}
 	return result
 }
 
-// knownDirectories returns all of the directories known to the given
-// snapshots, including workspace directories and their subdirectories.
-// It is responsibility of the caller to destroy the returned set.
-func knownDirectories(ctx context.Context, snapshots []*snapshot) knownDirsSet {
-	result := newKnownDirsSet()
-	for _, snapshot := range snapshots {
-		dirs := snapshot.dirs(ctx)
-		for _, dir := range dirs {
-			result.Insert(dir)
-		}
-		knownSubdirs := snapshot.getKnownSubdirs(dirs)
-		result.SetAll(knownSubdirs)
-		knownSubdirs.Destroy()
-	}
-	return result
-}
-
-// knownFilesInDir returns the files known to the snapshots in the session.
-// It does not respect symlinks.
-func knownFilesInDir(ctx context.Context, snapshots []*snapshot, dir span.URI) map[span.URI]struct{} {
-	files := map[span.URI]struct{}{}
-
-	for _, snapshot := range snapshots {
-		for _, uri := range snapshot.knownFilesInDir(ctx, dir) {
-			files[uri] = struct{}{}
-		}
-	}
-	return files
-}
-
 // Precondition: caller holds s.viewMu lock.
 // TODO(rfindley): move this to fs_overlay.go.
-func (fs *overlayFS) updateOverlays(ctx context.Context, changes []source.FileModification) error {
+func (fs *overlayFS) updateOverlays(ctx context.Context, changes []file.Modification) error {
 	fs.mu.Lock()
 	defer fs.mu.Unlock()
 
 	for _, c := range changes {
-		// Don't update overlays for metadata invalidations.
-		if c.Action == source.InvalidateMetadata {
-			continue
-		}
-
 		o, ok := fs.overlays[c.URI]
 
 		// If the file is not opened in an overlay and the change is on disk,
@@ -637,10 +596,10 @@ func (fs *overlayFS) updateOverlays(ctx context.Context, changes []source.FileMo
 		}
 
 		// Determine the file kind on open, otherwise, assume it has been cached.
-		var kind source.FileKind
+		var kind file.Kind
 		switch c.Action {
-		case source.Open:
-			kind = source.FileKindForLang(c.LanguageID)
+		case file.Open:
+			kind = file.KindForLang(c.LanguageID)
 		default:
 			if !ok {
 				return fmt.Errorf("updateOverlays: modifying unopened overlay %v", c.URI)
@@ -649,7 +608,7 @@ func (fs *overlayFS) updateOverlays(ctx context.Context, changes []source.FileMo
 		}
 
 		// Closing a file just deletes its overlay.
-		if c.Action == source.Close {
+		if c.Action == file.Close {
 			delete(fs.overlays, c.URI)
 			continue
 		}
@@ -658,7 +617,7 @@ func (fs *overlayFS) updateOverlays(ctx context.Context, changes []source.FileMo
 		// overlay. Saves and on-disk file changes don't come with the file's
 		// content.
 		text := c.Text
-		if text == nil && (c.Action == source.Save || c.OnDisk) {
+		if text == nil && (c.Action == file.Save || c.OnDisk) {
 			if !ok {
 				return fmt.Errorf("no known content for overlay for %s", c.Action)
 			}
@@ -666,15 +625,15 @@ func (fs *overlayFS) updateOverlays(ctx context.Context, changes []source.FileMo
 		}
 		// On-disk changes don't come with versions.
 		version := c.Version
-		if c.OnDisk || c.Action == source.Save {
+		if c.OnDisk || c.Action == file.Save {
 			version = o.version
 		}
-		hash := source.HashOf(text)
+		hash := file.HashOf(text)
 		var sameContentOnDisk bool
 		switch c.Action {
-		case source.Delete:
+		case file.Delete:
 			// Do nothing. sameContentOnDisk should be false.
-		case source.Save:
+		case file.Save:
 			// Make sure the version and content (if present) is the same.
 			if false && o.version != version { // Client no longer sends the version
 				return fmt.Errorf("updateOverlays: saving %s at version %v, currently at %v", c.URI, c.Version, o.version)
@@ -684,12 +643,9 @@ func (fs *overlayFS) updateOverlays(ctx context.Context, changes []source.FileMo
 			}
 			sameContentOnDisk = true
 		default:
-			fh, err := fs.delegate.ReadFile(ctx, c.URI)
-			if err != nil {
-				return err
-			}
+			fh := mustReadFile(ctx, fs.delegate, c.URI)
 			_, readErr := fh.Content()
-			sameContentOnDisk = (readErr == nil && fh.FileIdentity().Hash == hash)
+			sameContentOnDisk = (readErr == nil && fh.Identity().Hash == hash)
 		}
 		o = &Overlay{
 			uri:     c.URI,
@@ -709,15 +665,39 @@ func (fs *overlayFS) updateOverlays(ctx context.Context, changes []source.FileMo
 	return nil
 }
 
-// FileWatchingGlobPatterns returns glob patterns to watch every directory
-// known by the view. For views within a module, this is the module root,
-// any directory in the module root, and any replace targets.
+func mustReadFile(ctx context.Context, fs file.Source, uri protocol.DocumentURI) file.Handle {
+	ctx = xcontext.Detach(ctx)
+	fh, err := fs.ReadFile(ctx, uri)
+	if err != nil {
+		// ReadFile cannot fail with an uncancellable context.
+		bug.Reportf("reading file failed unexpectedly: %v", err)
+		return brokenFile{uri, err}
+	}
+	return fh
+}
+
+// A brokenFile represents an unexpected failure to read a file.
+type brokenFile struct {
+	uri protocol.DocumentURI
+	err error
+}
+
+func (b brokenFile) URI() protocol.DocumentURI { return b.uri }
+func (b brokenFile) Identity() file.Identity   { return file.Identity{URI: b.uri} }
+func (b brokenFile) SameContentsOnDisk() bool  { return false }
+func (b brokenFile) Version() int32            { return 0 }
+func (b brokenFile) Content() ([]byte, error)  { return nil, b.err }
+
+// FileWatchingGlobPatterns returns a new set of glob patterns to
+// watch every directory known by the view. For views within a module,
+// this is the module root, any directory in the module root, and any
+// replace targets.
 func (s *Session) FileWatchingGlobPatterns(ctx context.Context) map[string]struct{} {
 	s.viewMu.Lock()
 	defer s.viewMu.Unlock()
 	patterns := map[string]struct{}{}
 	for _, view := range s.views {
-		snapshot, release, err := view.getSnapshot()
+		snapshot, release, err := view.Snapshot()
 		if err != nil {
 			continue // view is shut down; continue with others
 		}
