@@ -11,21 +11,20 @@ import (
 	"go/types"
 	"strings"
 
+	"golang.org/x/tools/gopls/internal/astutil"
+	"golang.org/x/tools/gopls/internal/file"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
-	"golang.org/x/tools/gopls/internal/lsp/source"
-	"golang.org/x/tools/gopls/internal/span"
-	"golang.org/x/tools/internal/memoize"
 )
 
 // symbolize returns the result of symbolizing the file identified by uri, using a cache.
-func (s *snapshot) symbolize(ctx context.Context, uri span.URI) ([]source.Symbol, error) {
+func (s *Snapshot) symbolize(ctx context.Context, uri protocol.DocumentURI) ([]Symbol, error) {
 
 	s.mu.Lock()
 	entry, hit := s.symbolizeHandles.Get(uri)
 	s.mu.Unlock()
 
 	type symbolizeResult struct {
-		symbols []source.Symbol
+		symbols []Symbol
 		err     error
 	}
 
@@ -35,10 +34,10 @@ func (s *snapshot) symbolize(ctx context.Context, uri span.URI) ([]source.Symbol
 		if err != nil {
 			return nil, err
 		}
-		type symbolHandleKey source.Hash
-		key := symbolHandleKey(fh.FileIdentity().Hash)
+		type symbolHandleKey file.Hash
+		key := symbolHandleKey(fh.Identity().Hash)
 		promise, release := s.store.Promise(key, func(ctx context.Context, arg interface{}) interface{} {
-			symbols, err := symbolizeImpl(ctx, arg.(*snapshot), fh)
+			symbols, err := symbolizeImpl(ctx, arg.(*Snapshot), fh)
 			return symbolizeResult{symbols, err}
 		})
 
@@ -50,7 +49,7 @@ func (s *snapshot) symbolize(ctx context.Context, uri span.URI) ([]source.Symbol
 	}
 
 	// Await result.
-	v, err := s.awaitPromise(ctx, entry.(*memoize.Promise))
+	v, err := s.awaitPromise(ctx, entry)
 	if err != nil {
 		return nil, err
 	}
@@ -59,10 +58,8 @@ func (s *snapshot) symbolize(ctx context.Context, uri span.URI) ([]source.Symbol
 }
 
 // symbolizeImpl reads and parses a file and extracts symbols from it.
-// It may use a parsed file already present in the cache but
-// otherwise does not populate the cache.
-func symbolizeImpl(ctx context.Context, snapshot *snapshot, fh source.FileHandle) ([]source.Symbol, error) {
-	pgfs, err := snapshot.parseCache.parseFiles(ctx, token.NewFileSet(), source.ParseFull, fh)
+func symbolizeImpl(ctx context.Context, snapshot *Snapshot, fh file.Handle) ([]Symbol, error) {
+	pgfs, err := snapshot.view.parseCache.parseFiles(ctx, token.NewFileSet(), ParseFull, false, fh)
 	if err != nil {
 		return nil, err
 	}
@@ -81,7 +78,7 @@ type symbolWalker struct {
 	tokFile *token.File
 	mapper  *protocol.Mapper
 
-	symbols    []source.Symbol
+	symbols    []Symbol
 	firstError error
 }
 
@@ -100,7 +97,7 @@ func (w *symbolWalker) atNode(node ast.Node, name string, kind protocol.SymbolKi
 		w.error(err)
 		return
 	}
-	sym := source.Symbol{
+	sym := Symbol{
 		Name:  b.String(),
 		Kind:  kind,
 		Range: rng,
@@ -122,7 +119,7 @@ func (w *symbolWalker) fileDecls(decls []ast.Decl) {
 			var recv *ast.Ident
 			if decl.Recv.NumFields() > 0 {
 				kind = protocol.Method
-				recv = unpackRecv(decl.Recv.List[0].Type)
+				_, recv, _ = astutil.UnpackRecv(decl.Recv.List[0].Type)
 			}
 			w.atNode(decl.Name, decl.Name.Name, kind, recv)
 		case *ast.GenDecl:
@@ -158,25 +155,6 @@ func guessKind(spec *ast.TypeSpec) protocol.SymbolKind {
 	return protocol.Class
 }
 
-func unpackRecv(rtyp ast.Expr) *ast.Ident {
-	// Extract the receiver identifier. Lifted from go/types/resolver.go
-L:
-	for {
-		switch t := rtyp.(type) {
-		case *ast.ParenExpr:
-			rtyp = t.X
-		case *ast.StarExpr:
-			rtyp = t.X
-		default:
-			break L
-		}
-	}
-	if name, _ := rtyp.(*ast.Ident); name != nil {
-		return name
-	}
-	return nil
-}
-
 // walkType processes symbols related to a type expression. path is path of
 // nested type identifiers to the type expression.
 func (w *symbolWalker) walkType(typ ast.Expr, path ...*ast.Ident) {
