@@ -13,24 +13,26 @@ import (
 
 	"golang.org/x/tools/go/ast/inspector"
 	"golang.org/x/tools/gopls/internal/bug"
+	"golang.org/x/tools/gopls/internal/file"
 	"golang.org/x/tools/gopls/internal/lsp/analysis/fillstruct"
 	"golang.org/x/tools/gopls/internal/lsp/analysis/infertypeargs"
 	"golang.org/x/tools/gopls/internal/lsp/analysis/stubmethods"
+	"golang.org/x/tools/gopls/internal/lsp/cache"
 	"golang.org/x/tools/gopls/internal/lsp/command"
 	"golang.org/x/tools/gopls/internal/lsp/mod"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/source"
-	"golang.org/x/tools/gopls/internal/span"
+	"golang.org/x/tools/gopls/internal/settings"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/event/tag"
 	"golang.org/x/tools/internal/imports"
 )
 
-func (s *Server) codeAction(ctx context.Context, params *protocol.CodeActionParams) ([]protocol.CodeAction, error) {
+func (s *server) CodeAction(ctx context.Context, params *protocol.CodeActionParams) ([]protocol.CodeAction, error) {
 	ctx, done := event.Start(ctx, "lsp.Server.codeAction")
 	defer done()
 
-	snapshot, fh, ok, release, err := s.beginFileRequest(ctx, params.TextDocument.URI, source.UnknownKind)
+	snapshot, fh, ok, release, err := s.beginFileRequest(ctx, params.TextDocument.URI, file.UnknownKind)
 	defer release()
 	if !ok {
 		return nil, err
@@ -77,7 +79,7 @@ func (s *Server) codeAction(ctx context.Context, params *protocol.CodeActionPara
 	}
 
 	switch kind {
-	case source.Mod:
+	case file.Mod:
 		var actions []protocol.CodeAction
 
 		fixes, err := s.codeActionsMatchingDiagnostics(ctx, fh.URI(), snapshot, params.Context.Diagnostics, want)
@@ -109,7 +111,7 @@ func (s *Server) codeAction(ctx context.Context, params *protocol.CodeActionPara
 
 		return actions, nil
 
-	case source.Go:
+	case file.Go:
 		diagnostics := params.Context.Diagnostics
 
 		// Don't suggest fixes for generated files, since they are generally
@@ -138,7 +140,7 @@ func (s *Server) codeAction(ctx context.Context, params *protocol.CodeActionPara
 			if wantQuickFixes || want[protocol.SourceOrganizeImports] {
 				importEdits, importEditsPerFix, err := source.AllImportsFixes(ctx, snapshot, pgf)
 				if err != nil {
-					event.Error(ctx, "imports fixes", err, tag.File.Of(fh.URI().Filename()))
+					event.Error(ctx, "imports fixes", err, tag.File.Of(fh.URI().Path()))
 					importEdits = nil
 					importEditsPerFix = nil
 				}
@@ -223,8 +225,8 @@ func (s *Server) codeAction(ctx context.Context, params *protocol.CodeActionPara
 						return protocol.CodeAction{}, false, nil
 					}
 					cmd, err := command.NewApplyFixCommand(d.Message, command.ApplyFixArgs{
-						URI:   protocol.URIFromSpanURI(pgf.URI),
-						Fix:   source.StubMethods,
+						URI:   pgf.URI,
+						Fix:   string(settings.StubMethods),
 						Range: pd.Range,
 					})
 					if err != nil {
@@ -278,7 +280,7 @@ func (s *Server) codeAction(ctx context.Context, params *protocol.CodeActionPara
 	}
 }
 
-func (s *Server) findMatchingDiagnostics(uri span.URI, pd protocol.Diagnostic) []*source.Diagnostic {
+func (s *server) findMatchingDiagnostics(uri protocol.DocumentURI, pd protocol.Diagnostic) []*source.Diagnostic {
 	s.diagnosticsMu.Lock()
 	defer s.diagnosticsMu.Unlock()
 
@@ -297,7 +299,7 @@ func (s *Server) findMatchingDiagnostics(uri span.URI, pd protocol.Diagnostic) [
 	return sds
 }
 
-func (s *Server) getSupportedCodeActions() []protocol.CodeActionKind {
+func (s *server) getSupportedCodeActions() []protocol.CodeActionKind {
 	allCodeActionKinds := make(map[protocol.CodeActionKind]struct{})
 	for _, kinds := range s.Options().SupportedCodeActions {
 		for kind := range kinds {
@@ -373,12 +375,12 @@ func refactorExtract(ctx context.Context, snapshot source.Snapshot, pgf *source.
 	if err != nil {
 		return nil, err
 	}
-	puri := protocol.URIFromSpanURI(pgf.URI)
+	puri := pgf.URI
 	var commands []protocol.Command
 	if _, ok, methodOk, _ := source.CanExtractFunction(pgf.Tok, start, end, pgf.Src, pgf.File); ok {
 		cmd, err := command.NewApplyFixCommand("Extract function", command.ApplyFixArgs{
 			URI:   puri,
-			Fix:   source.ExtractFunction,
+			Fix:   string(settings.ExtractFunction),
 			Range: rng,
 		})
 		if err != nil {
@@ -388,7 +390,7 @@ func refactorExtract(ctx context.Context, snapshot source.Snapshot, pgf *source.
 		if methodOk {
 			cmd, err := command.NewApplyFixCommand("Extract method", command.ApplyFixArgs{
 				URI:   puri,
-				Fix:   source.ExtractMethod,
+				Fix:   string(settings.ExtractMethod),
 				Range: rng,
 			})
 			if err != nil {
@@ -400,7 +402,7 @@ func refactorExtract(ctx context.Context, snapshot source.Snapshot, pgf *source.
 	if _, _, ok, _ := source.CanExtractVariable(start, end, pgf.File); ok {
 		cmd, err := command.NewApplyFixCommand("Extract variable", command.ApplyFixArgs{
 			URI:   puri,
-			Fix:   source.ExtractVariable,
+			Fix:   string(settings.ExtractVariable),
 			Range: rng,
 		})
 		if err != nil {
@@ -419,7 +421,7 @@ func refactorExtract(ctx context.Context, snapshot source.Snapshot, pgf *source.
 	return actions, nil
 }
 
-func refactorRewrite(ctx context.Context, snapshot source.Snapshot, pkg source.Package, pgf *source.ParsedGoFile, fh source.FileHandle, rng protocol.Range) (_ []protocol.CodeAction, rerr error) {
+func refactorRewrite(ctx context.Context, snapshot source.Snapshot, pkg source.Package, pgf *source.ParsedGoFile, fh file.Handle, rng protocol.Range) (_ []protocol.CodeAction, rerr error) {
 	// golang/go#61693: code actions were refactored to run outside of the
 	// analysis framework, but as a result they lost their panic recovery.
 	//
@@ -436,7 +438,7 @@ func refactorRewrite(ctx context.Context, snapshot source.Snapshot, pkg source.P
 	if canRemoveParameter(pkg, pgf, rng) {
 		cmd, err := command.NewChangeSignatureCommand("remove unused parameter", command.ChangeSignatureArgs{
 			RemoveParameter: protocol.Location{
-				URI:   protocol.URIFromSpanURI(pgf.URI),
+				URI:   pgf.URI,
 				Range: rng,
 			},
 		})
@@ -450,6 +452,10 @@ func refactorRewrite(ctx context.Context, snapshot source.Snapshot, pkg source.P
 		})
 	}
 
+	if action, ok := source.ConvertStringLiteral(pgf, fh, rng); ok {
+		actions = append(actions, action)
+	}
+
 	start, end, err := pgf.RangePos(rng)
 	if err != nil {
 		return nil, err
@@ -458,8 +464,8 @@ func refactorRewrite(ctx context.Context, snapshot source.Snapshot, pkg source.P
 	var commands []protocol.Command
 	if _, ok, _ := source.CanInvertIfCondition(pgf.File, start, end); ok {
 		cmd, err := command.NewApplyFixCommand("Invert if condition", command.ApplyFixArgs{
-			URI:   protocol.URIFromSpanURI(pgf.URI),
-			Fix:   source.InvertIfCondition,
+			URI:   pgf.URI,
+			Fix:   string(settings.InvertIfCondition),
 			Range: rng,
 		})
 		if err != nil {
@@ -480,8 +486,8 @@ func refactorRewrite(ctx context.Context, snapshot source.Snapshot, pkg source.P
 				return nil, err
 			}
 			cmd, err := command.NewApplyFixCommand(d.Message, command.ApplyFixArgs{
-				URI:   protocol.URIFromSpanURI(pgf.URI),
-				Fix:   source.FillStruct,
+				URI:   pgf.URI,
+				Fix:   string(settings.FillStruct),
 				Range: rng,
 			})
 			if err != nil {
@@ -571,14 +577,14 @@ func canRemoveParameter(pkg source.Package, pgf *source.ParsedGoFile, rng protoc
 }
 
 // refactorInline returns inline actions available at the specified range.
-func refactorInline(ctx context.Context, snapshot source.Snapshot, pkg source.Package, pgf *source.ParsedGoFile, fh source.FileHandle, rng protocol.Range) ([]protocol.CodeAction, error) {
+func refactorInline(ctx context.Context, snapshot source.Snapshot, pkg source.Package, pgf *source.ParsedGoFile, fh file.Handle, rng protocol.Range) ([]protocol.CodeAction, error) {
 	var commands []protocol.Command
 
 	// If range is within call expression, offer inline action.
 	if _, fn, err := source.EnclosingStaticCall(pkg, pgf, rng); err == nil {
 		cmd, err := command.NewApplyFixCommand(fmt.Sprintf("Inline call to %s", fn.Name()), command.ApplyFixArgs{
-			URI:   protocol.URIFromSpanURI(pgf.URI),
-			Fix:   source.InlineCall,
+			URI:   pgf.URI,
+			Fix:   string(settings.InlineCall),
 			Range: rng,
 		})
 		if err != nil {
@@ -599,14 +605,14 @@ func refactorInline(ctx context.Context, snapshot source.Snapshot, pkg source.Pa
 	return actions, nil
 }
 
-func documentChanges(fh source.FileHandle, edits []protocol.TextEdit) []protocol.DocumentChanges {
+func documentChanges(fh file.Handle, edits []protocol.TextEdit) []protocol.DocumentChanges {
 	return []protocol.DocumentChanges{
 		{
 			TextDocumentEdit: &protocol.TextDocumentEdit{
 				TextDocument: protocol.OptionalVersionedTextDocumentIdentifier{
 					Version: fh.Version(),
 					TextDocumentIdentifier: protocol.TextDocumentIdentifier{
-						URI: protocol.URIFromSpanURI(fh.URI()),
+						URI: fh.URI(),
 					},
 				},
 				Edits: nonNilSliceTextEdit(edits),
@@ -620,11 +626,11 @@ func documentChanges(fh source.FileHandle, edits []protocol.TextEdit) []protocol
 // bundled protocol.Diagnostic.Data field, and failing that by falling back on
 // fetching a matching source.Diagnostic from the set of stored diagnostics for
 // this file.
-func (s *Server) codeActionsMatchingDiagnostics(ctx context.Context, uri span.URI, snapshot source.Snapshot, pds []protocol.Diagnostic, want map[protocol.CodeActionKind]bool) ([]protocol.CodeAction, error) {
+func (s *server) codeActionsMatchingDiagnostics(ctx context.Context, uri protocol.DocumentURI, snapshot source.Snapshot, pds []protocol.Diagnostic, want map[protocol.CodeActionKind]bool) ([]protocol.CodeAction, error) {
 	var actions []protocol.CodeAction
 	var unbundled []protocol.Diagnostic // diagnostics without bundled code actions in their Data field
 	for _, pd := range pds {
-		bundled := source.BundledQuickFixes(pd)
+		bundled := cache.BundledQuickFixes(pd)
 		if len(bundled) > 0 {
 			for _, fix := range bundled {
 				if want[fix.Kind] {
@@ -701,7 +707,7 @@ func goTest(ctx context.Context, snapshot source.Snapshot, pkg source.Package, p
 		return nil, nil
 	}
 
-	cmd, err := command.NewTestCommand("Run tests and benchmarks", protocol.URIFromSpanURI(pgf.URI), tests, benchmarks)
+	cmd, err := command.NewTestCommand("Run tests and benchmarks", pgf.URI, tests, benchmarks)
 	if err != nil {
 		return nil, err
 	}
