@@ -22,13 +22,14 @@ import (
 	"golang.org/x/mod/modfile"
 	"golang.org/x/tools/go/ast/astutil"
 	"golang.org/x/tools/gopls/internal/bug"
+	"golang.org/x/tools/gopls/internal/file"
 	"golang.org/x/tools/gopls/internal/lsp/cache"
 	"golang.org/x/tools/gopls/internal/lsp/command"
 	"golang.org/x/tools/gopls/internal/lsp/debug"
 	"golang.org/x/tools/gopls/internal/lsp/progress"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/source"
-	"golang.org/x/tools/gopls/internal/span"
+	"golang.org/x/tools/gopls/internal/settings"
 	"golang.org/x/tools/gopls/internal/telemetry"
 	"golang.org/x/tools/gopls/internal/vulncheck"
 	"golang.org/x/tools/gopls/internal/vulncheck/scan"
@@ -38,7 +39,7 @@ import (
 	"golang.org/x/tools/internal/xcontext"
 )
 
-func (s *Server) executeCommand(ctx context.Context, params *protocol.ExecuteCommandParams) (interface{}, error) {
+func (s *server) ExecuteCommand(ctx context.Context, params *protocol.ExecuteCommandParams) (interface{}, error) {
 	ctx, done := event.Start(ctx, "lsp.Server.executeCommand")
 	defer done()
 
@@ -61,7 +62,7 @@ func (s *Server) executeCommand(ctx context.Context, params *protocol.ExecuteCom
 }
 
 type commandHandler struct {
-	s      *Server
+	s      *server
 	params *protocol.ExecuteCommandParams
 }
 
@@ -92,8 +93,8 @@ type commandConfig struct {
 // be populated, depending on which configuration is set. See comments in-line
 // for details.
 type commandDeps struct {
-	snapshot source.Snapshot    // present if cfg.forURI was set
-	fh       source.FileHandle  // present if cfg.forURI was set
+	snapshot *cache.Snapshot    // present if cfg.forURI was set
+	fh       file.Handle        // present if cfg.forURI was set
 	work     *progress.WorkDone // present cfg.progress was set
 }
 
@@ -111,7 +112,7 @@ func (c *commandHandler) run(ctx context.Context, cfg commandConfig, run command
 		var unsaved []string
 		for _, overlay := range c.s.session.Overlays() {
 			if !overlay.SameContentsOnDisk() {
-				unsaved = append(unsaved, overlay.URI().Filename())
+				unsaved = append(unsaved, overlay.URI().Path())
 			}
 		}
 		if len(unsaved) > 0 {
@@ -125,7 +126,7 @@ func (c *commandHandler) run(ctx context.Context, cfg commandConfig, run command
 	if cfg.forURI != "" {
 		var ok bool
 		var release func()
-		deps.snapshot, deps.fh, ok, release, err = c.s.beginFileRequest(ctx, cfg.forURI, source.UnknownKind)
+		deps.snapshot, deps.fh, ok, release, err = c.s.beginFileRequest(ctx, cfg.forURI, file.UnknownKind)
 		defer release()
 		if !ok {
 			if err != nil {
@@ -186,7 +187,7 @@ func (c *commandHandler) ApplyFix(ctx context.Context, args command.ApplyFixArgs
 		// Note: no progress here. Applying fixes should be quick.
 		forURI: args.URI,
 	}, func(ctx context.Context, deps commandDeps) error {
-		edits, err := source.ApplyFix(ctx, args.Fix, deps.snapshot, deps.fh, args.Range)
+		edits, err := source.ApplyFix(ctx, settings.Fix(args.Fix), deps.snapshot, deps.fh, args.Range)
 		if err != nil {
 			return err
 		}
@@ -216,52 +217,61 @@ func (c *commandHandler) RegenerateCgo(ctx context.Context, args command.URIArg)
 	return c.run(ctx, commandConfig{
 		progress: "Regenerating Cgo",
 	}, func(ctx context.Context, _ commandDeps) error {
-		var wg sync.WaitGroup // tracks work done on behalf of this function, incl. diagnostics
-		wg.Add(1)
-		defer wg.Done()
-
-		// Track progress on this operation for testing.
-		if c.s.Options().VerboseWorkDoneProgress {
-			work := c.s.progress.Start(ctx, DiagnosticWorkTitle(FromRegenerateCgo), "Calculating file diagnostics...", nil, nil)
-			go func() {
-				wg.Wait()
-				work.End(ctx, "Done.")
-			}()
-		}
-
-		// Resetting the view causes cgo to be regenerated via `go list`.
-		v, err := c.s.session.ResetView(ctx, args.URI.SpanURI())
-		if err != nil {
-			return err
-		}
-
-		snapshot, release, err := v.Snapshot()
-		if err != nil {
-			return err
-		}
-		wg.Add(1)
-		go func() {
-			c.s.diagnoseSnapshot(snapshot, nil, true, 0)
-			release()
-			wg.Done()
-		}()
-		return nil
+		return c.modifyState(ctx, FromRegenerateCgo, func() (*cache.Snapshot, func(), error) {
+			// Resetting the view causes cgo to be regenerated via `go list`.
+			v, err := c.s.session.ResetView(ctx, args.URI)
+			if err != nil {
+				return nil, nil, err
+			}
+			return v.Snapshot()
+		})
 	})
 }
 
+// modifyState performs an operation that modifies the snapshot state.
+//
+// It causes a snapshot diagnosis for the provided ModificationSource.
+func (c *commandHandler) modifyState(ctx context.Context, source ModificationSource, work func() (*cache.Snapshot, func(), error)) error {
+	var wg sync.WaitGroup // tracks work done on behalf of this function, incl. diagnostics
+	wg.Add(1)
+	defer wg.Done()
+
+	// Track progress on this operation for testing.
+	if c.s.Options().VerboseWorkDoneProgress {
+		work := c.s.progress.Start(ctx, DiagnosticWorkTitle(source), "Calculating file diagnostics...", nil, nil)
+		go func() {
+			wg.Wait()
+			work.End(ctx, "Done.")
+		}()
+	}
+	snapshot, release, err := work()
+	if err != nil {
+		return err
+	}
+	wg.Add(1)
+	go func() {
+		c.s.diagnoseSnapshot(snapshot, nil, true, 0)
+		release()
+		wg.Done()
+	}()
+	return nil
+}
+
 func (c *commandHandler) CheckUpgrades(ctx context.Context, args command.CheckUpgradesArgs) error {
 	return c.run(ctx, commandConfig{
 		forURI:   args.URI,
 		progress: "Checking for upgrades",
 	}, func(ctx context.Context, deps commandDeps) error {
-		upgrades, err := c.s.getUpgrades(ctx, deps.snapshot, args.URI.SpanURI(), args.Modules)
-		if err != nil {
-			return err
-		}
-		deps.snapshot.View().RegisterModuleUpgrades(args.URI.SpanURI(), upgrades)
-		// Re-diagnose the snapshot to publish the new module diagnostics.
-		c.s.diagnoseSnapshot(deps.snapshot, nil, false, 0)
-		return nil
+		return c.modifyState(ctx, FromCheckUpgrades, func() (*cache.Snapshot, func(), error) {
+			upgrades, err := c.s.getUpgrades(ctx, deps.snapshot, args.URI, args.Modules)
+			if err != nil {
+				return nil, nil, err
+			}
+			snapshot, release := deps.snapshot.View().Invalidate(ctx, cache.StateChange{
+				ModuleUpgrades: map[protocol.DocumentURI]map[string]string{args.URI: upgrades},
+			})
+			return snapshot, release, nil
+		})
 	})
 }
 
@@ -277,22 +287,17 @@ func (c *commandHandler) ResetGoModDiagnostics(ctx context.Context, args command
 	return c.run(ctx, commandConfig{
 		forURI: args.URI,
 	}, func(ctx context.Context, deps commandDeps) error {
-		// Clear all diagnostics coming from the upgrade check source and vulncheck.
-		// This will clear the diagnostics in all go.mod files, but they
-		// will be re-calculated when the snapshot is diagnosed again.
-		if args.DiagnosticSource == "" || args.DiagnosticSource == string(source.UpgradeNotification) {
-			deps.snapshot.View().ClearModuleUpgrades(args.URI.SpanURI())
-			c.s.clearDiagnosticSource(modCheckUpgradesSource)
-		}
-
-		if args.DiagnosticSource == "" || args.DiagnosticSource == string(source.Govulncheck) {
-			deps.snapshot.View().SetVulnerabilities(args.URI.SpanURI(), nil)
-			c.s.clearDiagnosticSource(modVulncheckSource)
-		}
-
-		// Re-diagnose the snapshot to remove the diagnostics.
-		c.s.diagnoseSnapshot(deps.snapshot, nil, false, 0)
-		return nil
+		return c.modifyState(ctx, FromResetGoModDiagnostics, func() (*cache.Snapshot, func(), error) {
+			snapshot, release := deps.snapshot.View().Invalidate(ctx, cache.StateChange{
+				ModuleUpgrades: map[protocol.DocumentURI]map[string]string{
+					deps.fh.URI(): nil,
+				},
+				Vulns: map[protocol.DocumentURI]*vulncheck.Result{
+					deps.fh.URI(): nil,
+				},
+			})
+			return snapshot, release, nil
+		})
 	})
 }
 
@@ -301,7 +306,7 @@ func (c *commandHandler) GoGetModule(ctx context.Context, args command.Dependenc
 		progress: "Running go get",
 		forURI:   args.URI,
 	}, func(ctx context.Context, deps commandDeps) error {
-		return c.s.runGoModUpdateCommands(ctx, deps.snapshot, args.URI.SpanURI(), func(invoke func(...string) (*bytes.Buffer, error)) error {
+		return c.s.runGoModUpdateCommands(ctx, deps.snapshot, args.URI, func(invoke func(...string) (*bytes.Buffer, error)) error {
 			return runGoGetModule(invoke, args.AddRequire, args.GoCmdArgs)
 		})
 	})
@@ -313,7 +318,7 @@ func (c *commandHandler) UpdateGoSum(ctx context.Context, args command.URIArgs)
 		progress: "Updating go.sum",
 	}, func(ctx context.Context, deps commandDeps) error {
 		for _, uri := range args.URIs {
-			snapshot, fh, ok, release, err := c.s.beginFileRequest(ctx, uri, source.UnknownKind)
+			snapshot, fh, ok, release, err := c.s.beginFileRequest(ctx, uri, file.UnknownKind)
 			defer release()
 			if !ok {
 				return err
@@ -335,7 +340,7 @@ func (c *commandHandler) Tidy(ctx context.Context, args command.URIArgs) error {
 		progress:    "Running go mod tidy",
 	}, func(ctx context.Context, deps commandDeps) error {
 		for _, uri := range args.URIs {
-			snapshot, fh, ok, release, err := c.s.beginFileRequest(ctx, uri, source.UnknownKind)
+			snapshot, fh, ok, release, err := c.s.beginFileRequest(ctx, uri, file.UnknownKind)
 			defer release()
 			if !ok {
 				return err
@@ -368,7 +373,7 @@ func (c *commandHandler) Vendor(ctx context.Context, args command.URIArg) error
 		err := deps.snapshot.RunGoCommandPiped(ctx, source.Normal|source.AllowNetwork, &gocommand.Invocation{
 			Verb:       "mod",
 			Args:       []string{"vendor"},
-			WorkingDir: filepath.Dir(args.URI.SpanURI().Filename()),
+			WorkingDir: filepath.Dir(args.URI.Path()),
 		}, &bytes.Buffer{}, &bytes.Buffer{})
 		return err
 	})
@@ -379,7 +384,7 @@ func (c *commandHandler) EditGoDirective(ctx context.Context, args command.EditG
 		requireSave: true, // if go.mod isn't saved it could cause a problem
 		forURI:      args.URI,
 	}, func(ctx context.Context, deps commandDeps) error {
-		snapshot, fh, ok, release, err := c.s.beginFileRequest(ctx, args.URI, source.UnknownKind)
+		snapshot, fh, ok, release, err := c.s.beginFileRequest(ctx, args.URI, file.UnknownKind)
 		defer release()
 		if !ok {
 			return err
@@ -404,7 +409,7 @@ func (c *commandHandler) RemoveDependency(ctx context.Context, args command.Remo
 		// TODO(rfindley): In Go 1.17+, we will be able to use the go command
 		// without checking if the module is tidy.
 		if args.OnlyDiagnostic {
-			return c.s.runGoModUpdateCommands(ctx, deps.snapshot, args.URI.SpanURI(), func(invoke func(...string) (*bytes.Buffer, error)) error {
+			return c.s.runGoModUpdateCommands(ctx, deps.snapshot, args.URI, func(invoke func(...string) (*bytes.Buffer, error)) error {
 				if err := runGoGetModule(invoke, false, []string{args.ModulePath + "@none"}); err != nil {
 					return err
 				}
@@ -428,7 +433,7 @@ func (c *commandHandler) RemoveDependency(ctx context.Context, args command.Remo
 							TextDocument: protocol.OptionalVersionedTextDocumentIdentifier{
 								Version: deps.fh.Version(),
 								TextDocumentIdentifier: protocol.TextDocumentIdentifier{
-									URI: protocol.URIFromSpanURI(deps.fh.URI()),
+									URI: deps.fh.URI(),
 								},
 							},
 							Edits: nonNilSliceTextEdit(edits),
@@ -466,7 +471,7 @@ func dropDependency(snapshot source.Snapshot, pm *source.ParsedModule, modulePat
 	}
 	// Calculate the edits to be made due to the change.
 	diff := snapshot.Options().ComputeEdits(string(pm.Mapper.Content), string(newContent))
-	return source.ToProtocolEdits(pm.Mapper, diff)
+	return protocol.EditsFromDiffEdits(pm.Mapper, diff)
 }
 
 func (c *commandHandler) Test(ctx context.Context, uri protocol.DocumentURI, tests, benchmarks []string) error {
@@ -484,16 +489,13 @@ func (c *commandHandler) RunTests(ctx context.Context, args command.RunTestsArgs
 		requireSave: true,
 		forURI:      args.URI,
 	}, func(ctx context.Context, deps commandDeps) error {
-		if err := c.runTests(ctx, deps.snapshot, deps.work, args.URI, args.Tests, args.Benchmarks); err != nil {
-			return fmt.Errorf("running tests failed: %w", err)
-		}
-		return nil
+		return c.runTests(ctx, deps.snapshot, deps.work, args.URI, args.Tests, args.Benchmarks)
 	})
 }
 
-func (c *commandHandler) runTests(ctx context.Context, snapshot source.Snapshot, work *progress.WorkDone, uri protocol.DocumentURI, tests, benchmarks []string) error {
+func (c *commandHandler) runTests(ctx context.Context, snapshot *cache.Snapshot, work *progress.WorkDone, uri protocol.DocumentURI, tests, benchmarks []string) error {
 	// TODO: fix the error reporting when this runs async.
-	meta, err := source.NarrowestMetadataForFile(ctx, snapshot, uri.SpanURI())
+	meta, err := source.NarrowestMetadataForFile(ctx, snapshot, uri)
 	if err != nil {
 		return err
 	}
@@ -510,7 +512,7 @@ func (c *commandHandler) runTests(ctx context.Context, snapshot source.Snapshot,
 		inv := &gocommand.Invocation{
 			Verb:       "test",
 			Args:       []string{pkgPath, "-v", "-count=1", "-run", fmt.Sprintf("^%s$", funcName)},
-			WorkingDir: filepath.Dir(uri.SpanURI().Filename()),
+			WorkingDir: filepath.Dir(uri.Path()),
 		}
 		if err := snapshot.RunGoCommandPiped(ctx, source.Normal, inv, out, out); err != nil {
 			if errors.Is(err, context.Canceled) {
@@ -526,7 +528,7 @@ func (c *commandHandler) runTests(ctx context.Context, snapshot source.Snapshot,
 		inv := &gocommand.Invocation{
 			Verb:       "test",
 			Args:       []string{pkgPath, "-v", "-run=^$", "-bench", fmt.Sprintf("^%s$", funcName)},
-			WorkingDir: filepath.Dir(uri.SpanURI().Filename()),
+			WorkingDir: filepath.Dir(uri.Path()),
 		}
 		if err := snapshot.RunGoCommandPiped(ctx, source.Normal, inv, out, out); err != nil {
 			if errors.Is(err, context.Canceled) {
@@ -558,10 +560,15 @@ func (c *commandHandler) runTests(ctx context.Context, snapshot source.Snapshot,
 		message += "\n" + buf.String()
 	}
 
-	return c.s.client.ShowMessage(ctx, &protocol.ShowMessageParams{
+	_ = c.s.client.ShowMessage(ctx, &protocol.ShowMessageParams{
 		Type:    protocol.Info,
 		Message: message,
 	})
+
+	if failedTests > 0 || failedBenchmarks > 0 {
+		return errors.New("gopls.test command failed")
+	}
+	return nil
 }
 
 func (c *commandHandler) Generate(ctx context.Context, args command.GenerateArgs) error {
@@ -583,7 +590,7 @@ func (c *commandHandler) Generate(ctx context.Context, args command.GenerateArgs
 		inv := &gocommand.Invocation{
 			Verb:       "generate",
 			Args:       []string{"-x", pattern},
-			WorkingDir: args.Dir.SpanURI().Filename(),
+			WorkingDir: args.Dir.Path(),
 		}
 		stderr := io.MultiWriter(er, progress.NewWorkDoneWriter(ctx, deps.work))
 		if err := deps.snapshot.RunGoCommandPiped(ctx, source.Normal, inv, er, stderr); err != nil {
@@ -602,13 +609,13 @@ func (c *commandHandler) GoGetPackage(ctx context.Context, args command.GoGetPac
 		stdout, err := deps.snapshot.RunGoCommandDirect(ctx, source.WriteTemporaryModFile|source.AllowNetwork, &gocommand.Invocation{
 			Verb:       "list",
 			Args:       []string{"-f", "{{.Module.Path}}@{{.Module.Version}}", args.Pkg},
-			WorkingDir: filepath.Dir(args.URI.SpanURI().Filename()),
+			WorkingDir: filepath.Dir(args.URI.Path()),
 		})
 		if err != nil {
 			return err
 		}
 		ver := strings.TrimSpace(stdout.String())
-		return c.s.runGoModUpdateCommands(ctx, deps.snapshot, args.URI.SpanURI(), func(invoke func(...string) (*bytes.Buffer, error)) error {
+		return c.s.runGoModUpdateCommands(ctx, deps.snapshot, args.URI, func(invoke func(...string) (*bytes.Buffer, error)) error {
 			if args.AddRequire {
 				if err := addModuleRequire(invoke, []string{ver}); err != nil {
 					return err
@@ -620,16 +627,13 @@ func (c *commandHandler) GoGetPackage(ctx context.Context, args command.GoGetPac
 	})
 }
 
-func (s *Server) runGoModUpdateCommands(ctx context.Context, snapshot source.Snapshot, uri span.URI, run func(invoke func(...string) (*bytes.Buffer, error)) error) error {
-	tmpModfile, newModBytes, newSumBytes, err := snapshot.RunGoCommands(ctx, true, filepath.Dir(uri.Filename()), run)
+func (s *server) runGoModUpdateCommands(ctx context.Context, snapshot *cache.Snapshot, uri protocol.DocumentURI, run func(invoke func(...string) (*bytes.Buffer, error)) error) error {
+	newModBytes, newSumBytes, err := snapshot.RunGoModUpdateCommands(ctx, filepath.Dir(uri.Path()), run)
 	if err != nil {
 		return err
 	}
-	if !tmpModfile {
-		return nil
-	}
 	modURI := snapshot.GoModForFile(uri)
-	sumURI := span.URIFromPath(strings.TrimSuffix(modURI.Filename(), ".mod") + ".sum")
+	sumURI := protocol.URIFromPath(strings.TrimSuffix(modURI.Path(), ".mod") + ".sum")
 	modEdits, err := collectFileEdits(ctx, snapshot, modURI, newModBytes)
 	if err != nil {
 		return err
@@ -649,7 +653,7 @@ func (s *Server) runGoModUpdateCommands(ctx context.Context, snapshot source.Sna
 //
 // TODO(rfindley): fix this API asymmetry. It should be up to the caller to
 // write the file or apply the edits.
-func collectFileEdits(ctx context.Context, snapshot source.Snapshot, uri span.URI, newContent []byte) ([]protocol.TextDocumentEdit, error) {
+func collectFileEdits(ctx context.Context, snapshot *cache.Snapshot, uri protocol.DocumentURI, newContent []byte) ([]protocol.TextDocumentEdit, error) {
 	fh, err := snapshot.ReadFile(ctx, uri)
 	if err != nil {
 		return nil, err
@@ -667,13 +671,13 @@ func collectFileEdits(ctx context.Context, snapshot source.Snapshot, uri span.UR
 	// file and leave it unsaved. We would rather apply the changes directly,
 	// especially to go.sum, which should be mostly invisible to the user.
 	if !snapshot.IsOpen(uri) {
-		err := os.WriteFile(uri.Filename(), newContent, 0666)
+		err := os.WriteFile(uri.Path(), newContent, 0666)
 		return nil, err
 	}
 
 	m := protocol.NewMapper(fh.URI(), oldContent)
 	diff := snapshot.Options().ComputeEdits(string(oldContent), string(newContent))
-	edits, err := source.ToProtocolEdits(m, diff)
+	edits, err := protocol.EditsFromDiffEdits(m, diff)
 	if err != nil {
 		return nil, err
 	}
@@ -681,7 +685,7 @@ func collectFileEdits(ctx context.Context, snapshot source.Snapshot, uri span.UR
 		TextDocument: protocol.OptionalVersionedTextDocumentIdentifier{
 			Version: fh.Version(),
 			TextDocumentIdentifier: protocol.TextDocumentIdentifier{
-				URI: protocol.URIFromSpanURI(uri),
+				URI: uri,
 			},
 		},
 		Edits: edits,
@@ -732,12 +736,12 @@ func addModuleRequire(invoke func(...string) (*bytes.Buffer, error), args []stri
 	return err
 }
 
-func (s *Server) getUpgrades(ctx context.Context, snapshot source.Snapshot, uri span.URI, modules []string) (map[string]string, error) {
+// TODO(rfindley): inline.
+func (s *server) getUpgrades(ctx context.Context, snapshot source.Snapshot, uri protocol.DocumentURI, modules []string) (map[string]string, error) {
 	stdout, err := snapshot.RunGoCommandDirect(ctx, source.Normal|source.AllowNetwork, &gocommand.Invocation{
 		Verb:       "list",
 		Args:       append([]string{"-m", "-u", "-json"}, modules...),
-		WorkingDir: filepath.Dir(uri.Filename()),
-		ModFlag:    "readonly",
+		WorkingDir: filepath.Dir(uri.Path()),
 	})
 	if err != nil {
 		return nil, err
@@ -804,7 +808,7 @@ func (c *commandHandler) ListImports(ctx context.Context, args command.URIArg) (
 	err := c.run(ctx, commandConfig{
 		forURI: args.URI,
 	}, func(ctx context.Context, deps commandDeps) error {
-		fh, err := deps.snapshot.ReadFile(ctx, args.URI.SpanURI())
+		fh, err := deps.snapshot.ReadFile(ctx, args.URI)
 		if err != nil {
 			return err
 		}
@@ -828,7 +832,7 @@ func (c *commandHandler) ListImports(ctx context.Context, args command.URIArg) (
 				})
 			}
 		}
-		meta, err := source.NarrowestMetadataForFile(ctx, deps.snapshot, args.URI.SpanURI())
+		meta, err := source.NarrowestMetadataForFile(ctx, deps.snapshot, args.URI)
 		if err != nil {
 			return err // e.g. cancelled
 		}
@@ -924,7 +928,7 @@ func (c *commandHandler) StopProfile(ctx context.Context, args command.StopProfi
 	return result, nil
 }
 
-// Copy of pkgLoadConfig defined in internal/lsp/cmd/vulncheck.go
+// Copy of pkgLoadConfig defined in internal/cmd/vulncheck.go
 // TODO(hyangah): decide where to define this.
 type pkgLoadConfig struct {
 	// BuildFlags is a list of command-line flags to be passed through to
@@ -938,18 +942,18 @@ type pkgLoadConfig struct {
 func (c *commandHandler) FetchVulncheckResult(ctx context.Context, arg command.URIArg) (map[protocol.DocumentURI]*vulncheck.Result, error) {
 	ret := map[protocol.DocumentURI]*vulncheck.Result{}
 	err := c.run(ctx, commandConfig{forURI: arg.URI}, func(ctx context.Context, deps commandDeps) error {
-		if deps.snapshot.Options().Vulncheck == source.ModeVulncheckImports {
+		if deps.snapshot.Options().Vulncheck == settings.ModeVulncheckImports {
 			for _, modfile := range deps.snapshot.ModFiles() {
 				res, err := deps.snapshot.ModVuln(ctx, modfile)
 				if err != nil {
 					return err
 				}
-				ret[protocol.URIFromSpanURI(modfile)] = res
+				ret[modfile] = res
 			}
 		}
 		// Overwrite if there is any govulncheck-based result.
-		for modfile, result := range deps.snapshot.View().Vulnerabilities() {
-			ret[protocol.URIFromSpanURI(modfile)] = result
+		for modfile, result := range deps.snapshot.Vulnerabilities() {
+			ret[modfile] = result
 		}
 		return nil
 	})
@@ -976,7 +980,7 @@ func (c *commandHandler) RunGovulncheck(ctx context.Context, args command.Vulnch
 		tokenChan <- deps.work.Token()
 
 		workDoneWriter := progress.NewWorkDoneWriter(ctx, deps.work)
-		dir := filepath.Dir(args.URI.SpanURI().Filename())
+		dir := filepath.Dir(args.URI.Path())
 		pattern := args.Pattern
 
 		result, err := scan.RunGovulncheck(ctx, pattern, deps.snapshot, dir, workDoneWriter)
@@ -984,8 +988,11 @@ func (c *commandHandler) RunGovulncheck(ctx context.Context, args command.Vulnch
 			return err
 		}
 
-		deps.snapshot.View().SetVulnerabilities(args.URI.SpanURI(), result)
-		c.s.diagnoseSnapshot(deps.snapshot, nil, false, 0)
+		snapshot, release := deps.snapshot.View().Invalidate(ctx, cache.StateChange{
+			Vulns: map[protocol.DocumentURI]*vulncheck.Result{args.URI: result},
+		})
+		defer release()
+		c.s.diagnoseSnapshot(snapshot, nil, false, 0)
 
 		affecting := make(map[string]bool, len(result.Entries))
 		for _, finding := range result.Findings {
@@ -1145,8 +1152,8 @@ func (c *commandHandler) RunGoWorkCommand(ctx context.Context, args command.RunG
 		forView:  args.ViewID,
 	}, func(ctx context.Context, deps commandDeps) (runErr error) {
 		snapshot := deps.snapshot
-		view := snapshot.View().(*cache.View)
-		viewDir := view.Folder().Filename()
+		view := snapshot.View()
+		viewDir := snapshot.Folder().Path()
 
 		// If the user has explicitly set GOWORK=off, we should warn them
 		// explicitly and avoid potentially misleading errors below.
@@ -1154,7 +1161,7 @@ func (c *commandHandler) RunGoWorkCommand(ctx context.Context, args command.RunG
 		if off {
 			return fmt.Errorf("cannot modify go.work files when GOWORK=off")
 		}
-		gowork := goworkURI.Filename()
+		gowork := goworkURI.Path()
 
 		if goworkURI != "" {
 			fh, err := snapshot.ReadFile(ctx, goworkURI)
