@@ -20,8 +20,7 @@ import (
 	"golang.org/x/tools/gopls/internal/lsp/command"
 	"golang.org/x/tools/gopls/internal/lsp/glob"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
-	"golang.org/x/tools/gopls/internal/lsp/source"
-	"golang.org/x/tools/gopls/internal/span"
+	"golang.org/x/tools/gopls/internal/pathutil"
 	"golang.org/x/tools/internal/jsonrpc2"
 	"golang.org/x/tools/internal/jsonrpc2/servertest"
 	"golang.org/x/tools/internal/xcontext"
@@ -39,12 +38,13 @@ type Editor struct {
 	client     *Client
 	sandbox    *Sandbox
 
-	// TODO(adonovan): buffers should be keyed by protocol.DocumentURI.
+	// TODO(rfindley): buffers should be keyed by protocol.DocumentURI.
 	mu                 sync.Mutex
 	config             EditorConfig                // editor configuration
 	buffers            map[string]buffer           // open buffers (relative path -> buffer content)
 	serverCapabilities protocol.ServerCapabilities // capabilities / options
-	watchPatterns      []*glob.Glob                // glob patterns to watch
+	semTokOpts         protocol.SemanticTokensOptions
+	watchPatterns      []*glob.Glob // glob patterns to watch
 
 	// Call metrics for the purpose of expectations. This is done in an ad-hoc
 	// manner for now. Perhaps in the future we should do something more
@@ -224,7 +224,7 @@ func makeSettings(sandbox *Sandbox, config EditorConfig) map[string]interface{}
 		env[k] = v
 	}
 	for k, v := range env {
-		v = strings.ReplaceAll(v, "$SANDBOX_WORKDIR", sandbox.Workdir.RootURI().SpanURI().Filename())
+		v = strings.ReplaceAll(v, "$SANDBOX_WORKDIR", sandbox.Workdir.RootURI().Path())
 		env[k] = v
 	}
 
@@ -306,8 +306,13 @@ func (e *Editor) initialize(ctx context.Context) error {
 		if err != nil {
 			return fmt.Errorf("initialize: %w", err)
 		}
+		semTokOpts, err := marshalUnmarshal[protocol.SemanticTokensOptions](resp.Capabilities.SemanticTokensProvider)
+		if err != nil {
+			return fmt.Errorf("unmarshalling semantic tokens options: %v", err)
+		}
 		e.mu.Lock()
 		e.serverCapabilities = resp.Capabilities
+		e.semTokOpts = semTokOpts
 		e.mu.Unlock()
 
 		if err := e.Server.Initialized(ctx, &protocol.InitializedParams{}); err != nil {
@@ -318,6 +323,19 @@ func (e *Editor) initialize(ctx context.Context) error {
 	return nil
 }
 
+// marshalUnmarshal is a helper to json Marshal and then Unmarshal as a
+// different type. Used to work around cases where our protocol types are not
+// specific.
+func marshalUnmarshal[T any](v any) (T, error) {
+	var t T
+	data, err := json.Marshal(v)
+	if err != nil {
+		return t, err
+	}
+	err = json.Unmarshal(data, &t)
+	return t, err
+}
+
 // HasCommand reports whether the connected server supports the command with the given ID.
 func (e *Editor) HasCommand(id string) bool {
 	for _, command := range e.serverCapabilities.ExecuteCommandProvider.Commands {
@@ -388,7 +406,7 @@ func (e *Editor) onFileChanges(ctx context.Context, evts []protocol.FileEvent) {
 		}
 		var matchedEvts []protocol.FileEvent
 		for _, evt := range evts {
-			filename := filepath.ToSlash(evt.URI.SpanURI().Filename())
+			filename := filepath.ToSlash(evt.URI.Path())
 			for _, g := range e.watchPatterns {
 				if g.Match(filename) {
 					matchedEvts = append(matchedEvts, evt)
@@ -459,7 +477,7 @@ func (e *Editor) createBuffer(ctx context.Context, path string, dirty bool, cont
 		return fmt.Errorf("buffer %q already exists", path)
 	}
 
-	uri := e.sandbox.Workdir.URI(path).SpanURI()
+	uri := e.sandbox.Workdir.URI(path)
 	buf := buffer{
 		version: 1,
 		path:    path,
@@ -1035,7 +1053,7 @@ func (e *Editor) RunGenerate(ctx context.Context, dir string) error {
 	}
 	absDir := e.sandbox.Workdir.AbsPath(dir)
 	cmd, err := command.NewGenerateCommand("", command.GenerateArgs{
-		Dir:       protocol.URIFromSpanURI(span.URIFromPath(absDir)),
+		Dir:       protocol.URIFromPath(absDir),
 		Recursive: false,
 	})
 	if err != nil {
@@ -1284,7 +1302,7 @@ func (e *Editor) renameBuffers(ctx context.Context, oldPath, newPath string) (cl
 
 	for path := range e.buffers {
 		abs := e.sandbox.Workdir.AbsPath(path)
-		if oldAbs == abs || source.InDir(oldAbs, abs) {
+		if oldAbs == abs || pathutil.InDir(oldAbs, abs) {
 			rel, err := filepath.Rel(oldAbs, abs)
 			if err != nil {
 				return nil, nil, fmt.Errorf("filepath.Rel(%q, %q): %v", oldAbs, abs, err)
@@ -1492,3 +1510,81 @@ func (e *Editor) DocumentHighlight(ctx context.Context, loc protocol.Location) (
 
 	return e.Server.DocumentHighlight(ctx, params)
 }
+
+// SemanticTokensFull invokes textDocument/semanticTokens/full, and interprets
+// its result.
+func (e *Editor) SemanticTokensFull(ctx context.Context, path string) ([]SemanticToken, error) {
+	p := &protocol.SemanticTokensParams{
+		TextDocument: protocol.TextDocumentIdentifier{
+			URI: e.sandbox.Workdir.URI(path),
+		},
+	}
+	resp, err := e.Server.SemanticTokensFull(ctx, p)
+	if err != nil {
+		return nil, err
+	}
+	content, ok := e.BufferText(path)
+	if !ok {
+		return nil, fmt.Errorf("buffer %s is not open", path)
+	}
+	return e.interpretTokens(resp.Data, content), nil
+}
+
+// SemanticTokensRange invokes textDocument/semanticTokens/range, and
+// interprets its result.
+func (e *Editor) SemanticTokensRange(ctx context.Context, loc protocol.Location) ([]SemanticToken, error) {
+	p := &protocol.SemanticTokensRangeParams{
+		TextDocument: protocol.TextDocumentIdentifier{URI: loc.URI},
+		Range:        loc.Range,
+	}
+	resp, err := e.Server.SemanticTokensRange(ctx, p)
+	if err != nil {
+		return nil, err
+	}
+	path := e.sandbox.Workdir.URIToPath(loc.URI)
+	// As noted above: buffers should be keyed by protocol.DocumentURI.
+	content, ok := e.BufferText(path)
+	if !ok {
+		return nil, fmt.Errorf("buffer %s is not open", path)
+	}
+	return e.interpretTokens(resp.Data, content), nil
+}
+
+// A SemanticToken is an interpreted semantic token value.
+type SemanticToken struct {
+	Token     string
+	TokenType string
+	Mod       string
+}
+
+// Note: previously this function elided comment, string, and number tokens.
+// Instead, filtering of token types should be done by the caller.
+func (e *Editor) interpretTokens(x []uint32, contents string) []SemanticToken {
+	e.mu.Lock()
+	legend := e.semTokOpts.Legend
+	e.mu.Unlock()
+	lines := strings.Split(contents, "\n")
+	ans := []SemanticToken{}
+	line, col := 1, 1
+	for i := 0; i < len(x); i += 5 {
+		line += int(x[i])
+		col += int(x[i+1])
+		if x[i] != 0 { // new line
+			col = int(x[i+1]) + 1 // 1-based column numbers
+		}
+		sz := x[i+2]
+		t := legend.TokenTypes[x[i+3]]
+		l := x[i+4]
+		var mods []string
+		for i, mod := range legend.TokenModifiers {
+			if l&(1<<i) != 0 {
+				mods = append(mods, mod)
+			}
+		}
+		// Preexisting note: "col is a utf-8 offset"
+		// TODO(rfindley): is that true? Or is it UTF-16, like other columns in the LSP?
+		tok := lines[line-1][col-1 : col-1+int(sz)]
+		ans = append(ans, SemanticToken{tok, t, strings.Join(mods, " ")})
+	}
+	return ans
+}
