@@ -9,7 +9,7 @@ import (
 	"context"
 	"crypto/sha256"
 	"fmt"
-	"io/ioutil"
+	"io/fs"
 	"os"
 	"path/filepath"
 	"runtime"
@@ -19,17 +19,9 @@ import (
 	"time"
 
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
-	"golang.org/x/tools/gopls/internal/robustio"
-	"golang.org/x/tools/gopls/internal/span"
+	"golang.org/x/tools/internal/robustio"
 )
 
-// FileEvent wraps the protocol.FileEvent so that it can be associated with a
-// workdir-relative path.
-type FileEvent struct {
-	Path, Content string
-	ProtocolEvent protocol.FileEvent
-}
-
 // RelativeTo is a helper for operations relative to a given directory.
 type RelativeTo string
 
@@ -52,9 +44,10 @@ func (r RelativeTo) RelPath(fp string) string {
 	return filepath.ToSlash(fp)
 }
 
-// WriteFileData writes content to the relative path, replacing the special
-// token $SANDBOX_WORKDIR with the relative root given by rel.
-func WriteFileData(path string, content []byte, rel RelativeTo) error {
+// writeFileData writes content to the relative path, replacing the special
+// token $SANDBOX_WORKDIR with the relative root given by rel. It does not
+// trigger any file events.
+func writeFileData(path string, content []byte, rel RelativeTo) error {
 	content = bytes.ReplaceAll(content, []byte("$SANDBOX_WORKDIR"), []byte(rel))
 	fp := rel.AbsPath(path)
 	if err := os.MkdirAll(filepath.Dir(fp), 0755); err != nil {
@@ -62,7 +55,7 @@ func WriteFileData(path string, content []byte, rel RelativeTo) error {
 	}
 	backoff := 1 * time.Millisecond
 	for {
-		err := ioutil.WriteFile(fp, []byte(content), 0644)
+		err := os.WriteFile(fp, content, 0644)
 		if err != nil {
 			// This lock file violation is not handled by the robustio package, as it
 			// indicates a real race condition that could be avoided.
@@ -88,7 +81,7 @@ type Workdir struct {
 	RelativeTo
 
 	watcherMu sync.Mutex
-	watchers  []func(context.Context, []FileEvent)
+	watchers  []func(context.Context, []protocol.FileEvent)
 
 	fileMu sync.Mutex
 	// File identities we know about, for the purpose of detecting changes.
@@ -109,59 +102,37 @@ type Workdir struct {
 	files map[string]fileID
 }
 
-// fileID is a file identity for the purposes of detecting on-disk
-// modifications.
-type fileID struct {
-	hash  string
-	mtime time.Time
-}
-
 // NewWorkdir writes the txtar-encoded file data in txt to dir, and returns a
 // Workir for operating on these files using
-func NewWorkdir(dir string) *Workdir {
-	return &Workdir{RelativeTo: RelativeTo(dir)}
+func NewWorkdir(dir string, files map[string][]byte) (*Workdir, error) {
+	w := &Workdir{RelativeTo: RelativeTo(dir)}
+	for name, data := range files {
+		if err := writeFileData(name, data, w.RelativeTo); err != nil {
+			return nil, fmt.Errorf("writing to workdir: %w", err)
+		}
+	}
+	_, err := w.pollFiles() // poll files to populate the files map.
+	return w, err
+}
+
+// fileID identifies a file version on disk.
+type fileID struct {
+	mtime time.Time
+	hash  string // empty if mtime is old enough to be reliable; otherwise a file digest
 }
 
 func hashFile(data []byte) string {
 	return fmt.Sprintf("%x", sha256.Sum256(data))
 }
 
-func (w *Workdir) writeInitialFiles(files map[string][]byte) error {
-	w.files = map[string]fileID{}
-	for name, data := range files {
-		if err := WriteFileData(name, data, w.RelativeTo); err != nil {
-			return fmt.Errorf("writing to workdir: %w", err)
-		}
-		fp := w.AbsPath(name)
-
-		// We need the mtime of the file just written for the purposes of tracking
-		// file identity. Calling Stat here could theoretically return an mtime
-		// that is inconsistent with the file contents represented by the hash, but
-		// since we "own" this file we assume that the mtime is correct.
-		//
-		// Furthermore, see the documentation for Workdir.files for why mismatches
-		// between identifiers are considered to be benign.
-		fi, err := os.Stat(fp)
-		if err != nil {
-			return fmt.Errorf("reading file info: %v", err)
-		}
-
-		w.files[name] = fileID{
-			hash:  hashFile(data),
-			mtime: fi.ModTime(),
-		}
-	}
-	return nil
-}
-
 // RootURI returns the root URI for this working directory of this scratch
 // environment.
 func (w *Workdir) RootURI() protocol.DocumentURI {
-	return toURI(string(w.RelativeTo))
+	return protocol.URIFromPath(string(w.RelativeTo))
 }
 
 // AddWatcher registers the given func to be called on any file change.
-func (w *Workdir) AddWatcher(watcher func(context.Context, []FileEvent)) {
+func (w *Workdir) AddWatcher(watcher func(context.Context, []protocol.FileEvent)) {
 	w.watcherMu.Lock()
 	w.watchers = append(w.watchers, watcher)
 	w.watcherMu.Unlock()
@@ -169,25 +140,20 @@ func (w *Workdir) AddWatcher(watcher func(context.Context, []FileEvent)) {
 
 // URI returns the URI to a the workdir-relative path.
 func (w *Workdir) URI(path string) protocol.DocumentURI {
-	return toURI(w.AbsPath(path))
+	return protocol.URIFromPath(w.AbsPath(path))
 }
 
 // URIToPath converts a uri to a workdir-relative path (or an absolute path,
 // if the uri is outside of the workdir).
 func (w *Workdir) URIToPath(uri protocol.DocumentURI) string {
-	fp := uri.SpanURI().Filename()
-	return w.RelPath(fp)
-}
-
-func toURI(fp string) protocol.DocumentURI {
-	return protocol.DocumentURI(span.URIFromPath(fp))
+	return w.RelPath(uri.Path())
 }
 
 // ReadFile reads a text file specified by a workdir-relative path.
-func (w *Workdir) ReadFile(path string) (string, error) {
+func (w *Workdir) ReadFile(path string) ([]byte, error) {
 	backoff := 1 * time.Millisecond
 	for {
-		b, err := ioutil.ReadFile(w.AbsPath(path))
+		b, err := os.ReadFile(w.AbsPath(path))
 		if err != nil {
 			if runtime.GOOS == "plan9" && strings.HasSuffix(err.Error(), " exclusive use file already open") {
 				// Plan 9 enforces exclusive access to locked files.
@@ -196,149 +162,64 @@ func (w *Workdir) ReadFile(path string) (string, error) {
 				backoff *= 2
 				continue
 			}
-			return "", err
+			return nil, err
 		}
-		return string(b), nil
+		return b, nil
 	}
 }
 
-func (w *Workdir) RegexpRange(path, re string) (Pos, Pos, error) {
-	content, err := w.ReadFile(path)
-	if err != nil {
-		return Pos{}, Pos{}, err
-	}
-	return regexpRange(content, re)
-}
-
 // RegexpSearch searches the file corresponding to path for the first position
 // matching re.
-func (w *Workdir) RegexpSearch(path string, re string) (Pos, error) {
+func (w *Workdir) RegexpSearch(path string, re string) (protocol.Location, error) {
 	content, err := w.ReadFile(path)
 	if err != nil {
-		return Pos{}, err
+		return protocol.Location{}, err
 	}
-	start, _, err := regexpRange(content, re)
-	return start, err
+	mapper := protocol.NewMapper(w.URI(path), content)
+	return regexpLocation(mapper, re)
 }
 
-// ChangeFilesOnDisk executes the given on-disk file changes in a batch,
-// simulating the action of changing branches outside of an editor.
-func (w *Workdir) ChangeFilesOnDisk(ctx context.Context, events []FileEvent) error {
-	for _, e := range events {
-		switch e.ProtocolEvent.Type {
-		case protocol.Deleted:
-			fp := w.AbsPath(e.Path)
-			if err := os.Remove(fp); err != nil {
-				return fmt.Errorf("removing %q: %w", e.Path, err)
-			}
-		case protocol.Changed, protocol.Created:
-			if _, err := w.writeFile(ctx, e.Path, e.Content); err != nil {
-				return err
-			}
-		}
-	}
-	w.sendEvents(ctx, events)
-	return nil
-}
-
-// RemoveFile removes a workdir-relative file path.
+// RemoveFile removes a workdir-relative file path and notifies watchers of the
+// change.
 func (w *Workdir) RemoveFile(ctx context.Context, path string) error {
 	fp := w.AbsPath(path)
-	if err := os.RemoveAll(fp); err != nil {
+	if err := robustio.RemoveAll(fp); err != nil {
 		return fmt.Errorf("removing %q: %w", path, err)
 	}
-	w.fileMu.Lock()
-	defer w.fileMu.Unlock()
 
-	evts := []FileEvent{{
-		Path: path,
-		ProtocolEvent: protocol.FileEvent{
-			URI:  w.URI(path),
-			Type: protocol.Deleted,
-		},
-	}}
-	w.sendEvents(ctx, evts)
-	delete(w.files, path)
-	return nil
+	return w.CheckForFileChanges(ctx)
 }
 
-func (w *Workdir) sendEvents(ctx context.Context, evts []FileEvent) {
-	if len(evts) == 0 {
-		return
-	}
-	w.watcherMu.Lock()
-	watchers := make([]func(context.Context, []FileEvent), len(w.watchers))
-	copy(watchers, w.watchers)
-	w.watcherMu.Unlock()
-	for _, w := range watchers {
-		w(ctx, evts)
-	}
-}
-
-// WriteFiles writes the text file content to workdir-relative paths.
-// It batches notifications rather than sending them consecutively.
+// WriteFiles writes the text file content to workdir-relative paths and
+// notifies watchers of the changes.
 func (w *Workdir) WriteFiles(ctx context.Context, files map[string]string) error {
-	var evts []FileEvent
-	for filename, content := range files {
-		evt, err := w.writeFile(ctx, filename, content)
-		if err != nil {
+	for path, content := range files {
+		fp := w.AbsPath(path)
+		_, err := os.Stat(fp)
+		if err != nil && !os.IsNotExist(err) {
+			return fmt.Errorf("checking if %q exists: %w", path, err)
+		}
+		if err := writeFileData(path, []byte(content), w.RelativeTo); err != nil {
 			return err
 		}
-		evts = append(evts, evt)
 	}
-	w.sendEvents(ctx, evts)
-	return nil
+	return w.CheckForFileChanges(ctx)
 }
 
-// WriteFile writes text file content to a workdir-relative path.
+// WriteFile writes text file content to a workdir-relative path and notifies
+// watchers of the change.
 func (w *Workdir) WriteFile(ctx context.Context, path, content string) error {
-	evt, err := w.writeFile(ctx, path, content)
-	if err != nil {
-		return err
-	}
-	w.sendEvents(ctx, []FileEvent{evt})
-	return nil
-}
-
-func (w *Workdir) writeFile(ctx context.Context, path, content string) (FileEvent, error) {
-	fp := w.AbsPath(path)
-	_, err := os.Stat(fp)
-	if err != nil && !os.IsNotExist(err) {
-		return FileEvent{}, fmt.Errorf("checking if %q exists: %w", path, err)
-	}
-	var changeType protocol.FileChangeType
-	if os.IsNotExist(err) {
-		changeType = protocol.Created
-	} else {
-		changeType = protocol.Changed
-	}
-	if err := WriteFileData(path, []byte(content), w.RelativeTo); err != nil {
-		return FileEvent{}, err
-	}
-	return w.fileEvent(path, changeType), nil
-}
-
-func (w *Workdir) fileEvent(path string, changeType protocol.FileChangeType) FileEvent {
-	return FileEvent{
-		Path: path,
-		ProtocolEvent: protocol.FileEvent{
-			URI:  w.URI(path),
-			Type: changeType,
-		},
-	}
+	return w.WriteFiles(ctx, map[string]string{path: content})
 }
 
 // RenameFile performs an on disk-renaming of the workdir-relative oldPath to
-// workdir-relative newPath.
+// workdir-relative newPath, and notifies watchers of the changes.
 //
 // oldPath must either be a regular file or in the same directory as newPath.
 func (w *Workdir) RenameFile(ctx context.Context, oldPath, newPath string) error {
 	oldAbs := w.AbsPath(oldPath)
 	newAbs := w.AbsPath(newPath)
 
-	w.fileMu.Lock()
-	defer w.fileMu.Unlock()
-
 	// For os.Rename, “OS-specific restrictions may apply when oldpath and newpath
 	// are in different directories.” If that applies here, we may fall back to
 	// ReadFile, WriteFile, and RemoveFile to perform the rename non-atomically.
@@ -394,7 +275,7 @@ func (w *Workdir) RenameFile(ctx context.Context, oldPath, newPath string) error
 			// the error from Rename may be accurate.
 			return renameErr
 		}
-		if writeErr := WriteFileData(newPath, []byte(content), w.RelativeTo); writeErr != nil {
+		if writeErr := writeFileData(newPath, content, w.RelativeTo); writeErr != nil {
 			// At this point we have tried to actually write the file.
 			// If it still doesn't exist, assume that the error from Rename was accurate:
 			// for example, maybe we don't have permission to create the new path.
@@ -413,64 +294,27 @@ func (w *Workdir) RenameFile(ctx context.Context, oldPath, newPath string) error
 		}
 	}
 
-	// Send synthetic file events for the renaming. Renamed files are handled as
-	// Delete+Create events:
-	// https://microsoft.github.io/language-server-protocol/specifications/lsp/3.17/specification/#fileChangeType
-	events := []FileEvent{
-		w.fileEvent(oldPath, protocol.Deleted),
-		w.fileEvent(newPath, protocol.Created),
-	}
-	w.sendEvents(ctx, events)
-	delete(w.files, oldPath)
-	return nil
+	return w.CheckForFileChanges(ctx)
 }
 
 // ListFiles returns a new sorted list of the relative paths of files in dir,
 // recursively.
 func (w *Workdir) ListFiles(dir string) ([]string, error) {
-	m, err := w.listFiles(dir)
-	if err != nil {
-		return nil, err
-	}
-
-	var paths []string
-	for p := range m {
-		paths = append(paths, p)
-	}
-	sort.Strings(paths)
-	return paths, nil
-}
-
-// listFiles lists files in the given directory, returning a map of relative
-// path to contents and modification time.
-func (w *Workdir) listFiles(dir string) (map[string]fileID, error) {
-	files := make(map[string]fileID)
 	absDir := w.AbsPath(dir)
+	var paths []string
 	if err := filepath.Walk(absDir, func(fp string, info os.FileInfo, err error) error {
 		if err != nil {
 			return err
 		}
-		if info.IsDir() {
-			return nil
-		}
-		path := w.RelPath(fp)
-
-		data, err := ioutil.ReadFile(fp)
-		if err != nil {
-			return err
-		}
-		// The content returned by ioutil.ReadFile could be inconsistent with
-		// info.ModTime(), due to a subsequent modification. See the documentation
-		// for w.files for why we consider this to be benign.
-		files[path] = fileID{
-			hash:  hashFile(data),
-			mtime: info.ModTime(),
+		if info.Mode()&(fs.ModeDir|fs.ModeSymlink) == 0 {
+			paths = append(paths, w.RelPath(fp))
 		}
 		return nil
 	}); err != nil {
 		return nil, err
 	}
-	return files, nil
+	sort.Strings(paths)
+	return paths, nil
 }
 
 // CheckForFileChanges walks the working directory and checks for any files
@@ -480,52 +324,101 @@ func (w *Workdir) CheckForFileChanges(ctx context.Context) error {
 	if err != nil {
 		return err
 	}
-	w.sendEvents(ctx, evts)
+	if len(evts) == 0 {
+		return nil
+	}
+	w.watcherMu.Lock()
+	watchers := make([]func(context.Context, []protocol.FileEvent), len(w.watchers))
+	copy(watchers, w.watchers)
+	w.watcherMu.Unlock()
+	for _, w := range watchers {
+		w(ctx, evts)
+	}
 	return nil
 }
 
 // pollFiles updates w.files and calculates FileEvents corresponding to file
 // state changes since the last poll. It does not call sendEvents.
-func (w *Workdir) pollFiles() ([]FileEvent, error) {
+func (w *Workdir) pollFiles() ([]protocol.FileEvent, error) {
 	w.fileMu.Lock()
 	defer w.fileMu.Unlock()
 
-	files, err := w.listFiles(".")
-	if err != nil {
+	newFiles := make(map[string]fileID)
+	var evts []protocol.FileEvent
+	if err := filepath.Walk(string(w.RelativeTo), func(fp string, info os.FileInfo, err error) error {
+		if err != nil {
+			return err
+		}
+		// Skip directories and symbolic links (which may be links to directories).
+		//
+		// The latter matters for repos like Kubernetes, which use symlinks.
+		if info.Mode()&(fs.ModeDir|fs.ModeSymlink) != 0 {
+			return nil
+		}
+
+		// Opt: avoid reading the file if mtime is sufficiently old to be reliable.
+		//
+		// If mtime is recent, it may not sufficiently identify the file contents:
+		// a subsequent write could result in the same mtime. For these cases, we
+		// must read the file contents.
+		id := fileID{mtime: info.ModTime()}
+		if time.Since(info.ModTime()) < 2*time.Second {
+			data, err := os.ReadFile(fp)
+			if err != nil {
+				return err
+			}
+			id.hash = hashFile(data)
+		}
+		path := w.RelPath(fp)
+		newFiles[path] = id
+
+		if w.files != nil {
+			oldID, ok := w.files[path]
+			delete(w.files, path)
+			switch {
+			case !ok:
+				evts = append(evts, protocol.FileEvent{
+					URI:  w.URI(path),
+					Type: protocol.Created,
+				})
+			case oldID != id:
+				changed := true
+
+				// Check whether oldID and id do not match because oldID was polled at
+				// a recent enough to time such as to require hashing.
+				//
+				// In this case, read the content to check whether the file actually
+				// changed.
+				if oldID.mtime.Equal(id.mtime) && oldID.hash != "" && id.hash == "" {
+					data, err := os.ReadFile(fp)
+					if err != nil {
+						return err
+					}
+					if hashFile(data) == oldID.hash {
+						changed = false
+					}
+				}
+				if changed {
+					evts = append(evts, protocol.FileEvent{
+						URI:  w.URI(path),
+						Type: protocol.Changed,
+					})
+				}
+			}
+		}
+
+		return nil
+	}); err != nil {
 		return nil, err
 	}
-	var evts []FileEvent
-	// Check which files have been added or modified.
-	for path, id := range files {
-		oldID, ok := w.files[path]
-		delete(w.files, path)
-		var typ protocol.FileChangeType
-		switch {
-		case !ok:
-			typ = protocol.Created
-		case oldID != id:
-			typ = protocol.Changed
-		default:
-			continue
-		}
-		evts = append(evts, FileEvent{
-			Path: path,
-			ProtocolEvent: protocol.FileEvent{
-				URI:  w.URI(path),
-				Type: typ,
-			},
-		})
-	}
+
 	// Any remaining files must have been deleted.
 	for path := range w.files {
-		evts = append(evts, FileEvent{
-			Path: path,
-			ProtocolEvent: protocol.FileEvent{
-				URI:  w.URI(path),
-				Type: protocol.Deleted,
-			},
+		evts = append(evts, protocol.FileEvent{
+			URI:  w.URI(path),
+			Type: protocol.Deleted,
 		})
 	}
-	w.files = files
+	w.files = newFiles
 	return evts, nil
 }
