@@ -17,26 +17,27 @@ import (
 	"golang.org/x/mod/modfile"
 	"golang.org/x/mod/semver"
 	"golang.org/x/sync/errgroup"
-	"golang.org/x/tools/gopls/internal/govulncheck"
+	"golang.org/x/tools/gopls/internal/file"
+	"golang.org/x/tools/gopls/internal/lsp/cache"
 	"golang.org/x/tools/gopls/internal/lsp/command"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/source"
-	"golang.org/x/tools/gopls/internal/span"
+	"golang.org/x/tools/gopls/internal/settings"
+	"golang.org/x/tools/gopls/internal/vulncheck/govulncheck"
 	"golang.org/x/tools/internal/event"
-	"golang.org/x/vuln/osv"
 )
 
 // Diagnostics returns diagnostics from parsing the modules in the workspace.
-func Diagnostics(ctx context.Context, snapshot source.Snapshot) (map[span.URI][]*source.Diagnostic, error) {
-	ctx, done := event.Start(ctx, "mod.Diagnostics", source.SnapshotLabels(snapshot)...)
+func Diagnostics(ctx context.Context, snapshot *cache.Snapshot) (map[protocol.DocumentURI][]*source.Diagnostic, error) {
+	ctx, done := event.Start(ctx, "mod.Diagnostics", snapshot.Labels()...)
 	defer done()
 
 	return collectDiagnostics(ctx, snapshot, ModParseDiagnostics)
 }
 
 // Diagnostics returns diagnostics from running go mod tidy.
-func TidyDiagnostics(ctx context.Context, snapshot source.Snapshot) (map[span.URI][]*source.Diagnostic, error) {
-	ctx, done := event.Start(ctx, "mod.Diagnostics", source.SnapshotLabels(snapshot)...)
+func TidyDiagnostics(ctx context.Context, snapshot *cache.Snapshot) (map[protocol.DocumentURI][]*source.Diagnostic, error) {
+	ctx, done := event.Start(ctx, "mod.Diagnostics", snapshot.Labels()...)
 	defer done()
 
 	return collectDiagnostics(ctx, snapshot, ModTidyDiagnostics)
@@ -44,8 +45,8 @@ func TidyDiagnostics(ctx context.Context, snapshot source.Snapshot) (map[span.UR
 
 // UpgradeDiagnostics returns upgrade diagnostics for the modules in the
 // workspace with known upgrades.
-func UpgradeDiagnostics(ctx context.Context, snapshot source.Snapshot) (map[span.URI][]*source.Diagnostic, error) {
-	ctx, done := event.Start(ctx, "mod.UpgradeDiagnostics", source.SnapshotLabels(snapshot)...)
+func UpgradeDiagnostics(ctx context.Context, snapshot *cache.Snapshot) (map[protocol.DocumentURI][]*source.Diagnostic, error) {
+	ctx, done := event.Start(ctx, "mod.UpgradeDiagnostics", snapshot.Labels()...)
 	defer done()
 
 	return collectDiagnostics(ctx, snapshot, ModUpgradeDiagnostics)
@@ -53,21 +54,20 @@ func UpgradeDiagnostics(ctx context.Context, snapshot source.Snapshot) (map[span
 
 // VulnerabilityDiagnostics returns vulnerability diagnostics for the active modules in the
 // workspace with known vulnerabilities.
-func VulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot) (map[span.URI][]*source.Diagnostic, error) {
-	ctx, done := event.Start(ctx, "mod.VulnerabilityDiagnostics", source.SnapshotLabels(snapshot)...)
+func VulnerabilityDiagnostics(ctx context.Context, snapshot *cache.Snapshot) (map[protocol.DocumentURI][]*source.Diagnostic, error) {
+	ctx, done := event.Start(ctx, "mod.VulnerabilityDiagnostics", snapshot.Labels()...)
 	defer done()
 
 	return collectDiagnostics(ctx, snapshot, ModVulnerabilityDiagnostics)
 }
 
-func collectDiagnostics(ctx context.Context, snapshot source.Snapshot, diagFn func(context.Context, source.Snapshot, source.FileHandle) ([]*source.Diagnostic, error)) (map[span.URI][]*source.Diagnostic, error) {
-
+func collectDiagnostics(ctx context.Context, snapshot *cache.Snapshot, diagFn func(context.Context, *cache.Snapshot, file.Handle) ([]*source.Diagnostic, error)) (map[protocol.DocumentURI][]*source.Diagnostic, error) {
 	g, ctx := errgroup.WithContext(ctx)
 	cpulimit := runtime.GOMAXPROCS(0)
 	g.SetLimit(cpulimit)
 
 	var mu sync.Mutex
-	reports := make(map[span.URI][]*source.Diagnostic)
+	reports := make(map[protocol.DocumentURI][]*source.Diagnostic)
 
 	for _, uri := range snapshot.ModFiles() {
 		uri := uri
@@ -96,7 +96,7 @@ func collectDiagnostics(ctx context.Context, snapshot source.Snapshot, diagFn fu
 }
 
 // ModParseDiagnostics reports diagnostics from parsing the mod file.
-func ModParseDiagnostics(ctx context.Context, snapshot source.Snapshot, fh source.FileHandle) (diagnostics []*source.Diagnostic, err error) {
+func ModParseDiagnostics(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle) (diagnostics []*source.Diagnostic, err error) {
 	pm, err := snapshot.ParseMod(ctx, fh)
 	if err != nil {
 		if pm == nil || len(pm.ParseErrors) == 0 {
@@ -108,14 +108,19 @@ func ModParseDiagnostics(ctx context.Context, snapshot source.Snapshot, fh sourc
 }
 
 // ModTidyDiagnostics reports diagnostics from running go mod tidy.
-func ModTidyDiagnostics(ctx context.Context, snapshot source.Snapshot, fh source.FileHandle) (diagnostics []*source.Diagnostic, err error) {
+func ModTidyDiagnostics(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle) (diagnostics []*source.Diagnostic, err error) {
 	pm, err := snapshot.ParseMod(ctx, fh) // memoized
 	if err != nil {
 		return nil, nil // errors reported by ModDiagnostics above
 	}
 
 	tidied, err := snapshot.ModTidy(ctx, pm)
-	if err != nil && !source.IsNonFatalGoModError(err) {
+	if err != nil && err != cache.ErrNoModOnDisk {
+		// TODO(rfindley): the check for ErrNoModOnDisk was historically determined
+		// to be benign, but may date back to the time when the Go command did not
+		// have overlay support.
+		//
+		// See if we can pass the overlay to the Go command, and eliminate this guard..
 		event.Error(ctx, fmt.Sprintf("tidy: diagnosing %s", pm.URI), err)
 	}
 	if err == nil {
@@ -131,7 +136,7 @@ func ModTidyDiagnostics(ctx context.Context, snapshot source.Snapshot, fh source
 
 // ModUpgradeDiagnostics adds upgrade quick fixes for individual modules if the upgrades
 // are recorded in the view.
-func ModUpgradeDiagnostics(ctx context.Context, snapshot source.Snapshot, fh source.FileHandle) (upgradeDiagnostics []*source.Diagnostic, err error) {
+func ModUpgradeDiagnostics(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle) (upgradeDiagnostics []*source.Diagnostic, err error) {
 	pm, err := snapshot.ParseMod(ctx, fh)
 	if err != nil {
 		// Don't return an error if there are parse error diagnostics to be shown, but also do not
@@ -142,7 +147,7 @@ func ModUpgradeDiagnostics(ctx context.Context, snapshot source.Snapshot, fh sou
 		return nil, err
 	}
 
-	upgrades := snapshot.View().ModuleUpgrades(fh.URI())
+	upgrades := snapshot.ModuleUpgrades(fh.URI())
 	for _, req := range pm.File.Require {
 		ver, ok := upgrades[req.Mod.Path]
 		if !ok || req.Mod.Version == ver {
@@ -155,7 +160,7 @@ func ModUpgradeDiagnostics(ctx context.Context, snapshot source.Snapshot, fh sou
 		// Upgrade to the exact version we offer the user, not the most recent.
 		title := fmt.Sprintf("%s%v", upgradeCodeActionPrefix, ver)
 		cmd, err := command.NewUpgradeDependencyCommand(title, command.DependencyArgs{
-			URI:        protocol.URIFromSpanURI(fh.URI()),
+			URI:        fh.URI(),
 			AddRequire: false,
 			GoCmdArgs:  []string{req.Mod.Path + "@" + ver},
 		})
@@ -168,7 +173,7 @@ func ModUpgradeDiagnostics(ctx context.Context, snapshot source.Snapshot, fh sou
 			Severity:       protocol.SeverityInformation,
 			Source:         source.UpgradeNotification,
 			Message:        fmt.Sprintf("%v can be upgraded", req.Mod.Path),
-			SuggestedFixes: []source.SuggestedFix{source.SuggestedFixFromCommand(cmd, protocol.QuickFix)},
+			SuggestedFixes: []source.SuggestedFix{cache.SuggestedFixFromCommand(cmd, protocol.QuickFix)},
 		})
 	}
 
@@ -179,7 +184,7 @@ const upgradeCodeActionPrefix = "Upgrade to "
 
 // ModVulnerabilityDiagnostics adds diagnostics for vulnerabilities in individual modules
 // if the vulnerability is recorded in the view.
-func ModVulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot, fh source.FileHandle) (vulnDiagnostics []*source.Diagnostic, err error) {
+func ModVulnerabilityDiagnostics(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle) (vulnDiagnostics []*source.Diagnostic, err error) {
 	pm, err := snapshot.ParseMod(ctx, fh)
 	if err != nil {
 		// Don't return an error if there are parse error diagnostics to be shown, but also do not
@@ -191,15 +196,15 @@ func ModVulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot,
 	}
 
 	diagSource := source.Govulncheck
-	vs := snapshot.View().Vulnerabilities(fh.URI())[fh.URI()]
-	if vs == nil && snapshot.View().Options().Vulncheck == source.ModeVulncheckImports {
+	vs := snapshot.Vulnerabilities(fh.URI())[fh.URI()]
+	if vs == nil && snapshot.Options().Vulncheck == settings.ModeVulncheckImports {
 		vs, err = snapshot.ModVuln(ctx, fh.URI())
 		if err != nil {
 			return nil, err
 		}
 		diagSource = source.Vulncheck
 	}
-	if vs == nil || len(vs.Vulns) == 0 {
+	if vs == nil || len(vs.Findings) == 0 {
 		return nil, nil
 	}
 
@@ -208,20 +213,17 @@ func ModVulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot,
 		// must not happen
 		return nil, err // TODO: bug report
 	}
-	type modVuln struct {
-		mod  *govulncheck.Module
-		vuln *govulncheck.Vuln
-	}
-	vulnsByModule := make(map[string][]modVuln)
-	for _, vuln := range vs.Vulns {
-		for _, mod := range vuln.Modules {
-			vulnsByModule[mod.Path] = append(vulnsByModule[mod.Path], modVuln{mod, vuln})
+	vulnsByModule := make(map[string][]*govulncheck.Finding)
+
+	for _, finding := range vs.Findings {
+		if vuln, typ := foundVuln(finding); typ == vulnCalled || typ == vulnImported {
+			vulnsByModule[vuln.Module] = append(vulnsByModule[vuln.Module], finding)
 		}
 	}
-
 	for _, req := range pm.File.Require {
-		vulns := vulnsByModule[req.Mod.Path]
-		if len(vulns) == 0 {
+		mod := req.Mod.Path
+		findings := vulnsByModule[mod]
+		if len(findings) == 0 {
 			continue
 		}
 		// note: req.Syntax is the line corresponding to 'require', which means
@@ -239,10 +241,8 @@ func ModVulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot,
 		// others to 'info' level diagnostics.
 		// Fixes will include only the upgrades for warning level diagnostics.
 		var warningFixes, infoFixes []source.SuggestedFix
-		var warning, info []string
-		var relatedInfo []protocol.DiagnosticRelatedInformation
-		for _, mv := range vulns {
-			mod, vuln := mv.mod, mv.vuln
+		var warningSet, infoSet = map[string]bool{}, map[string]bool{}
+		for _, finding := range findings {
 			// It is possible that the source code was changed since the last
 			// govulncheck run and information in the `vulns` info is stale.
 			// For example, imagine that a user is in the middle of updating
@@ -259,51 +259,57 @@ func ModVulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot,
 			// version in the require statement is equal to or higher than the
 			// fixed version, skip generating a diagnostic about the vulnerability.
 			// Eventually, the user has to rerun govulncheck.
-			if mod.FixedVersion != "" && semver.IsValid(req.Mod.Version) && semver.Compare(mod.FixedVersion, req.Mod.Version) <= 0 {
+			if finding.FixedVersion != "" && semver.IsValid(req.Mod.Version) && semver.Compare(finding.FixedVersion, req.Mod.Version) <= 0 {
 				continue
 			}
-			if !vuln.IsCalled() {
-				info = append(info, vuln.OSV.ID)
-			} else {
-				warning = append(warning, vuln.OSV.ID)
-				relatedInfo = append(relatedInfo, listRelatedInfo(ctx, snapshot, vuln)...)
+			switch _, typ := foundVuln(finding); typ {
+			case vulnImported:
+				infoSet[finding.OSV] = true
+			case vulnCalled:
+				warningSet[finding.OSV] = true
 			}
 			// Upgrade to the exact version we offer the user, not the most recent.
-			if fixedVersion := mod.FixedVersion; semver.IsValid(fixedVersion) && semver.Compare(req.Mod.Version, fixedVersion) < 0 {
+			if fixedVersion := finding.FixedVersion; semver.IsValid(fixedVersion) && semver.Compare(req.Mod.Version, fixedVersion) < 0 {
 				cmd, err := getUpgradeCodeAction(fh, req, fixedVersion)
 				if err != nil {
 					return nil, err // TODO: bug report
 				}
-				sf := source.SuggestedFixFromCommand(cmd, protocol.QuickFix)
-				if !vuln.IsCalled() {
+				sf := cache.SuggestedFixFromCommand(cmd, protocol.QuickFix)
+				switch _, typ := foundVuln(finding); typ {
+				case vulnImported:
 					infoFixes = append(infoFixes, sf)
-				} else {
+				case vulnCalled:
 					warningFixes = append(warningFixes, sf)
 				}
 			}
 		}
 
-		if len(warning) == 0 && len(info) == 0 {
+		if len(warningSet) == 0 && len(infoSet) == 0 {
 			continue
 		}
+		// Remove affecting osvs from the non-affecting osv list if any.
+		if len(warningSet) > 0 {
+			for k := range infoSet {
+				if warningSet[k] {
+					delete(infoSet, k)
+				}
+			}
+		}
 		// Add an upgrade for module@latest.
 		// TODO(suzmue): verify if latest is the same as fixedVersion.
 		latest, err := getUpgradeCodeAction(fh, req, "latest")
 		if err != nil {
 			return nil, err // TODO: bug report
 		}
-		sf := source.SuggestedFixFromCommand(latest, protocol.QuickFix)
+		sf := cache.SuggestedFixFromCommand(latest, protocol.QuickFix)
 		if len(warningFixes) > 0 {
 			warningFixes = append(warningFixes, sf)
 		}
 		if len(infoFixes) > 0 {
 			infoFixes = append(infoFixes, sf)
 		}
-
-		sort.Strings(warning)
-		sort.Strings(info)
-
-		if len(warning) > 0 {
+		if len(warningSet) > 0 {
+			warning := sortedKeys(warningSet)
 			warningFixes = append(warningFixes, suggestRunOrResetGovulncheck)
 			vulnDiagnostics = append(vulnDiagnostics, &source.Diagnostic{
 				URI:            fh.URI(),
@@ -312,10 +318,10 @@ func ModVulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot,
 				Source:         diagSource,
 				Message:        getVulnMessage(req.Mod.Path, warning, true, diagSource == source.Govulncheck),
 				SuggestedFixes: warningFixes,
-				Related:        relatedInfo,
 			})
 		}
-		if len(info) > 0 {
+		if len(infoSet) > 0 {
+			info := sortedKeys(infoSet)
 			infoFixes = append(infoFixes, suggestRunOrResetGovulncheck)
 			vulnDiagnostics = append(vulnDiagnostics, &source.Diagnostic{
 				URI:            fh.URI(),
@@ -350,41 +356,44 @@ func ModVulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot,
 			return vulnDiagnostics, nil // TODO: bug report
 		}
 
-		stdlib := stdlibVulns[0].mod.FoundVersion
-		var warning, info []string
-		var relatedInfo []protocol.DiagnosticRelatedInformation
-		for _, mv := range stdlibVulns {
-			vuln := mv.vuln
-			stdlib = mv.mod.FoundVersion
-			if !vuln.IsCalled() {
-				info = append(info, vuln.OSV.ID)
-			} else {
-				warning = append(warning, vuln.OSV.ID)
-				relatedInfo = append(relatedInfo, listRelatedInfo(ctx, snapshot, vuln)...)
+		var warningSet, infoSet = map[string]bool{}, map[string]bool{}
+		for _, finding := range stdlibVulns {
+			switch _, typ := foundVuln(finding); typ {
+			case vulnImported:
+				infoSet[finding.OSV] = true
+			case vulnCalled:
+				warningSet[finding.OSV] = true
 			}
 		}
-		if len(warning) > 0 {
+		if len(warningSet) > 0 {
+			warning := sortedKeys(warningSet)
 			fixes := []source.SuggestedFix{suggestRunOrResetGovulncheck}
 			vulnDiagnostics = append(vulnDiagnostics, &source.Diagnostic{
 				URI:            fh.URI(),
 				Range:          rng,
 				Severity:       protocol.SeverityWarning,
 				Source:         diagSource,
-				Message:        getVulnMessage(stdlib, warning, true, diagSource == source.Govulncheck),
+				Message:        getVulnMessage("go", warning, true, diagSource == source.Govulncheck),
 				SuggestedFixes: fixes,
-				Related:        relatedInfo,
 			})
+
+			// remove affecting osvs from the non-affecting osv list if any.
+			for k := range infoSet {
+				if warningSet[k] {
+					delete(infoSet, k)
+				}
+			}
 		}
-		if len(info) > 0 {
+		if len(infoSet) > 0 {
+			info := sortedKeys(infoSet)
 			fixes := []source.SuggestedFix{suggestRunOrResetGovulncheck}
 			vulnDiagnostics = append(vulnDiagnostics, &source.Diagnostic{
 				URI:            fh.URI(),
 				Range:          rng,
 				Severity:       protocol.SeverityInformation,
 				Source:         diagSource,
-				Message:        getVulnMessage(stdlib, info, false, diagSource == source.Govulncheck),
+				Message:        getVulnMessage("go", info, false, diagSource == source.Govulncheck),
 				SuggestedFixes: fixes,
-				Related:        relatedInfo,
 			})
 		}
 	}
@@ -392,29 +401,69 @@ func ModVulnerabilityDiagnostics(ctx context.Context, snapshot source.Snapshot,
 	return vulnDiagnostics, nil
 }
 
+type vulnFindingType int
+
+const (
+	vulnUnknown vulnFindingType = iota
+	vulnCalled
+	vulnImported
+	vulnRequired
+)
+
+// foundVuln returns the frame info describing discovered vulnerable symbol/package/module
+// and how this vulnerability affects the analyzed package or module.
+func foundVuln(finding *govulncheck.Finding) (*govulncheck.Frame, vulnFindingType) {
+	// finding.Trace is sorted from the imported vulnerable symbol to
+	// the entry point in the callstack.
+	// If Function is set, then Package must be set. Module will always be set.
+	// If Function is set it was found in the call graph, otherwise if Package is set
+	// it was found in the import graph, otherwise it was found in the require graph.
+	// See the documentation of govulncheck.Finding.
+	if len(finding.Trace) == 0 { // this shouldn't happen, but just in case...
+		return nil, vulnUnknown
+	}
+	vuln := finding.Trace[0]
+	if vuln.Package == "" {
+		return vuln, vulnRequired
+	}
+	if vuln.Function == "" {
+		return vuln, vulnImported
+	}
+	return vuln, vulnCalled
+}
+
+func sortedKeys(m map[string]bool) []string {
+	ret := make([]string, 0, len(m))
+	for k := range m {
+		ret = append(ret, k)
+	}
+	sort.Strings(ret)
+	return ret
+}
+
 // suggestGovulncheckAction returns a code action that suggests either run govulncheck
 // for more accurate investigation (if the present vulncheck diagnostics are based on
 // analysis less accurate than govulncheck) or reset the existing govulncheck result
 // (if the present vulncheck diagnostics are already based on govulncheck run).
-func suggestGovulncheckAction(fromGovulncheck bool, uri span.URI) (source.SuggestedFix, error) {
+func suggestGovulncheckAction(fromGovulncheck bool, uri protocol.DocumentURI) (source.SuggestedFix, error) {
 	if fromGovulncheck {
 		resetVulncheck, err := command.NewResetGoModDiagnosticsCommand("Reset govulncheck result", command.ResetGoModDiagnosticsArgs{
-			URIArg:           command.URIArg{URI: protocol.DocumentURI(uri)},
+			URIArg:           command.URIArg{URI: uri},
 			DiagnosticSource: string(source.Govulncheck),
 		})
 		if err != nil {
 			return source.SuggestedFix{}, err
 		}
-		return source.SuggestedFixFromCommand(resetVulncheck, protocol.QuickFix), nil
+		return cache.SuggestedFixFromCommand(resetVulncheck, protocol.QuickFix), nil
 	}
 	vulncheck, err := command.NewRunGovulncheckCommand("Run govulncheck to verify", command.VulncheckArgs{
-		URI:     protocol.DocumentURI(uri),
+		URI:     uri,
 		Pattern: "./...",
 	})
 	if err != nil {
 		return source.SuggestedFix{}, err
 	}
-	return source.SuggestedFixFromCommand(vulncheck, protocol.QuickFix), nil
+	return cache.SuggestedFixFromCommand(vulncheck, protocol.QuickFix), nil
 }
 
 func getVulnMessage(mod string, vulns []string, used, fromGovulncheck bool) string {
@@ -446,71 +495,17 @@ func getVulnMessage(mod string, vulns []string, used, fromGovulncheck bool) stri
 	return b.String()
 }
 
-func listRelatedInfo(ctx context.Context, snapshot source.Snapshot, vuln *govulncheck.Vuln) []protocol.DiagnosticRelatedInformation {
-	var ri []protocol.DiagnosticRelatedInformation
-	for _, m := range vuln.Modules {
-		for _, p := range m.Packages {
-			for _, c := range p.CallStacks {
-				if len(c.Frames) == 0 {
-					continue
-				}
-				entry := c.Frames[0]
-				pos := entry.Position
-				if pos.Filename == "" {
-					continue // token.Position Filename is an optional field.
-				}
-				uri := span.URIFromPath(pos.Filename)
-				startPos := protocol.Position{
-					Line: uint32(pos.Line) - 1,
-					// We need to read the file contents to precisesly map
-					// token.Position (pos) to the UTF16-based column offset
-					// protocol.Position requires. That can be expensive.
-					// We need this related info to just help users to open
-					// the entry points of the callstack and once the file is
-					// open, we will compute the precise location based on the
-					// open file contents. So, use the beginning of the line
-					// as the position here instead of precise UTF16-based
-					// position computation.
-					Character: 0,
-				}
-				ri = append(ri, protocol.DiagnosticRelatedInformation{
-					Location: protocol.Location{
-						URI: protocol.URIFromSpanURI(uri),
-						Range: protocol.Range{
-							Start: startPos,
-							End:   startPos,
-						},
-					},
-					Message: fmt.Sprintf("[%v] %v -> %v.%v", vuln.OSV.ID, entry.Name(), p.Path, c.Symbol),
-				})
-			}
-		}
-	}
-	return ri
-}
-
-func formatMessage(v *govulncheck.Vuln) string {
-	details := []byte(v.OSV.Details)
-	// Remove any new lines that are not preceded or followed by a new line.
-	for i, r := range details {
-		if r == '\n' && i > 0 && details[i-1] != '\n' && i+1 < len(details) && details[i+1] != '\n' {
-			details[i] = ' '
-		}
-	}
-	return strings.TrimSpace(strings.Replace(string(details), "\n\n", "\n\n  ", -1))
-}
-
 // href returns the url for the vulnerability information.
 // Eventually we should retrieve the url embedded in the osv.Entry.
 // While vuln.go.dev is under development, this always returns
 // the page in pkg.go.dev.
-func href(vuln *osv.Entry) string {
-	return fmt.Sprintf("https://pkg.go.dev/vuln/%s", vuln.ID)
+func href(vulnID string) string {
+	return fmt.Sprintf("https://pkg.go.dev/vuln/%s", vulnID)
 }
 
-func getUpgradeCodeAction(fh source.FileHandle, req *modfile.Require, version string) (protocol.Command, error) {
+func getUpgradeCodeAction(fh file.Handle, req *modfile.Require, version string) (protocol.Command, error) {
 	cmd, err := command.NewUpgradeDependencyCommand(upgradeTitle(version), command.DependencyArgs{
-		URI:        protocol.URIFromSpanURI(fh.URI()),
+		URI:        fh.URI(),
 		AddRequire: false,
 		GoCmdArgs:  []string{req.Mod.Path + "@" + version},
 	})
