@@ -7,6 +7,7 @@ package lsp
 import (
 	"context"
 
+	"golang.org/x/tools/gopls/internal/file"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/source"
 	"golang.org/x/tools/gopls/internal/lsp/template"
@@ -15,7 +16,7 @@ import (
 	"golang.org/x/tools/internal/event/tag"
 )
 
-func (s *Server) references(ctx context.Context, params *protocol.ReferenceParams) (_ []protocol.Location, rerr error) {
+func (s *server) References(ctx context.Context, params *protocol.ReferenceParams) (_ []protocol.Location, rerr error) {
 	recordLatency := telemetry.StartLatencyTimer("references")
 	defer func() {
 		recordLatency(ctx, rerr)
@@ -24,12 +25,12 @@ func (s *Server) references(ctx context.Context, params *protocol.ReferenceParam
 	ctx, done := event.Start(ctx, "lsp.Server.references", tag.URI.Of(params.TextDocument.URI))
 	defer done()
 
-	snapshot, fh, ok, release, err := s.beginFileRequest(ctx, params.TextDocument.URI, source.UnknownKind)
+	snapshot, fh, ok, release, err := s.beginFileRequest(ctx, params.TextDocument.URI, file.UnknownKind)
 	defer release()
 	if !ok {
 		return nil, err
 	}
-	if snapshot.FileKind(fh) == source.Tmpl {
+	if snapshot.FileKind(fh) == file.Tmpl {
 		return template.References(ctx, snapshot, fh, params)
 	}
 	return source.References(ctx, snapshot, fh, params.Position, params.Context.IncludeDeclaration)
