@@ -33,10 +33,10 @@ import (
 	"golang.org/x/tools/gopls/internal/lsp/lsprpc"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/safetoken"
-	"golang.org/x/tools/gopls/internal/lsp/source"
 	"golang.org/x/tools/gopls/internal/lsp/tests"
 	"golang.org/x/tools/gopls/internal/lsp/tests/compare"
 	"golang.org/x/tools/internal/diff"
+	"golang.org/x/tools/internal/diff/myers"
 	"golang.org/x/tools/internal/jsonrpc2"
 	"golang.org/x/tools/internal/jsonrpc2/servertest"
 	"golang.org/x/tools/internal/testenv"
@@ -106,7 +106,7 @@ var update = flag.Bool("update", false, "if set, update test data during marker
 //     the file content used as the skip message.
 //   - "flags": this file is treated as a whitespace-separated list of flags
 //     that configure the MarkerTest instance. Supported flags:
-//     -min_go=go1.18 sets the minimum Go version for the test;
+//     -min_go=go1.20 sets the minimum Go version for the test;
 //     -cgo requires that CGO_ENABLED is set and the cgo tool is available
 //     -write_sumfile=a,b,c instructs the test runner to generate go.sum files
 //     in these directories before running the test.
@@ -222,6 +222,10 @@ var update = flag.Bool("update", false, "if set, update test data during marker
 //     textDocument/implementation query at the src location and
 //     checks that the resulting set of locations matches want.
 //
+//   - incomingcalls(src location, want ...location): makes a
+//     callHierarchy/incomingCalls query at the src location, and checks that
+//     the set of call.From locations matches want.
+//
 //   - item(label, details, kind): defines a completion item with the provided
 //     fields. This information is not positional, and therefore @item markers
 //     may occur anywhere in the source. Used in conjunction with @complete,
@@ -233,6 +237,10 @@ var update = flag.Bool("update", false, "if set, update test data during marker
 //   - loc(name, location): specifies the name for a location in the source. These
 //     locations may be referenced by other markers.
 //
+//   - outgoingcalls(src location, want ...location): makes a
+//     callHierarchy/outgoingCalls query at the src location, and checks that
+//     the set of call.To locations matches want.
+//
 //   - preparerename(src, spn, placeholder): asserts that a textDocument/prepareRename
 //     request at the src location expands to the spn location, with given
 //     placeholder. If placeholder is "", this is treated as a negative
@@ -255,6 +263,11 @@ var update = flag.Bool("update", false, "if set, update test data during marker
 //     This action is executed for its editing effects on the source files.
 //     Like rename, the golden directory contains the expected transformed files.
 //
+//   - suggestedfixerr(location, regexp, kind, wantError): specifies that the
+//     suggestedfix operation should fail with an error that matches the expectation.
+//     (Failures in the computation to offer a fix do not generally result
+//     in LSP errors, so this marker is not appropriate for testing them.)
+//
 //   - rank(location, ...completionItem): executes a textDocument/completion
 //     request at the given location, and verifies that each expected
 //     completion item occurs in the results, in the expected order. Other
@@ -288,6 +301,10 @@ var update = flag.Bool("update", false, "if set, update test data during marker
 //     location information. There is no point to using more than one
 //     @symbol marker in a given file.
 //
+//   - token(location, tokenType, mod): makes a textDocument/semanticTokens/range
+//     request at the given location, and asserts that the result includes
+//     exactly one token with the given token type and modifier string.
+//
 //   - workspacesymbol(query, golden): makes a workspace/symbol request for the
 //     given query, formats the response with one symbol per line, and compares
 //     against the named golden file. As workspace symbols are by definition a
@@ -377,10 +394,6 @@ var update = flag.Bool("update", false, "if set, update test data during marker
 //
 // # TODO
 //
-// This API is a work-in-progress, as we migrate existing marker tests from
-// internal/lsp/tests.
-//
-// Remaining TODO:
 //   - reorganize regtest packages (and rename to just 'test'?)
 //   - Rename the files .txtar.
 //   - Provide some means by which locations in the standard library
@@ -390,14 +403,6 @@ var update = flag.Bool("update", false, "if set, update test data during marker
 //     multiple variations of a marker, it would be nice to support a more
 //     flexible signature: can codeaction, codeactionedit, codeactionerr, and
 //     suggestedfix be consolidated?
-//
-// Existing marker tests (in ../testdata) to port:
-//   - CallHierarchy
-//   - SemanticTokens
-//   - SuggestedFixes
-//   - InlayHints
-//   - Renames
-//   - SelectionRanges
 func RunMarkerTests(t *testing.T, dir string) {
 	// The marker tests must be able to run go/packages.Load.
 	testenv.NeedsGoPackages(t)
@@ -563,9 +568,37 @@ type marker struct {
 	note *expect.Note
 }
 
+// ctx returns the mark context.
+func (m marker) ctx() context.Context { return m.run.env.Ctx }
+
+// T returns the testing.TB for this mark.
+func (m marker) T() testing.TB { return m.run.env.T }
+
 // server returns the LSP server for the marker test run.
-func (m marker) server() protocol.Server {
-	return m.run.env.Editor.Server
+func (m marker) server() protocol.Server { return m.run.env.Editor.Server }
+
+// uri returns the URI of the file containing the marker.
+func (mark marker) uri() protocol.DocumentURI {
+	return mark.run.env.Sandbox.Workdir.URI(mark.run.test.fset.File(mark.note.Pos).Name())
+}
+
+// document returns a protocol.TextDocumentIdentifier for the current file.
+func (mark marker) document() protocol.TextDocumentIdentifier {
+	return protocol.TextDocumentIdentifier{URI: mark.uri()}
+}
+
+// path returns the relative path to the file containing the marker.
+func (mark marker) path() string {
+	return mark.run.env.Sandbox.Workdir.RelPath(mark.run.test.fset.File(mark.note.Pos).Name())
+}
+
+// mapper returns a *protocol.Mapper for the current file.
+func (mark marker) mapper() *protocol.Mapper {
+	mapper, err := mark.run.env.Editor.Mapper(mark.path())
+	if err != nil {
+		mark.T().Fatalf("failed to get mapper for current mark: %v", err)
+	}
+	return mapper
 }
 
 // errorf reports an error with a prefix indicating the position of the marker note.
@@ -577,7 +610,7 @@ func (mark marker) errorf(format string, args ...any) {
 	// t.Errorf to avoid reporting uninteresting positions in the Go source of
 	// the driver. However, this loses the order of stderr wrt "FAIL: TestFoo"
 	// subtest dividers.
-	mark.run.env.T.Errorf("%s: %s", mark.run.fmtPos(mark.note.Pos), msg)
+	mark.T().Errorf("%s: %s", mark.run.fmtPos(mark.note.Pos), msg)
 }
 
 // valueMarkerFunc returns a wrapper around a function that allows it to be
@@ -729,16 +762,22 @@ var actionMarkerFuncs = map[string]func(marker){
 	"highlight":        actionMarkerFunc(highlightMarker),
 	"hover":            actionMarkerFunc(hoverMarker),
 	"implementation":   actionMarkerFunc(implementationMarker),
+	"incomingcalls":    actionMarkerFunc(incomingCallsMarker),
+	"inlayhints":       actionMarkerFunc(inlayhintsMarker),
+	"outgoingcalls":    actionMarkerFunc(outgoingCallsMarker),
 	"preparerename":    actionMarkerFunc(prepareRenameMarker),
 	"rank":             actionMarkerFunc(rankMarker),
 	"rankl":            actionMarkerFunc(ranklMarker),
 	"refs":             actionMarkerFunc(refsMarker),
 	"rename":           actionMarkerFunc(renameMarker),
 	"renameerr":        actionMarkerFunc(renameErrMarker),
+	"selectionrange":   actionMarkerFunc(selectionRangeMarker),
 	"signature":        actionMarkerFunc(signatureMarker),
 	"snippet":          actionMarkerFunc(snippetMarker),
 	"suggestedfix":     actionMarkerFunc(suggestedfixMarker),
+	"suggestedfixerr":  actionMarkerFunc(suggestedfixErrMarker),
 	"symbol":           actionMarkerFunc(symbolMarker),
+	"token":            actionMarkerFunc(tokenMarker),
 	"typedef":          actionMarkerFunc(typedefMarker),
 	"workspacesymbol":  actionMarkerFunc(workspaceSymbolMarker),
 }
@@ -1124,16 +1163,6 @@ func (c *marker) sprintf(format string, args ...any) string {
 	return fmt.Sprintf(format, args2...)
 }
 
-// uri returns the URI of the file containing the marker.
-func (mark marker) uri() protocol.DocumentURI {
-	return mark.run.env.Sandbox.Workdir.URI(mark.run.test.fset.File(mark.note.Pos).Name())
-}
-
-// path returns the relative path to the file containing the marker.
-func (mark marker) path() string {
-	return mark.run.env.Sandbox.Workdir.RelPath(mark.run.test.fset.File(mark.note.Pos).Name())
-}
-
 // fmtLoc formats the given pos in the context of the test, using
 // archive-relative paths for files and including the line number in the full
 // archive file.
@@ -1194,21 +1223,24 @@ func (run *markerTestRun) fmtLocDetails(loc protocol.Location, includeTxtPos boo
 		run.env.T.Errorf("internal error: %v", err)
 		return "<invalid location>"
 	}
-	s, err := m.LocationSpan(loc)
+	start, end, err := m.RangeOffsets(loc.Range)
 	if err != nil {
 		run.env.T.Errorf("error formatting location %s: %v", loc, err)
 		return "<invalid location>"
 	}
-
-	innerSpan := fmt.Sprintf("%d:%d", s.Start().Line(), s.Start().Column())       // relative to the embedded file
-	outerSpan := fmt.Sprintf("%d:%d", lines+s.Start().Line(), s.Start().Column()) // relative to the archive file
-	if s.Start() != s.End() {
-		if s.End().Line() == s.Start().Line() {
-			innerSpan += fmt.Sprintf("-%d", s.End().Column())
-			outerSpan += fmt.Sprintf("-%d", s.End().Column())
+	var (
+		startLine, startCol8 = m.OffsetLineCol8(start)
+		endLine, endCol8     = m.OffsetLineCol8(end)
+	)
+	innerSpan := fmt.Sprintf("%d:%d", startLine, startCol8)       // relative to the embedded file
+	outerSpan := fmt.Sprintf("%d:%d", lines+startLine, startCol8) // relative to the archive file
+	if start != end {
+		if endLine == startLine {
+			innerSpan += fmt.Sprintf("-%d", endCol8)
+			outerSpan += fmt.Sprintf("-%d", endCol8)
 		} else {
-			innerSpan += fmt.Sprintf("-%d:%d", s.End().Line(), s.End().Column())
-			innerSpan += fmt.Sprintf("-%d:%d", lines+s.End().Line(), s.End().Column())
+			innerSpan += fmt.Sprintf("-%d:%d", endLine, endCol8)
+			outerSpan += fmt.Sprintf("-%d:%d", lines+endLine, endCol8)
 		}
 	}
 
@@ -1381,7 +1413,7 @@ func (we wantError) check(mark marker, err error) {
 
 	if we.golden != nil {
 		// Error message must match @id golden file.
-		wantBytes, ok := we.golden.Get(mark.run.env.T, "", []byte(got))
+		wantBytes, ok := we.golden.Get(mark.T(), "", []byte(got))
 		if !ok {
 			mark.errorf("@%s: missing @%s entry", mark.note.Name, we.golden.id)
 			return
@@ -1409,7 +1441,7 @@ func (we wantError) check(mark marker, err error) {
 func checkChangedFiles(mark marker, changed map[string][]byte, golden *Golden) {
 	// Check changed files match expectations.
 	for filename, got := range changed {
-		if want, ok := golden.Get(mark.run.env.T, filename, got); !ok {
+		if want, ok := golden.Get(mark.T(), filename, got); !ok {
 			mark.errorf("%s: unexpected change to file %s; got:\n%s",
 				mark.note.Name, filename, got)
 
@@ -1423,7 +1455,7 @@ func checkChangedFiles(mark marker, changed map[string][]byte, golden *Golden) {
 	// Report unmet expectations.
 	for filename := range golden.data {
 		if _, ok := changed[filename]; !ok {
-			want, _ := golden.Get(mark.run.env.T, filename, nil)
+			want, _ := golden.Get(mark.T(), filename, nil)
 			mark.errorf("%s: missing change to file %s; want:\n%s",
 				mark.note.Name, filename, want)
 		}
@@ -1436,17 +1468,24 @@ func checkDiffs(mark marker, changed map[string][]byte, golden *Golden) {
 	diffs := make(map[string]string)
 	for name, after := range changed {
 		before := mark.run.env.FileContent(name)
-		edits := diff.Strings(before, string(after))
+		// TODO(golang/go#64023): switch back to diff.Strings.
+		edits := myers.ComputeEdits(before, string(after))
 		d, err := diff.ToUnified("before", "after", before, edits, 0)
 		if err != nil {
 			// Can't happen: edits are consistent.
 			log.Fatalf("internal error in diff.ToUnified: %v", err)
 		}
-		diffs[name] = d
+		// Trim the unified header from diffs, as it is unnecessary and repetitive.
+		difflines := strings.Split(d, "\n")
+		if len(difflines) >= 2 && strings.HasPrefix(difflines[1], "+++") {
+			diffs[name] = strings.Join(difflines[2:], "\n")
+		} else {
+			diffs[name] = d
+		}
 	}
 	// Check changed files match expectations.
 	for filename, got := range diffs {
-		if want, ok := golden.Get(mark.run.env.T, filename, []byte(got)); !ok {
+		if want, ok := golden.Get(mark.T(), filename, []byte(got)); !ok {
 			mark.errorf("%s: unexpected change to file %s; got diff:\n%s",
 				mark.note.Name, filename, got)
 
@@ -1458,7 +1497,7 @@ func checkDiffs(mark marker, changed map[string][]byte, golden *Golden) {
 	// Report unmet expectations.
 	for filename := range golden.data {
 		if _, ok := changed[filename]; !ok {
-			want, _ := golden.Get(mark.run.env.T, filename, nil)
+			want, _ := golden.Get(mark.T(), filename, nil)
 			mark.errorf("%s: missing change to file %s; want:\n%s",
 				mark.note.Name, filename, want)
 		}
@@ -1640,15 +1679,8 @@ func acceptCompletionMarker(mark marker, src protocol.Location, label string, go
 		return
 	}
 	filename := mark.path()
-	mapper, err := mark.run.env.Editor.Mapper(filename)
-	if err != nil {
-		mark.errorf("Editor.Mapper(%s) failed: %v", filename, err)
-		return
-	}
-
-	patched, _, err := source.ApplyProtocolEdits(mapper, append([]protocol.TextEdit{
-		*selected.TextEdit,
-	}, selected.AdditionalTextEdits...))
+	mapper := mark.mapper()
+	patched, _, err := protocol.ApplyEdits(mapper, append([]protocol.TextEdit{*selected.TextEdit}, selected.AdditionalTextEdits...))
 
 	if err != nil {
 		mark.errorf("ApplyProtocolEdits failed: %v", err)
@@ -1683,7 +1715,7 @@ func typedefMarker(mark marker, src, dst protocol.Location) {
 func foldingRangeMarker(mark marker, g *Golden) {
 	env := mark.run.env
 	ranges, err := mark.server().FoldingRange(env.Ctx, &protocol.FoldingRangeParams{
-		TextDocument: protocol.TextDocumentIdentifier{URI: mark.uri()},
+		TextDocument: mark.document(),
 	})
 	if err != nil {
 		mark.errorf("foldingRange failed: %v", err)
@@ -1710,12 +1742,12 @@ func foldingRangeMarker(mark marker, g *Golden) {
 		mark.errorf("Editor.Mapper(%s) failed: %v", filename, err)
 		return
 	}
-	got, _, err := source.ApplyProtocolEdits(mapper, edits)
+	got, _, err := protocol.ApplyEdits(mapper, edits)
 	if err != nil {
 		mark.errorf("ApplyProtocolEdits failed: %v", err)
 		return
 	}
-	want, _ := g.Get(mark.run.env.T, "", got)
+	want, _ := g.Get(mark.T(), "", got)
 	if diff := compare.Bytes(want, got); diff != "" {
 		mark.errorf("foldingRange mismatch:\n%s", diff)
 	}
@@ -1723,8 +1755,8 @@ func foldingRangeMarker(mark marker, g *Golden) {
 
 // formatMarker implements the @format marker.
 func formatMarker(mark marker, golden *Golden) {
-	edits, err := mark.server().Formatting(mark.run.env.Ctx, &protocol.DocumentFormattingParams{
-		TextDocument: protocol.TextDocumentIdentifier{URI: mark.uri()},
+	edits, err := mark.server().Formatting(mark.ctx(), &protocol.DocumentFormattingParams{
+		TextDocument: mark.document(),
 	})
 	var got []byte
 	if err != nil {
@@ -1737,7 +1769,7 @@ func formatMarker(mark marker, golden *Golden) {
 			mark.errorf("Editor.Mapper(%s) failed: %v", filename, err)
 		}
 
-		got, _, err = source.ApplyProtocolEdits(mapper, edits)
+		got, _, err = protocol.ApplyEdits(mapper, edits)
 		if err != nil {
 			mark.errorf("ApplyProtocolEdits failed: %v", err)
 			return
@@ -1788,7 +1820,7 @@ func hoverMarker(mark marker, src, dst protocol.Location, golden *Golden) {
 	}
 	wantMD := ""
 	if golden != nil {
-		wantBytes, _ := golden.Get(mark.run.env.T, "hover.md", []byte(gotMD))
+		wantBytes, _ := golden.Get(mark.T(), "hover.md", []byte(gotMD))
 		wantMD = string(wantBytes)
 	}
 	// Normalize newline termination: archive files can't express non-newline
@@ -1838,7 +1870,7 @@ func renameMarker(mark marker, loc protocol.Location, newName string, golden *Go
 		mark.errorf("rename failed: %v. (Use @renameerr for expected errors.)", err)
 		return
 	}
-	checkChangedFiles(mark, changed, golden)
+	checkDiffs(mark, changed, golden)
 }
 
 // renameErrMarker implements the @renamererr(location, new, error) marker.
@@ -1847,9 +1879,67 @@ func renameErrMarker(mark marker, loc protocol.Location, newName string, wantErr
 	wantErr.check(mark, err)
 }
 
+func selectionRangeMarker(mark marker, loc protocol.Location, g *Golden) {
+	ranges, err := mark.server().SelectionRange(mark.ctx(), &protocol.SelectionRangeParams{
+		TextDocument: mark.document(),
+		Positions:    []protocol.Position{loc.Range.Start},
+	})
+	if err != nil {
+		mark.errorf("SelectionRange failed: %v", err)
+		return
+	}
+	var buf bytes.Buffer
+	m := mark.mapper()
+	for i, path := range ranges {
+		fmt.Fprintf(&buf, "Ranges %d:", i)
+		rng := path
+		for {
+			s, e, err := m.RangeOffsets(rng.Range)
+			if err != nil {
+				mark.errorf("RangeOffsets failed: %v", err)
+				return
+			}
+
+			var snippet string
+			if e-s < 30 {
+				snippet = string(m.Content[s:e])
+			} else {
+				snippet = string(m.Content[s:s+15]) + "..." + string(m.Content[e-15:e])
+			}
+
+			fmt.Fprintf(&buf, "\n\t%v %q", rng.Range, strings.ReplaceAll(snippet, "\n", "\\n"))
+
+			if rng.Parent == nil {
+				break
+			}
+			rng = *rng.Parent
+		}
+		buf.WriteRune('\n')
+	}
+	compareGolden(mark, "selection range", buf.Bytes(), g)
+}
+
+func tokenMarker(mark marker, loc protocol.Location, tokenType, mod string) {
+	tokens := mark.run.env.SemanticTokensRange(loc)
+	if len(tokens) != 1 {
+		mark.errorf("got %d tokens, want 1", len(tokens))
+		return
+	}
+	tok := tokens[0]
+	if tok.TokenType != tokenType {
+		mark.errorf("token type = %q, want %q", tok.TokenType, tokenType)
+	}
+	if tok.Mod != mod {
+		mark.errorf("token mod = %q, want %q", tok.Mod, mod)
+	}
+}
+
 func signatureMarker(mark marker, src protocol.Location, label string, active int64) {
 	got := mark.run.env.SignatureHelp(src)
 	if label == "" {
+		// A null result is expected.
+		// (There's no point having a @signatureerr marker
+		// because the server handler suppresses all errors.)
 		if got != nil && len(got.Signatures) > 0 {
 			mark.errorf("signatureHelp = %v, want 0 signatures", got)
 		}
@@ -1879,7 +1969,7 @@ func rename(env *Env, loc protocol.Location, newName string) (map[string][]byte,
 	editMap, err := env.Editor.Server.Rename(env.Ctx, &protocol.RenameParams{
 		TextDocument: protocol.TextDocumentIdentifier{URI: loc.URI},
 		Position:     loc.Range.Start,
-		NewName:      string(newName),
+		NewName:      newName,
 	})
 	if err != nil {
 		return nil, err
@@ -1921,7 +2011,7 @@ func applyDocumentChanges(env *Env, changes []protocol.DocumentChanges, fileChan
 			if err != nil {
 				return err
 			}
-			patched, _, err := source.ApplyProtocolEdits(mapper, change.TextDocumentEdit.Edits)
+			patched, _, err := protocol.ApplyEdits(mapper, change.TextDocumentEdit.Edits)
 			if err != nil {
 				return err
 			}
@@ -2055,6 +2145,20 @@ func suggestedfixMarker(mark marker, loc protocol.Location, re *regexp.Regexp, g
 	checkDiffs(mark, changed, golden)
 }
 
+func suggestedfixErrMarker(mark marker, loc protocol.Location, re *regexp.Regexp, wantErr wantError) {
+	loc.Range.End = loc.Range.Start // diagnostics ignore end position.
+	// Find and remove the matching diagnostic.
+	diag, ok := removeDiagnostic(mark, loc, re)
+	if !ok {
+		mark.errorf("no diagnostic at %v matches %q", loc, re)
+		return
+	}
+
+	// Apply the fix it suggests.
+	_, err := codeAction(mark.run.env, loc.URI, diag.Range, "quickfix", &diag, nil)
+	wantErr.check(mark, err)
+}
+
 // codeAction executes a textDocument/codeAction request for the specified
 // location and kind. If diag is non-nil, it is used as the code action
 // context.
@@ -2170,12 +2274,10 @@ func codeActionChanges(env *Env, uri protocol.DocumentURI, rng protocol.Range, a
 	return nil, nil
 }
 
-// TODO(adonovan): suggestedfixerr
-
 // refsMarker implements the @refs marker.
 func refsMarker(mark marker, src protocol.Location, want ...protocol.Location) {
 	refs := func(includeDeclaration bool, want []protocol.Location) error {
-		got, err := mark.server().References(mark.run.env.Ctx, &protocol.ReferenceParams{
+		got, err := mark.server().References(mark.ctx(), &protocol.ReferenceParams{
 			TextDocumentPositionParams: protocol.LocationTextDocumentPositionParams(src),
 			Context: protocol.ReferenceContext{
 				IncludeDeclaration: includeDeclaration,
@@ -2205,7 +2307,7 @@ func refsMarker(mark marker, src protocol.Location, want ...protocol.Location) {
 
 // implementationMarker implements the @implementation marker.
 func implementationMarker(mark marker, src protocol.Location, want ...protocol.Location) {
-	got, err := mark.server().Implementation(mark.run.env.Ctx, &protocol.ImplementationParams{
+	got, err := mark.server().Implementation(mark.ctx(), &protocol.ImplementationParams{
 		TextDocumentPositionParams: protocol.LocationTextDocumentPositionParams(src),
 	})
 	if err != nil {
@@ -2217,13 +2319,118 @@ func implementationMarker(mark marker, src protocol.Location, want ...protocol.L
 	}
 }
 
+func itemLocation(item protocol.CallHierarchyItem) protocol.Location {
+	return protocol.Location{
+		URI:   item.URI,
+		Range: item.Range,
+	}
+}
+
+func incomingCallsMarker(mark marker, src protocol.Location, want ...protocol.Location) {
+	getCalls := func(item protocol.CallHierarchyItem) ([]protocol.Location, error) {
+		calls, err := mark.server().IncomingCalls(mark.ctx(), &protocol.CallHierarchyIncomingCallsParams{Item: item})
+		if err != nil {
+			return nil, err
+		}
+		var locs []protocol.Location
+		for _, call := range calls {
+			locs = append(locs, itemLocation(call.From))
+		}
+		return locs, nil
+	}
+	callHierarchy(mark, src, getCalls, want)
+}
+
+func outgoingCallsMarker(mark marker, src protocol.Location, want ...protocol.Location) {
+	getCalls := func(item protocol.CallHierarchyItem) ([]protocol.Location, error) {
+		calls, err := mark.server().OutgoingCalls(mark.ctx(), &protocol.CallHierarchyOutgoingCallsParams{Item: item})
+		if err != nil {
+			return nil, err
+		}
+		var locs []protocol.Location
+		for _, call := range calls {
+			locs = append(locs, itemLocation(call.To))
+		}
+		return locs, nil
+	}
+	callHierarchy(mark, src, getCalls, want)
+}
+
+type callHierarchyFunc = func(protocol.CallHierarchyItem) ([]protocol.Location, error)
+
+func callHierarchy(mark marker, src protocol.Location, getCalls callHierarchyFunc, want []protocol.Location) {
+	items, err := mark.server().PrepareCallHierarchy(mark.ctx(), &protocol.CallHierarchyPrepareParams{
+		TextDocumentPositionParams: protocol.LocationTextDocumentPositionParams(src),
+	})
+	if err != nil {
+		mark.errorf("PrepareCallHierarchy failed: %v", err)
+		return
+	}
+	if nitems := len(items); nitems != 1 {
+		mark.errorf("PrepareCallHierarchy returned %d items, want exactly 1", nitems)
+		return
+	}
+	if loc := itemLocation(items[0]); loc != src {
+		mark.errorf("PrepareCallHierarchy found call %v, want %v", loc, src)
+		return
+	}
+	calls, err := getCalls(items[0])
+	if err != nil {
+		mark.errorf("call hierarchy failed: %v", err)
+		return
+	}
+	if calls == nil {
+		calls = []protocol.Location{}
+	}
+	// TODO(rfindley): why aren't call hierarchy results stable?
+	sortLocs := func(locs []protocol.Location) {
+		sort.Slice(locs, func(i, j int) bool {
+			return protocol.CompareLocation(locs[i], locs[j]) < 0
+		})
+	}
+	sortLocs(want)
+	sortLocs(calls)
+	if d := cmp.Diff(want, calls); d != "" {
+		mark.errorf("call hierarchy: unexpected results (-want +got):\n%s", d)
+	}
+}
+
+func inlayhintsMarker(mark marker, g *Golden) {
+	hints := mark.run.env.InlayHints(mark.path())
+
+	// Map inlay hints to text edits.
+	edits := make([]protocol.TextEdit, len(hints))
+	for i, hint := range hints {
+		var paddingLeft, paddingRight string
+		if hint.PaddingLeft {
+			paddingLeft = " "
+		}
+		if hint.PaddingRight {
+			paddingRight = " "
+		}
+		edits[i] = protocol.TextEdit{
+			Range:   protocol.Range{Start: hint.Position, End: hint.Position},
+			NewText: fmt.Sprintf("<%s%s%s>", paddingLeft, hint.Label[0].Value, paddingRight),
+		}
+	}
+
+	m := mark.mapper()
+	got, _, err := protocol.ApplyEdits(m, edits)
+	if err != nil {
+		mark.errorf("ApplyProtocolEdits: %v", err)
+		return
+	}
+
+	compareGolden(mark, "inlay hints", got, g)
+}
+
 func prepareRenameMarker(mark marker, src, spn protocol.Location, placeholder string) {
 	params := &protocol.PrepareRenameParams{
 		TextDocumentPositionParams: protocol.LocationTextDocumentPositionParams(src),
 	}
-	got, err := mark.run.env.Editor.Server.PrepareRename(mark.run.env.Ctx, params)
+	got, err := mark.server().PrepareRename(mark.ctx(), params)
 	if err != nil {
-		mark.run.env.T.Fatal(err)
+		mark.T().Fatal(err)
 	}
 	if placeholder == "" {
 		if got != nil {
@@ -2240,7 +2447,7 @@ func prepareRenameMarker(mark marker, src, spn protocol.Location, placeholder st
 // symbolMarker implements the @symbol marker.
 func symbolMarker(mark marker, golden *Golden) {
 	// Retrieve information about all symbols in this file.
-	symbols, err := mark.server().DocumentSymbol(mark.run.env.Ctx, &protocol.DocumentSymbolParams{
+	symbols, err := mark.server().DocumentSymbol(mark.ctx(), &protocol.DocumentSymbolParams{
 		TextDocument: protocol.TextDocumentIdentifier{URI: mark.uri()},
 	})
 	if err != nil {
@@ -2256,7 +2463,7 @@ func symbolMarker(mark marker, golden *Golden) {
 		// so we ascertain which one and then transcode.
 		data, err := json.Marshal(symbol)
 		if err != nil {
-			mark.run.env.T.Fatal(err)
+			mark.T().Fatal(err)
 		}
 		if _, ok := symbol.(map[string]any)["location"]; ok {
 			// This case is not reached because Editor initialization
@@ -2264,14 +2471,14 @@ func symbolMarker(mark marker, golden *Golden) {
 			// TODO(adonovan): test this too.
 			var sym protocol.SymbolInformation
 			if err := json.Unmarshal(data, &sym); err != nil {
-				mark.run.env.T.Fatal(err)
+				mark.T().Fatal(err)
 			}
 			mark.errorf("fake Editor doesn't support SymbolInformation")
 
 		} else {
 			var sym protocol.DocumentSymbol // new hierarchical hotness
 			if err := json.Unmarshal(data, &sym); err != nil {
-				mark.run.env.T.Fatal(err)
+				mark.T().Fatal(err)
 			}
 
 			// Print each symbol in the response tree.
@@ -2297,7 +2504,7 @@ func symbolMarker(mark marker, golden *Golden) {
 	got := []byte(strings.Join(lines, "\n"))
 
 	// Compare with golden.
-	want, ok := golden.Get(mark.run.env.T, "", got)
+	want, ok := golden.Get(mark.T(), "", got)
 	if !ok {
 		mark.errorf("%s: missing golden file @%s", mark.note.Name, golden.id)
 	} else if diff := cmp.Diff(string(got), string(want)); diff != "" {
@@ -2329,7 +2536,7 @@ func workspaceSymbolMarker(mark marker, query string, golden *Golden) {
 		Query: query,
 	}
 
-	gotSymbols, err := mark.server().Symbol(mark.run.env.Ctx, params)
+	gotSymbols, err := mark.server().Symbol(mark.ctx(), params)
 	if err != nil {
 		mark.errorf("Symbol(%q) failed: %v", query, err)
 		return
@@ -2355,7 +2562,7 @@ func workspaceSymbolMarker(mark marker, query string, golden *Golden) {
 //
 // TODO(rfindley): use this helper in more places.
 func compareGolden(mark marker, op string, got []byte, g *Golden) {
-	want, ok := g.Get(mark.run.env.T, "", got)
+	want, ok := g.Get(mark.T(), "", got)
 	if !ok {
 		mark.errorf("missing golden file @%s", g.id)
 		return
