@@ -18,6 +18,7 @@ import (
 	"strings"
 	"time"
 
+	"golang.org/x/tools/gopls/internal/file"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/safetoken"
 	"golang.org/x/tools/gopls/internal/lsp/source"
@@ -38,27 +39,19 @@ const maxFullFileSize int = 100000
 // semDebug should NEVER be true in checked-in code
 const semDebug = false
 
-func (s *Server) semanticTokensFull(ctx context.Context, params *protocol.SemanticTokensParams) (*protocol.SemanticTokens, error) {
-	ctx, done := event.Start(ctx, "lsp.Server.semanticTokensFull", tag.URI.Of(params.TextDocument.URI))
-	defer done()
-
-	ret, err := s.computeSemanticTokens(ctx, params.TextDocument, nil)
-	return ret, err
+func (s *server) SemanticTokensFull(ctx context.Context, params *protocol.SemanticTokensParams) (*protocol.SemanticTokens, error) {
+	return s.semanticTokens(ctx, params.TextDocument, nil)
 }
 
-func (s *Server) semanticTokensRange(ctx context.Context, params *protocol.SemanticTokensRangeParams) (*protocol.SemanticTokens, error) {
-	ctx, done := event.Start(ctx, "lsp.Server.semanticTokensRange", tag.URI.Of(params.TextDocument.URI))
-	defer done()
-
-	ret, err := s.computeSemanticTokens(ctx, params.TextDocument, &params.Range)
-	return ret, err
+func (s *server) SemanticTokensRange(ctx context.Context, params *protocol.SemanticTokensRangeParams) (*protocol.SemanticTokens, error) {
+	return s.semanticTokens(ctx, params.TextDocument, &params.Range)
 }
 
-func (s *Server) computeSemanticTokens(ctx context.Context, td protocol.TextDocumentIdentifier, rng *protocol.Range) (*protocol.SemanticTokens, error) {
-	ans := protocol.SemanticTokens{
-		Data: []uint32{},
-	}
-	snapshot, fh, ok, release, err := s.beginFileRequest(ctx, td.URI, source.UnknownKind)
+func (s *server) semanticTokens(ctx context.Context, td protocol.TextDocumentIdentifier, rng *protocol.Range) (*protocol.SemanticTokens, error) {
+	ctx, done := event.Start(ctx, "lsp.Server.semanticTokens", tag.URI.Of(td.URI))
+	defer done()
+
+	snapshot, fh, ok, release, err := s.beginFileRequest(ctx, td.URI, file.UnknownKind)
 	defer release()
 	if !ok {
 		return nil, err
@@ -68,18 +61,19 @@ func (s *Server) computeSemanticTokens(ctx context.Context, td protocol.TextDocu
 		// the client won't remember the wrong answer
 		return nil, fmt.Errorf("semantictokens are disabled")
 	}
+
 	kind := snapshot.FileKind(fh)
-	if kind == source.Tmpl {
+	if kind == file.Tmpl {
 		// this is a little cumbersome to avoid both exporting 'encoded' and its methods
 		// and to avoid import cycles
 		e := &encoded{
 			ctx:            ctx,
 			metadataSource: snapshot,
-			rng:            rng,
 			tokTypes:       snapshot.Options().SemanticTypes,
 			tokMods:        snapshot.Options().SemanticMods,
 		}
 		add := func(line, start uint32, len uint32) {
+			// TODO(adonovan): don't ignore the rng restriction, if any.
 			e.add(line, start, len, tokMacro, nil)
 		}
 		data := func() []uint32 {
@@ -87,7 +81,7 @@ func (s *Server) computeSemanticTokens(ctx context.Context, td protocol.TextDocu
 		}
 		return template.SemanticTokens(ctx, snapshot, fh.URI(), add, data)
 	}
-	if kind != source.Go {
+	if kind != file.Go {
 		return nil, nil
 	}
 	pkg, pgf, err := source.NarrowestPackageForFile(ctx, snapshot, fh.URI())
@@ -95,16 +89,30 @@ func (s *Server) computeSemanticTokens(ctx context.Context, td protocol.TextDocu
 		return nil, err
 	}
 
-	if rng == nil && len(pgf.Src) > maxFullFileSize {
-		err := fmt.Errorf("semantic tokens: file %s too large for full (%d>%d)",
-			fh.URI().Filename(), len(pgf.Src), maxFullFileSize)
+	// Select range.
+	var start, end token.Pos
+	if rng != nil {
+		var err error
+		start, end, err = pgf.RangePos(*rng)
+		if err != nil {
+			return nil, fmt.Errorf("range span (%w) error for %s", err, pgf.File.Name)
+		}
+	} else {
+		tok := pgf.Tok
+		start, end = tok.Pos(0), tok.Pos(tok.Size()) // entire file
+	}
+	if int(end-start) > maxFullFileSize {
+		err := fmt.Errorf("semantic tokens: range %s too large (%d > %d)",
+			fh.URI().Path(), end-start, maxFullFileSize)
 		return nil, err
 	}
+
 	e := &encoded{
 		ctx:            ctx,
 		metadataSource: snapshot,
 		pgf:            pgf,
-		rng:            rng,
+		start:          start,
+		end:            end,
 		ti:             pkg.GetTypesInfo(),
 		pkg:            pkg,
 		fset:           pkg.FileSet(),
@@ -113,16 +121,12 @@ func (s *Server) computeSemanticTokens(ctx context.Context, td protocol.TextDocu
 		noStrings:      snapshot.Options().NoSemanticString,
 		noNumbers:      snapshot.Options().NoSemanticNumber,
 	}
-	if err := e.init(); err != nil {
-		// e.init should never return an error, unless there's some
-		// seemingly impossible race condition
-		return nil, err
-	}
 	e.semantics()
-	ans.Data = e.Data()
-	// For delta requests, but we've never seen any.
-	ans.ResultID = fmt.Sprintf("%v", time.Now())
-	return &ans, nil
+	return &protocol.SemanticTokens{
+		Data: e.Data(),
+		// For delta requests, but we've never seen any.
+		ResultID: fmt.Sprintf("%v", time.Now()),
+	}, nil
 }
 
 func (e *encoded) semantics() {
@@ -224,13 +228,10 @@ type encoded struct {
 	metadataSource    source.MetadataSource
 	tokTypes, tokMods []string
 	pgf               *source.ParsedGoFile
-	rng               *protocol.Range
+	start, end        token.Pos // range of interest
 	ti                *types.Info
 	pkg               source.Package
 	fset              *token.FileSet
-	// allowed starting and ending token.Pos, set by init
-	// used to avoid looking at declarations not in range
-	start, end token.Pos
 	// path from the root of the parse tree, used for debugging
 	stack []ast.Node
 }
@@ -839,20 +840,6 @@ func (e *encoded) findKeyword(keyword string, start, end token.Pos) token.Pos {
 	return token.NoPos
 }
 
-func (e *encoded) init() error {
-	if e.rng != nil {
-		var err error
-		e.start, e.end, err = e.pgf.RangePos(*e.rng)
-		if err != nil {
-			return fmt.Errorf("range span (%w) error for %s", err, e.pgf.File.Name)
-		}
-	} else {
-		tok := e.pgf.Tok
-		e.start, e.end = tok.Pos(0), tok.Pos(tok.Size()) // entire file
-	}
-	return nil
-}
-
 func (e *encoded) Data() []uint32 {
 	// binary operators, at least, will be out of order
 	sort.Slice(e.items, func(i, j int) bool {
@@ -944,29 +931,6 @@ func (e *encoded) unexpected(msg string) {
 	event.Error(e.ctx, e.strStack(), errors.New(msg))
 }
 
-// SemType returns a string equivalent of the type, for gopls semtok
-func SemType(n int) string {
-	tokTypes := SemanticTypes()
-	tokMods := SemanticModifiers()
-	if n >= 0 && n < len(tokTypes) {
-		return tokTypes[n]
-	}
-	// not found for some reason
-	return fmt.Sprintf("?%d[%d,%d]?", n, len(tokTypes), len(tokMods))
-}
-
-// SemMods returns the []string equivalent of the mods, for gopls semtok.
-func SemMods(n int) []string {
-	tokMods := SemanticModifiers()
-	mods := []string{}
-	for i := 0; i < len(tokMods); i++ {
-		if (n & (1 << uint(i))) != 0 {
-			mods = append(mods, tokMods[i])
-		}
-	}
-	return mods
-}
-
 func (e *encoded) maps() (map[tokenType]int, map[string]int) {
 	tmap := make(map[tokenType]int)
 	mmap := make(map[string]int)
@@ -979,29 +943,6 @@ func (e *encoded) maps() (map[tokenType]int, map[string]int) {
 	return tmap, mmap
 }
 
-// SemanticTypes to use in case there is no client, as in the command line, or tests
-func SemanticTypes() []string {
-	return semanticTypes[:]
-}
-
-// SemanticModifiers to use in case there is no client.
-func SemanticModifiers() []string {
-	return semanticModifiers[:]
-}
-
-var (
-	semanticTypes = [...]string{
-		"namespace", "type", "class", "enum", "interface",
-		"struct", "typeParameter", "parameter", "variable", "property", "enumMember",
-		"event", "function", "method", "macro", "keyword", "modifier", "comment",
-		"string", "number", "regexp", "operator",
-	}
-	semanticModifiers = [...]string{
-		"declaration", "definition", "readonly", "static",
-		"deprecated", "abstract", "async", "modification", "documentation", "defaultLibrary",
-	}
-)
-
 var godirectives = map[string]struct{}{
 	// https://pkg.go.dev/cmd/compile
 	"noescape":       {},
