@@ -17,15 +17,16 @@ import (
 
 	"golang.org/x/tools/go/ast/astutil"
 	"golang.org/x/tools/gopls/internal/bug"
+	"golang.org/x/tools/gopls/internal/file"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/safetoken"
-	"golang.org/x/tools/gopls/internal/span"
 	"golang.org/x/tools/imports"
 	internalastutil "golang.org/x/tools/internal/astutil"
 	"golang.org/x/tools/internal/diff"
 	"golang.org/x/tools/internal/refactor/inline"
 	"golang.org/x/tools/internal/tokeninternal"
 	"golang.org/x/tools/internal/typesinternal"
+	"golang.org/x/tools/internal/versions"
 )
 
 // RemoveUnusedParameter computes a refactoring to remove the parameter
@@ -37,7 +38,7 @@ import (
 //   - Improve the extra newlines in output.
 //   - Stream type checking via ForEachPackage.
 //   - Avoid unnecessary additional type checking.
-func RemoveUnusedParameter(ctx context.Context, fh FileHandle, rng protocol.Range, snapshot Snapshot) ([]protocol.DocumentChanges, error) {
+func RemoveUnusedParameter(ctx context.Context, fh file.Handle, rng protocol.Range, snapshot Snapshot) ([]protocol.DocumentChanges, error) {
 	pkg, pgf, err := NarrowestPackageForFile(ctx, snapshot, fh.URI())
 	if err != nil {
 		return nil, err
@@ -163,7 +164,7 @@ func RemoveUnusedParameter(ctx context.Context, fh FileHandle, rng protocol.Rang
 		}
 		edits := diff.Bytes(before, after)
 		mapper := protocol.NewMapper(uri, before)
-		pedits, err := ToProtocolEdits(mapper, edits)
+		pedits, err := protocol.EditsFromDiffEdits(mapper, edits)
 		if err != nil {
 			return nil, fmt.Errorf("computing edits for %s: %v", uri, err)
 		}
@@ -171,7 +172,7 @@ func RemoveUnusedParameter(ctx context.Context, fh FileHandle, rng protocol.Rang
 			TextDocumentEdit: &protocol.TextDocumentEdit{
 				TextDocument: protocol.OptionalVersionedTextDocumentIdentifier{
 					Version:                fh.Version(),
-					TextDocumentIdentifier: protocol.TextDocumentIdentifier{URI: protocol.URIFromSpanURI(uri)},
+					TextDocumentIdentifier: protocol.TextDocumentIdentifier{URI: uri},
 				},
 				Edits: pedits,
 			},
@@ -352,7 +353,7 @@ type signatureRewrite struct {
 // By passing an entirely new declaration, rewriteCalls may be used for
 // signature refactorings that may affect the function body, such as removing
 // or adding return values.
-func rewriteCalls(ctx context.Context, rw signatureRewrite) (map[span.URI][]byte, error) {
+func rewriteCalls(ctx context.Context, rw signatureRewrite) (map[protocol.DocumentURI][]byte, error) {
 	// tag is a unique prefix that is added to the delegated declaration.
 	//
 	// It must have a ~0% probability of causing collisions with existing names.
@@ -404,7 +405,7 @@ func rewriteCalls(ctx context.Context, rw signatureRewrite) (map[span.URI][]byte
 		// by returning the modified AST from replaceDecl. Investigate if that is
 		// accurate.
 		modifiedSrc = append(modifiedSrc, []byte("\n\n"+FormatNode(fset, wrapper))...)
-		modifiedFile, err = parser.ParseFile(rw.pkg.FileSet(), rw.pgf.URI.Filename(), modifiedSrc, parser.ParseComments|parser.SkipObjectResolution)
+		modifiedFile, err = parser.ParseFile(rw.pkg.FileSet(), rw.pgf.URI.Path(), modifiedSrc, parser.ParseComments|parser.SkipObjectResolution)
 		if err != nil {
 			return nil, err
 		}
@@ -414,7 +415,7 @@ func rewriteCalls(ctx context.Context, rw signatureRewrite) (map[span.URI][]byte
 	// Type check pkg again with the modified file, to compute the synthetic
 	// callee.
 	logf := logger(ctx, "change signature", rw.snapshot.Options().VerboseOutput)
-	pkg2, info, err := reTypeCheck(logf, rw.pkg, map[span.URI]*ast.File{rw.pgf.URI: modifiedFile}, false)
+	pkg2, info, err := reTypeCheck(logf, rw.pkg, map[protocol.DocumentURI]*ast.File{rw.pgf.URI: modifiedFile}, false)
 	if err != nil {
 		return nil, err
 	}
@@ -435,7 +436,7 @@ func rewriteCalls(ctx context.Context, rw signatureRewrite) (map[span.URI][]byte
 // If expectErrors is true, reTypeCheck allows errors in the new package.
 // TODO(rfindley): perhaps this should be a filter to specify which errors are
 // acceptable.
-func reTypeCheck(logf func(string, ...any), orig Package, fileMask map[span.URI]*ast.File, expectErrors bool) (*types.Package, *types.Info, error) {
+func reTypeCheck(logf func(string, ...any), orig Package, fileMask map[protocol.DocumentURI]*ast.File, expectErrors bool) (*types.Package, *types.Info, error) {
 	pkg := types.NewPackage(string(orig.Metadata().PkgPath), string(orig.Metadata().Name))
 	info := &types.Info{
 		Types:      make(map[ast.Expr]types.TypeAndValue),
@@ -446,6 +447,7 @@ func reTypeCheck(logf func(string, ...any), orig Package, fileMask map[span.URI]
 		Scopes:     make(map[ast.Node]*types.Scope),
 		Instances:  make(map[*ast.Ident]types.Instance),
 	}
+	versions.InitFileVersions(info)
 	{
 		var files []*ast.File
 		for _, pgf := range orig.CompiledGoFiles() {
