@@ -17,7 +17,6 @@ import (
 	"golang.org/x/tools/gopls/internal/lsp/safetoken"
 	"golang.org/x/tools/gopls/internal/lsp/snippet"
 	"golang.org/x/tools/gopls/internal/lsp/source"
-	"golang.org/x/tools/gopls/internal/span"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/imports"
 	"golang.org/x/tools/internal/typeparams"
@@ -183,11 +182,18 @@ Suffixes:
 	if cand.convertTo != nil {
 		typeName := types.TypeString(cand.convertTo, c.qf)
 
-		switch cand.convertTo.(type) {
+		switch t := cand.convertTo.(type) {
 		// We need extra parens when casting to these types. For example,
 		// we need "(*int)(foo)", not "*int(foo)".
 		case *types.Pointer, *types.Signature:
 			typeName = "(" + typeName + ")"
+		case *types.Basic:
+			// If the types are incompatible (as determined by typeMatches), then we
+			// must need a conversion here. However, if the target type is untyped,
+			// don't suggest converting to e.g. "untyped float" (golang/go#62141).
+			if t.Info()&types.IsUntyped != 0 {
+				typeName = types.TypeString(types.Default(cand.convertTo), c.qf)
+			}
 		}
 
 		prefix = typeName + "(" + prefix
@@ -256,9 +262,9 @@ Suffixes:
 	// TODO(rfindley): It doesn't look like this does the right thing for
 	// multi-line comments.
 	if strings.HasPrefix(comment.Text(), "Deprecated") {
-		if c.snapshot.View().Options().CompletionTags {
+		if c.snapshot.Options().CompletionTags {
 			item.Tags = []protocol.CompletionItemTag{protocol.ComplDeprecated}
-		} else if c.snapshot.View().Options().CompletionDeprecated {
+		} else if c.snapshot.Options().CompletionDeprecated {
 			item.Deprecated = true
 		}
 	}
@@ -272,7 +278,7 @@ func (c *completer) importEdits(imp *importInfo) ([]protocol.TextEdit, error) {
 		return nil, nil
 	}
 
-	pgf, err := c.pkg.File(span.URIFromPath(c.filename))
+	pgf, err := c.pkg.File(protocol.URIFromPath(c.filename))
 	if err != nil {
 		return nil, err
 	}
