@@ -21,7 +21,7 @@ import (
 // literal generates composite literal, function literal, and make()
 // completion items.
 func (c *completer) literal(ctx context.Context, literalType types.Type, imp *importInfo) {
-	if !c.opts.literal {
+	if !c.opts.snippets {
 		return
 	}
 
@@ -129,7 +129,7 @@ func (c *completer) literal(ctx context.Context, literalType types.Type, imp *im
 			// Add a literal completion for a signature type that implements
 			// an interface. For example, offer "http.HandlerFunc()" when
 			// expected type is "http.Handler".
-			if source.IsInterface(expType) {
+			if expType != nil && types.IsInterface(expType) {
 				c.basicLiteral(t, snip.Clone(), typeName, float64(score), addlEdits)
 			}
 		case *types.Basic:
@@ -137,7 +137,7 @@ func (c *completer) literal(ctx context.Context, literalType types.Type, imp *im
 			// expected interface (e.g. named string type http.Dir
 			// implements http.FileSystem), or are identical to our expected
 			// type (i.e. yielding a type conversion such as "float64()").
-			if source.IsInterface(expType) || types.Identical(expType, literalType) {
+			if expType != nil && (types.IsInterface(expType) || types.Identical(expType, literalType)) {
 				c.basicLiteral(t, snip.Clone(), typeName, float64(score), addlEdits)
 			}
 		}
@@ -159,10 +159,10 @@ func (c *completer) literal(ctx context.Context, literalType types.Type, imp *im
 	}
 
 	// If prefix matches "func", client may want a function literal.
-	if score := c.matcher.Score("func"); !cand.hasMod(reference) && score > 0 && !source.IsInterface(expType) {
+	if score := c.matcher.Score("func"); !cand.hasMod(reference) && score > 0 && (expType == nil || !types.IsInterface(expType)) {
 		switch t := literalType.Underlying().(type) {
 		case *types.Signature:
-			c.functionLiteral(t, float64(score))
+			c.functionLiteral(ctx, t, float64(score))
 		}
 	}
 }
@@ -175,7 +175,7 @@ const literalCandidateScore = highScore / 2
 
 // functionLiteral adds a function literal completion item for the
 // given signature.
-func (c *completer) functionLiteral(sig *types.Signature, matchScore float64) {
+func (c *completer) functionLiteral(ctx context.Context, sig *types.Signature, matchScore float64) {
 	snip := &snippet.Builder{}
 	snip.WriteText("func(")
 
@@ -202,10 +202,18 @@ func (c *completer) functionLiteral(sig *types.Signature, matchScore float64) {
 			// If the param has no name in the signature, guess a name based
 			// on the type. Use an empty qualifier to ignore the package.
 			// For example, we want to name "http.Request" "r", not "hr".
-			name = source.FormatVarType(c.snapshot.FileSet(), c.pkg, p, func(p *types.Package) string {
-				return ""
-			})
-			name = abbreviateTypeName(name)
+			typeName, err := source.FormatVarType(ctx, c.snapshot, c.pkg, p,
+				func(p *types.Package) string { return "" },
+				func(source.PackageName, source.ImportPath, source.PackagePath) string { return "" })
+			if err != nil {
+				// In general, the only error we should encounter while formatting is
+				// context cancellation.
+				if ctx.Err() == nil {
+					event.Error(ctx, "formatting var type", err)
+				}
+				return
+			}
+			name = abbreviateTypeName(typeName)
 		}
 		paramNames[i] = name
 		if name != "_" {
@@ -264,7 +272,15 @@ func (c *completer) functionLiteral(sig *types.Signature, matchScore float64) {
 		// of "i int, j int".
 		if i == sig.Params().Len()-1 || !types.Identical(p.Type(), sig.Params().At(i+1).Type()) {
 			snip.WriteText(" ")
-			typeStr := source.FormatVarType(c.snapshot.FileSet(), c.pkg, p, c.qf)
+			typeStr, err := source.FormatVarType(ctx, c.snapshot, c.pkg, p, c.qf, c.mq)
+			if err != nil {
+				// In general, the only error we should encounter while formatting is
+				// context cancellation.
+				if ctx.Err() == nil {
+					event.Error(ctx, "formatting var type", err)
+				}
+				return
+			}
 			if sig.Variadic() && i == sig.Params().Len()-1 {
 				typeStr = strings.Replace(typeStr, "[]", "...", 1)
 			}
@@ -314,7 +330,15 @@ func (c *completer) functionLiteral(sig *types.Signature, matchScore float64) {
 			snip.WriteText(name + " ")
 		}
 
-		text := source.FormatVarType(c.snapshot.FileSet(), c.pkg, r, c.qf)
+		text, err := source.FormatVarType(ctx, c.snapshot, c.pkg, r, c.qf, c.mq)
+		if err != nil {
+			// In general, the only error we should encounter while formatting is
+			// context cancellation.
+			if ctx.Err() == nil {
+				event.Error(ctx, "formatting var type", err)
+			}
+			return
+		}
 		if tp, _ := r.Type().(*typeparams.TypeParam); tp != nil && !c.typeParamInScope(tp) {
 			snip.WritePlaceholder(func(snip *snippet.Builder) {
 				snip.WriteText(text)
