@@ -14,6 +14,8 @@ import (
 	"go/format"
 	"go/token"
 	"go/types"
+	"io/fs"
+	"path/filepath"
 	"strconv"
 	"strings"
 	"time"
@@ -22,11 +24,13 @@ import (
 	"golang.org/x/text/unicode/runenames"
 	"golang.org/x/tools/go/ast/astutil"
 	"golang.org/x/tools/go/types/typeutil"
+	"golang.org/x/tools/gopls/internal/bug"
+	"golang.org/x/tools/gopls/internal/file"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/safetoken"
-	"golang.org/x/tools/gopls/internal/span"
-	"golang.org/x/tools/internal/bug"
+	"golang.org/x/tools/gopls/internal/settings"
 	"golang.org/x/tools/internal/event"
+	"golang.org/x/tools/internal/tokeninternal"
 	"golang.org/x/tools/internal/typeparams"
 )
 
@@ -59,7 +63,7 @@ type HoverJSON struct {
 }
 
 // Hover implements the "textDocument/hover" RPC for Go files.
-func Hover(ctx context.Context, snapshot Snapshot, fh FileHandle, position protocol.Position) (*protocol.Hover, error) {
+func Hover(ctx context.Context, snapshot Snapshot, fh file.Handle, position protocol.Position) (*protocol.Hover, error) {
 	ctx, done := event.Start(ctx, "source.Hover")
 	defer done()
 
@@ -70,13 +74,13 @@ func Hover(ctx context.Context, snapshot Snapshot, fh FileHandle, position proto
 	if h == nil {
 		return nil, nil
 	}
-	hover, err := formatHover(h, snapshot.View().Options())
+	hover, err := formatHover(h, snapshot.Options())
 	if err != nil {
 		return nil, err
 	}
 	return &protocol.Hover{
 		Contents: protocol.MarkupContent{
-			Kind:  snapshot.View().Options().PreferredContentFormat,
+			Kind:  snapshot.Options().PreferredContentFormat,
 			Value: hover,
 		},
 		Range: rng,
@@ -86,8 +90,8 @@ func Hover(ctx context.Context, snapshot Snapshot, fh FileHandle, position proto
 // hover computes hover information at the given position. If we do not support
 // hovering at the position, it returns _, nil, nil: an error is only returned
 // if the position is valid but we fail to compute hover information.
-func hover(ctx context.Context, snapshot Snapshot, fh FileHandle, pp protocol.Position) (protocol.Range, *HoverJSON, error) {
-	pkg, pgf, err := PackageForFile(ctx, snapshot, fh.URI(), NarrowestPackage)
+func hover(ctx context.Context, snapshot Snapshot, fh file.Handle, pp protocol.Position) (protocol.Range, *HoverJSON, error) {
+	pkg, pgf, err := NarrowestPackageForFile(ctx, snapshot, fh.URI())
 	if err != nil {
 		return protocol.Range{}, nil, err
 	}
@@ -120,6 +124,28 @@ func hover(ctx context.Context, snapshot Snapshot, fh FileHandle, pp protocol.Po
 		}
 	}
 
+	// Handle hovering over embed directive argument.
+	pattern, embedRng := parseEmbedDirective(pgf.Mapper, pp)
+	if pattern != "" {
+		return hoverEmbed(fh, embedRng, pattern)
+	}
+
+	// Handle linkname directive by overriding what to look for.
+	var linkedRange *protocol.Range // range referenced by linkname directive, or nil
+	if pkgPath, name, offset := parseLinkname(pgf.Mapper, pp); pkgPath != "" && name != "" {
+		// rng covering 2nd linkname argument: pkgPath.name.
+		rng, err := pgf.PosRange(pgf.Tok.Pos(offset), pgf.Tok.Pos(offset+len(pkgPath)+len(".")+len(name)))
+		if err != nil {
+			return protocol.Range{}, nil, fmt.Errorf("range over linkname arg: %w", err)
+		}
+		linkedRange = &rng
+
+		pkg, pgf, pos, err = findLinkname(ctx, snapshot, PackagePath(pkgPath), name)
+		if err != nil {
+			return protocol.Range{}, nil, fmt.Errorf("find linkname: %w", err)
+		}
+	}
+
 	// The general case: compute hover information for the object referenced by
 	// the identifier at pos.
 	ident, obj, selectedType := referencedObject(pkg, pgf, pos)
@@ -127,9 +153,15 @@ func hover(ctx context.Context, snapshot Snapshot, fh FileHandle, pp protocol.Po
 		return protocol.Range{}, nil, nil // no object to hover
 	}
 
-	rng, err := pgf.NodeRange(ident)
-	if err != nil {
-		return protocol.Range{}, nil, err
+	// Unless otherwise specified, rng covers the ident being hovered.
+	var rng protocol.Range
+	if linkedRange != nil {
+		rng = *linkedRange
+	} else {
+		rng, err = pgf.NodeRange(ident)
+		if err != nil {
+			return protocol.Range{}, nil, err
+		}
 	}
 
 	// By convention, we qualify hover information relative to the package
@@ -142,7 +174,7 @@ func hover(ctx context.Context, snapshot Snapshot, fh FileHandle, pp protocol.Po
 	// There's not much useful information to provide.
 	if selectedType != nil {
 		fakeObj := types.NewVar(obj.Pos(), obj.Pkg(), obj.Name(), selectedType)
-		signature := objectString(fakeObj, qf, nil)
+		signature := types.ObjectString(fakeObj, qf)
 		return rng, &HoverJSON{
 			Signature:  signature,
 			SingleLine: signature,
@@ -167,10 +199,14 @@ func hover(ctx context.Context, snapshot Snapshot, fh FileHandle, pp protocol.Po
 	docText := comment.Text()
 
 	// By default, types.ObjectString provides a reasonable signature.
-	signature := objectString(obj, qf, nil)
+	signature := objectString(obj, qf, declPos, declPGF.Tok, spec)
+	singleLineSignature := signature
+
 	// TODO(rfindley): we could do much better for inferred signatures.
 	if inferred := inferredSignature(pkg.GetTypesInfo(), ident); inferred != nil {
-		signature = objectString(obj, qf, inferred)
+		if s := inferredSignatureString(obj, qf, inferred); s != "" {
+			signature = s
+		}
 	}
 
 	// For "objects defined by a type spec", the signature produced by
@@ -195,7 +231,7 @@ func hover(ctx context.Context, snapshot Snapshot, fh FileHandle, pp protocol.Po
 		spec2.Comment = nil
 		var b strings.Builder
 		b.WriteString("type ")
-		fset := FileSetFor(declPGF.Tok)
+		fset := tokeninternal.FileSetFor(declPGF.Tok)
 		if err := format.Node(&b, fset, &spec2); err != nil {
 			return protocol.Range{}, nil, err
 		}
@@ -213,7 +249,7 @@ func hover(ctx context.Context, snapshot Snapshot, fh FileHandle, pp protocol.Po
 				if (m.Obj().Exported() || m.Obj().Pkg() == pkg.GetTypes()) && len(m.Index()) == 1 {
 					b.WriteString(sep)
 					sep = "\n"
-					b.WriteString(objectString(m.Obj(), qf, nil))
+					b.WriteString(types.ObjectString(m.Obj(), qf))
 				}
 			}
 		}
@@ -310,7 +346,7 @@ func hover(ctx context.Context, snapshot Snapshot, fh FileHandle, pp protocol.Po
 		}
 	}
 
-	if snapshot.View().IsGoPrivatePath(linkPath) || linkMeta.ForTest != "" {
+	if snapshot.IsGoPrivatePath(linkPath) || linkMeta.ForTest != "" {
 		linkPath = ""
 	} else if linkMeta.Module != nil && linkMeta.Module.Version != "" {
 		mod := linkMeta.Module
@@ -320,7 +356,7 @@ func hover(ctx context.Context, snapshot Snapshot, fh FileHandle, pp protocol.Po
 	return rng, &HoverJSON{
 		Synopsis:          doc.Synopsis(docText),
 		FullDocumentation: docText,
-		SingleLine:        objectString(obj, qf, nil),
+		SingleLine:        singleLineSignature,
 		SymbolName:        linkName,
 		Signature:         signature,
 		LinkPath:          linkPath,
@@ -337,8 +373,6 @@ func hoverBuiltin(ctx context.Context, snapshot Snapshot, obj types.Object) (*Ho
 		return nil, err
 	}
 
-	// TODO(rfindley): add a test for jump to definition of error.Error (which is
-	// probably failing, considering it lacks special handling).
 	if obj.Name() == "Error" {
 		signature := obj.String()
 		return &HoverJSON{
@@ -424,7 +458,7 @@ func hoverImport(ctx context.Context, snapshot Snapshot, pkg Package, pgf *Parse
 	// Find the first file with a package doc comment.
 	var comment *ast.CommentGroup
 	for _, f := range impMetadata.CompiledGoFiles {
-		fh, err := snapshot.GetFile(ctx, f)
+		fh, err := snapshot.ReadFile(ctx, f)
 		if err != nil {
 			if ctx.Err() != nil {
 				return protocol.Range{}, nil, ctx.Err()
@@ -482,8 +516,11 @@ func hoverPackageName(pkg Package, pgf *ParsedGoFile) (protocol.Range, *HoverJSO
 //
 //	'∑', U+2211, N-ARY SUMMATION
 func hoverLit(pgf *ParsedGoFile, lit *ast.BasicLit, pos token.Pos) (protocol.Range, *HoverJSON, error) {
-	var r rune
-	var start, end token.Pos
+	var (
+		value      string    // if non-empty, a constant value to format in hover
+		r          rune      // if non-zero, format a description of this rune in hover
+		start, end token.Pos // hover span
+	)
 	// Extract a rune from the current position.
 	// 'Ω', "...Ω...", or 0x03A9 => 'Ω', U+03A9, GREEK CAPITAL LETTER OMEGA
 	switch lit.Kind {
@@ -499,23 +536,34 @@ func hoverLit(pgf *ParsedGoFile, lit *ast.BasicLit, pos token.Pos) (protocol.Ran
 			return protocol.Range{}, nil, fmt.Errorf("rune error")
 		}
 		start, end = lit.Pos(), lit.End()
-	case token.INT:
-		// TODO(rfindley): add support for hex/octal/binary->int conversion here.
 
-		// It's an integer, scan only if it is a hex literal whose bitsize in
-		// ranging from 8 to 32.
-		if !(strings.HasPrefix(lit.Value, "0x") && len(lit.Value[2:]) >= 2 && len(lit.Value[2:]) <= 8) {
+	case token.INT:
+		// Short literals (e.g. 99 decimal, 07 octal) are uninteresting.
+		if len(lit.Value) < 3 {
 			return protocol.Range{}, nil, nil
 		}
-		v, err := strconv.ParseUint(lit.Value[2:], 16, 32)
-		if err != nil {
-			return protocol.Range{}, nil, fmt.Errorf("parsing int: %v", err)
+
+		v := constant.MakeFromLiteral(lit.Value, lit.Kind, 0)
+		if v.Kind() != constant.Int {
+			return protocol.Range{}, nil, nil
 		}
-		r = rune(v)
-		if r == utf8.RuneError {
-			return protocol.Range{}, nil, fmt.Errorf("rune error")
+
+		switch lit.Value[:2] {
+		case "0x", "0X":
+			// As a special case, try to recognize hexadecimal literals as runes if
+			// they are within the range of valid unicode values.
+			if v, ok := constant.Int64Val(v); ok && v > 0 && v <= utf8.MaxRune && utf8.ValidRune(rune(v)) {
+				r = rune(v)
+			}
+			fallthrough
+		case "0o", "0O", "0b", "0B":
+			// Format the decimal value of non-decimal literals.
+			value = v.ExactString()
+			start, end = lit.Pos(), lit.End()
+		default:
+			return protocol.Range{}, nil, nil
 		}
-		start, end = lit.Pos(), lit.End()
+
 	case token.STRING:
 		// It's a string, scan only if it contains a unicode escape sequence under or before the
 		// current cursor position.
@@ -550,38 +598,88 @@ func hoverLit(pgf *ParsedGoFile, lit *ast.BasicLit, pos token.Pos) (protocol.Ran
 			}
 		}
 	}
-	if r == 0 {
+
+	if value == "" && r == 0 { // nothing to format
 		return protocol.Range{}, nil, nil
 	}
+
 	rng, err := pgf.PosRange(start, end)
 	if err != nil {
 		return protocol.Range{}, nil, err
 	}
 
-	var desc string
-	runeName := runenames.Name(r)
-	if len(runeName) > 0 && runeName[0] == '<' {
-		// Check if the rune looks like an HTML tag. If so, trim the surrounding <>
-		// characters to work around https://github.com/microsoft/vscode/issues/124042.
-		runeName = strings.TrimRight(runeName[1:], ">")
+	var b strings.Builder
+	if value != "" {
+		b.WriteString(value)
 	}
-	if strconv.IsPrint(r) {
-		desc = fmt.Sprintf("'%s', U+%04X, %s", string(r), uint32(r), runeName)
-	} else {
-		desc = fmt.Sprintf("U+%04X, %s", uint32(r), runeName)
+	if r != 0 {
+		runeName := runenames.Name(r)
+		if len(runeName) > 0 && runeName[0] == '<' {
+			// Check if the rune looks like an HTML tag. If so, trim the surrounding <>
+			// characters to work around https://github.com/microsoft/vscode/issues/124042.
+			runeName = strings.TrimRight(runeName[1:], ">")
+		}
+		if b.Len() > 0 {
+			b.WriteString(", ")
+		}
+		if strconv.IsPrint(r) {
+			fmt.Fprintf(&b, "'%c', ", r)
+		}
+		fmt.Fprintf(&b, "U+%04X, %s", r, runeName)
 	}
+	hover := b.String()
 	return rng, &HoverJSON{
-		Synopsis:          desc,
-		FullDocumentation: desc,
+		Synopsis:          hover,
+		FullDocumentation: hover,
 	}, nil
 }
 
-// objectString is a wrapper around the types.ObjectString function.
-// It handles adding more information to the object string.
-//
-// TODO(rfindley): this function does too much. We should lift the special
-// handling to callsites.
-func objectString(obj types.Object, qf types.Qualifier, inferred *types.Signature) string {
+// hoverEmbed computes hover information for a filepath.Match pattern.
+// Assumes that the pattern is relative to the location of fh.
+func hoverEmbed(fh file.Handle, rng protocol.Range, pattern string) (protocol.Range, *HoverJSON, error) {
+	s := &strings.Builder{}
+
+	dir := filepath.Dir(fh.URI().Path())
+	var matches []string
+	err := filepath.WalkDir(dir, func(abs string, d fs.DirEntry, e error) error {
+		if e != nil {
+			return e
+		}
+		rel, err := filepath.Rel(dir, abs)
+		if err != nil {
+			return err
+		}
+		ok, err := filepath.Match(pattern, rel)
+		if err != nil {
+			return err
+		}
+		if ok && !d.IsDir() {
+			matches = append(matches, rel)
+		}
+		return nil
+	})
+	if err != nil {
+		return protocol.Range{}, nil, err
+	}
+
+	for _, m := range matches {
+		// TODO: Renders each file as separate markdown paragraphs.
+		// If forcing (a single) newline is possible it might be more clear.
+		fmt.Fprintf(s, "%s\n\n", m)
+	}
+
+	json := &HoverJSON{
+		Signature:         fmt.Sprintf("Embedding %q", pattern),
+		Synopsis:          s.String(),
+		FullDocumentation: s.String(),
+	}
+	return rng, json, nil
+}
+
+// inferredSignatureString is a wrapper around the types.ObjectString function
+// that adds more information to inferred signatures. It will return an empty string
+// if the passed types.Object is not a signature.
+func inferredSignatureString(obj types.Object, qf types.Qualifier, inferred *types.Signature) string {
 	// If the signature type was inferred, prefer the inferred signature with a
 	// comment showing the generic signature.
 	if sig, _ := obj.Type().(*types.Signature); sig != nil && typeparams.ForSignature(sig).Len() > 0 && inferred != nil {
@@ -596,22 +694,65 @@ func objectString(obj types.Object, qf types.Qualifier, inferred *types.Signatur
 		str += "// " + types.TypeString(sig, qf)
 		return str
 	}
+	return ""
+}
+
+// objectString is a wrapper around the types.ObjectString function.
+// It handles adding more information to the object string.
+// If spec is non-nil, it may be used to format additional declaration
+// syntax, and file must be the token.File describing its positions.
+func objectString(obj types.Object, qf types.Qualifier, declPos token.Pos, file *token.File, spec ast.Spec) string {
 	str := types.ObjectString(obj, qf)
+
 	switch obj := obj.(type) {
 	case *types.Const:
-		str = fmt.Sprintf("%s = %s", str, obj.Val())
+		var (
+			declaration = obj.Val().String() // default formatted declaration
+			comment     = ""                 // if non-empty, a clarifying comment
+		)
 
-		// Try to add a formatted duration as an inline comment
-		typ, ok := obj.Type().(*types.Named)
-		if !ok {
-			break
-		}
-		pkg := typ.Obj().Pkg()
-		if pkg.Path() == "time" && typ.Obj().Name() == "Duration" {
-			if d, ok := constant.Int64Val(obj.Val()); ok {
-				str += " // " + time.Duration(d).String()
+		// Try to use the original declaration.
+		switch obj.Val().Kind() {
+		case constant.String:
+			// Usually the original declaration of a string doesn't carry much information.
+			// Also strings can be very long. So, just use the constant's value.
+
+		default:
+			if spec, _ := spec.(*ast.ValueSpec); spec != nil {
+				for i, name := range spec.Names {
+					if declPos == name.Pos() {
+						if i < len(spec.Values) {
+							originalDeclaration := FormatNodeFile(file, spec.Values[i])
+							if originalDeclaration != declaration {
+								comment = declaration
+								declaration = originalDeclaration
+							}
+						}
+						break
+					}
+				}
 			}
 		}
+
+		// Special formatting cases.
+		switch typ := obj.Type().(type) {
+		case *types.Named:
+			// Try to add a formatted duration as an inline comment.
+			pkg := typ.Obj().Pkg()
+			if pkg.Path() == "time" && typ.Obj().Name() == "Duration" {
+				if d, ok := constant.Int64Val(obj.Val()); ok {
+					comment = time.Duration(d).String()
+				}
+			}
+		}
+		if comment == declaration {
+			comment = ""
+		}
+
+		str += " = " + declaration
+		if comment != "" {
+			str += " // " + comment
+		}
 	}
 	return str
 }
@@ -683,8 +824,8 @@ func parseFull(ctx context.Context, snapshot Snapshot, fset *token.FileSet, pos
 		return nil, 0, bug.Errorf("internal error: no file for position %d", pos)
 	}
 
-	uri := span.URIFromPath(f.Name())
-	fh, err := snapshot.GetFile(ctx, uri)
+	uri := protocol.URIFromPath(f.Name())
+	fh, err := snapshot.ReadFile(ctx, uri)
 	if err != nil {
 		return nil, 0, err
 	}
@@ -707,37 +848,15 @@ func parseFull(ctx context.Context, snapshot Snapshot, fset *token.FileSet, pos
 	return pgf, fullPos, nil
 }
 
-// extractFieldList recursively tries to extract a field list.
-// If it is not found, nil is returned.
-func extractFieldList(specType ast.Expr) *ast.FieldList {
-	switch t := specType.(type) {
-	case *ast.StructType:
-		return t.Fields
-	case *ast.InterfaceType:
-		return t.Methods
-	case *ast.ArrayType:
-		return extractFieldList(t.Elt)
-	case *ast.MapType:
-		// Map value has a greater chance to be a struct
-		if fields := extractFieldList(t.Value); fields != nil {
-			return fields
-		}
-		return extractFieldList(t.Key)
-	case *ast.ChanType:
-		return extractFieldList(t.Value)
-	}
-	return nil
-}
-
-func formatHover(h *HoverJSON, options *Options) (string, error) {
+func formatHover(h *HoverJSON, options *settings.Options) (string, error) {
 	signature := formatSignature(h, options)
 
 	switch options.HoverKind {
-	case SingleLine:
+	case settings.SingleLine:
 		return h.SingleLine, nil
-	case NoDocumentation:
+	case settings.NoDocumentation:
 		return signature, nil
-	case Structured:
+	case settings.Structured:
 		b, err := json.Marshal(h)
 		if err != nil {
 			return "", err
@@ -768,7 +887,7 @@ func formatHover(h *HoverJSON, options *Options) (string, error) {
 	return b.String(), nil
 }
 
-func formatSignature(h *HoverJSON, options *Options) string {
+func formatSignature(h *HoverJSON, options *settings.Options) string {
 	signature := h.Signature
 	if signature != "" && options.PreferredContentFormat == protocol.Markdown {
 		signature = fmt.Sprintf("```go\n%s\n```", signature)
@@ -776,7 +895,7 @@ func formatSignature(h *HoverJSON, options *Options) string {
 	return signature
 }
 
-func formatLink(h *HoverJSON, options *Options) string {
+func formatLink(h *HoverJSON, options *settings.Options) string {
 	if !options.LinksInHover || options.LinkTarget == "" || h.LinkPath == "" {
 		return ""
 	}
@@ -800,12 +919,12 @@ func BuildLink(target, path, anchor string) string {
 	return link + "#" + anchor
 }
 
-func formatDoc(h *HoverJSON, options *Options) string {
+func formatDoc(h *HoverJSON, options *settings.Options) string {
 	var doc string
 	switch options.HoverKind {
-	case SynopsisDocumentation:
+	case settings.SynopsisDocumentation:
 		doc = h.Synopsis
-	case FullDocumentation:
+	case settings.FullDocumentation:
 		doc = h.FullDocumentation
 	}
 	if options.PreferredContentFormat == protocol.Markdown {
