@@ -60,9 +60,9 @@ import (
 	"golang.org/x/tools/go/types/objectpath"
 	"golang.org/x/tools/go/types/typeutil"
 	"golang.org/x/tools/gopls/internal/bug"
+	"golang.org/x/tools/gopls/internal/file"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/gopls/internal/lsp/safetoken"
-	"golang.org/x/tools/gopls/internal/span"
 	"golang.org/x/tools/internal/diff"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/typeparams"
@@ -95,7 +95,7 @@ type PrepareItem struct {
 // The returned usererr is intended to be displayed to the user to explain why
 // the prepare fails. Probably we could eliminate the redundancy in returning
 // two errors, but for now this is done defensively.
-func PrepareRename(ctx context.Context, snapshot Snapshot, f FileHandle, pp protocol.Position) (_ *PrepareItem, usererr, err error) {
+func PrepareRename(ctx context.Context, snapshot Snapshot, f file.Handle, pp protocol.Position) (_ *PrepareItem, usererr, err error) {
 	ctx, done := event.Start(ctx, "source.PrepareRename")
 	defer done()
 
@@ -213,7 +213,7 @@ func checkRenamable(obj types.Object) error {
 // Rename returns a map of TextEdits for each file modified when renaming a
 // given identifier within a package and a boolean value of true for renaming
 // package and false otherwise.
-func Rename(ctx context.Context, snapshot Snapshot, f FileHandle, pp protocol.Position, newName string) (map[span.URI][]protocol.TextEdit, bool, error) {
+func Rename(ctx context.Context, snapshot Snapshot, f file.Handle, pp protocol.Position, newName string) (map[protocol.DocumentURI][]protocol.TextEdit, bool, error) {
 	ctx, done := event.Start(ctx, "source.Rename")
 	defer done()
 
@@ -227,7 +227,7 @@ func Rename(ctx context.Context, snapshot Snapshot, f FileHandle, pp protocol.Po
 		return nil, false, err
 	}
 
-	var editMap map[span.URI][]diff.Edit
+	var editMap map[protocol.DocumentURI][]diff.Edit
 	if inPackageName {
 		editMap, err = renamePackageName(ctx, snapshot, f, PackageName(newName))
 	} else {
@@ -238,7 +238,7 @@ func Rename(ctx context.Context, snapshot Snapshot, f FileHandle, pp protocol.Po
 	}
 
 	// Convert edits to protocol form.
-	result := make(map[span.URI][]protocol.TextEdit)
+	result := make(map[protocol.DocumentURI][]protocol.TextEdit)
 	for uri, edits := range editMap {
 		// Sort and de-duplicate edits.
 		//
@@ -276,7 +276,7 @@ func Rename(ctx context.Context, snapshot Snapshot, f FileHandle, pp protocol.Po
 			return nil, false, err
 		}
 		m := protocol.NewMapper(uri, data)
-		protocolEdits, err := ToProtocolEdits(m, edits)
+		protocolEdits, err := protocol.EditsFromDiffEdits(m, edits)
 		if err != nil {
 			return nil, false, err
 		}
@@ -287,7 +287,7 @@ func Rename(ctx context.Context, snapshot Snapshot, f FileHandle, pp protocol.Po
 }
 
 // renameOrdinary renames an ordinary (non-package) name throughout the workspace.
-func renameOrdinary(ctx context.Context, snapshot Snapshot, f FileHandle, pp protocol.Position, newName string) (map[span.URI][]diff.Edit, error) {
+func renameOrdinary(ctx context.Context, snapshot Snapshot, f file.Handle, pp protocol.Position, newName string) (map[protocol.DocumentURI][]diff.Edit, error) {
 	// Type-check the referring package and locate the object(s).
 	//
 	// Unlike NarrowestPackageForFile, this operation prefers the
@@ -387,7 +387,7 @@ func renameOrdinary(ctx context.Context, snapshot Snapshot, f FileHandle, pp pro
 		for obj := range targets {
 			objects = append(objects, obj)
 		}
-		editMap, _, err := renameObjects(ctx, snapshot, newName, pkg, objects...)
+		editMap, _, err := renameObjects(newName, pkg, objects...)
 		return editMap, err
 	}
 
@@ -437,7 +437,7 @@ func renameOrdinary(ctx context.Context, snapshot Snapshot, f FileHandle, pp pro
 	}
 
 	// Type-check all the packages to inspect.
-	declURI := span.URIFromPath(pkg.FileSet().File(obj.Pos()).Name())
+	declURI := protocol.URIFromPath(pkg.FileSet().File(obj.Pos()).Name())
 	pkgs, err := typeCheckReverseDependencies(ctx, snapshot, declURI, transitive)
 	if err != nil {
 		return nil, err
@@ -445,7 +445,7 @@ func renameOrdinary(ctx context.Context, snapshot Snapshot, f FileHandle, pp pro
 
 	// Apply the renaming to the (initial) object.
 	declPkgPath := PackagePath(obj.Pkg().Path())
-	return renameExported(ctx, snapshot, pkgs, declPkgPath, declObjPath, newName)
+	return renameExported(pkgs, declPkgPath, declObjPath, newName)
 }
 
 // funcOrigin is a go1.18-portable implementation of (*types.Func).Origin.
@@ -476,7 +476,7 @@ func funcOrigin(fn *types.Func) *types.Func {
 // (This neglects obscure edge cases where a _test.go file changes the
 // selectors used only in an ITV, but life is short. Also sin must be
 // punished.)
-func typeCheckReverseDependencies(ctx context.Context, snapshot Snapshot, declURI span.URI, transitive bool) ([]Package, error) {
+func typeCheckReverseDependencies(ctx context.Context, snapshot Snapshot, declURI protocol.DocumentURI, transitive bool) ([]Package, error) {
 	variants, err := snapshot.MetadataForFile(ctx, declURI)
 	if err != nil {
 		return nil, err
@@ -549,7 +549,7 @@ func SortPostOrder(meta MetadataSource, ids []PackageID) {
 // within the specified packages, along with any other objects that
 // must be renamed as a consequence. The slice of packages must be
 // topologically ordered.
-func renameExported(ctx context.Context, snapshot Snapshot, pkgs []Package, declPkgPath PackagePath, declObjPath objectpath.Path, newName string) (map[span.URI][]diff.Edit, error) {
+func renameExported(pkgs []Package, declPkgPath PackagePath, declObjPath objectpath.Path, newName string) (map[protocol.DocumentURI][]diff.Edit, error) {
 
 	// A target is a name for an object that is stable across types.Packages.
 	type target struct {
@@ -575,7 +575,7 @@ func renameExported(ctx context.Context, snapshot Snapshot, pkgs []Package, decl
 	targets := map[target]bool{{declPkgPath, declObjPath}: true}
 
 	// Apply the renaming operation to each package.
-	allEdits := make(map[span.URI][]diff.Edit)
+	allEdits := make(map[protocol.DocumentURI][]diff.Edit)
 	for _, pkg := range pkgs {
 
 		// Resolved target objects within package pkg.
@@ -605,7 +605,7 @@ func renameExported(ctx context.Context, snapshot Snapshot, pkgs []Package, decl
 		}
 
 		// Apply the renaming.
-		editMap, moreObjects, err := renameObjects(ctx, snapshot, newName, pkg, objects...)
+		editMap, moreObjects, err := renameObjects(newName, pkg, objects...)
 		if err != nil {
 			return nil, err
 		}
@@ -638,7 +638,7 @@ func renameExported(ctx context.Context, snapshot Snapshot, pkgs []Package, decl
 }
 
 // renamePackageName renames package declarations, imports, and go.mod files.
-func renamePackageName(ctx context.Context, s Snapshot, f FileHandle, newName PackageName) (map[span.URI][]diff.Edit, error) {
+func renamePackageName(ctx context.Context, s Snapshot, f file.Handle, newName PackageName) (map[protocol.DocumentURI][]diff.Edit, error) {
 	// Rename the package decl and all imports.
 	renamingEdits, err := renamePackage(ctx, s, f, newName)
 	if err != nil {
@@ -646,7 +646,7 @@ func renamePackageName(ctx context.Context, s Snapshot, f FileHandle, newName Pa
 	}
 
 	// Update the last component of the file's enclosing directory.
-	oldBase := filepath.Dir(f.URI().Filename())
+	oldBase := filepath.Dir(f.URI().Path())
 	newPkgDir := filepath.Join(filepath.Dir(oldBase), string(newName))
 
 	// Update any affected replace directives in go.mod files.
@@ -666,7 +666,7 @@ func renamePackageName(ctx context.Context, s Snapshot, f FileHandle, newName Pa
 			return nil, err
 		}
 
-		modFileDir := filepath.Dir(pm.URI.Filename())
+		modFileDir := filepath.Dir(pm.URI.Path())
 		affectedReplaces := []*modfile.Replace{}
 
 		// Check if any replace directives need to be fixed
@@ -702,7 +702,7 @@ func renamePackageName(ctx context.Context, s Snapshot, f FileHandle, newName Pa
 				replacedPath = filepath.Join(modFileDir, r.New.Path)
 			}
 
-			suffix := strings.TrimPrefix(replacedPath, string(oldBase))
+			suffix := strings.TrimPrefix(replacedPath, oldBase)
 
 			newReplacedPath, err := filepath.Rel(modFileDir, newPkgDir+suffix)
 			if err != nil {
@@ -741,7 +741,7 @@ func renamePackageName(ctx context.Context, s Snapshot, f FileHandle, newName Pa
 // It updates package clauses and import paths for the renamed package as well
 // as any other packages affected by the directory renaming among all packages
 // known to the snapshot.
-func renamePackage(ctx context.Context, s Snapshot, f FileHandle, newName PackageName) (map[span.URI][]diff.Edit, error) {
+func renamePackage(ctx context.Context, s Snapshot, f file.Handle, newName PackageName) (map[protocol.DocumentURI][]diff.Edit, error) {
 	if strings.HasSuffix(string(newName), "_test") {
 		return nil, fmt.Errorf("cannot rename to _test package")
 	}
@@ -774,7 +774,7 @@ func renamePackage(ctx context.Context, s Snapshot, f FileHandle, newName Packag
 	}
 
 	// Rename package and import declarations in all relevant packages.
-	edits := make(map[span.URI][]diff.Edit)
+	edits := make(map[protocol.DocumentURI][]diff.Edit)
 	for _, m := range allMetadata {
 		// Special case: x_test packages for the renamed package will not have the
 		// package path as a dir prefix, but still need their package clauses
@@ -808,7 +808,7 @@ func renamePackage(ctx context.Context, s Snapshot, f FileHandle, newName Packag
 
 		pkgName := m.Name
 		if m.PkgPath == oldPkgPath {
-			pkgName = PackageName(newName)
+			pkgName = newName
 
 			if err := renamePackageClause(ctx, m, s, newName, edits); err != nil {
 				return nil, err
@@ -828,7 +828,7 @@ func renamePackage(ctx context.Context, s Snapshot, f FileHandle, newName Packag
 // the package described by the given metadata, to newName.
 //
 // Edits are written into the edits map.
-func renamePackageClause(ctx context.Context, m *Metadata, snapshot Snapshot, newName PackageName, edits map[span.URI][]diff.Edit) error {
+func renamePackageClause(ctx context.Context, m *Metadata, snapshot Snapshot, newName PackageName, edits map[protocol.DocumentURI][]diff.Edit) error {
 	// Rename internal references to the package in the renaming package.
 	for _, uri := range m.CompiledGoFiles {
 		fh, err := snapshot.ReadFile(ctx, uri)
@@ -858,14 +858,14 @@ func renamePackageClause(ctx context.Context, m *Metadata, snapshot Snapshot, ne
 // newPath and name newName.
 //
 // Edits are written into the edits map.
-func renameImports(ctx context.Context, snapshot Snapshot, m *Metadata, newPath ImportPath, newName PackageName, allEdits map[span.URI][]diff.Edit) error {
+func renameImports(ctx context.Context, snapshot Snapshot, m *Metadata, newPath ImportPath, newName PackageName, allEdits map[protocol.DocumentURI][]diff.Edit) error {
 	rdeps, err := snapshot.ReverseDependencies(ctx, m.ID, false) // find direct importers
 	if err != nil {
 		return err
 	}
 
 	// Pass 1: rename import paths in import declarations.
-	needsTypeCheck := make(map[PackageID][]span.URI)
+	needsTypeCheck := make(map[PackageID][]protocol.DocumentURI)
 	for _, rdep := range rdeps {
 		if rdep.IsIntermediateTestVariant() {
 			continue // for renaming, these variants are redundant
@@ -970,7 +970,7 @@ func renameImports(ctx context.Context, snapshot Snapshot, m *Metadata, newPath
 				//   become shadowed by an intervening declaration that
 				//   uses the new name.
 				// It returns the edits if no conflict was detected.
-				editMap, _, err := renameObjects(ctx, snapshot, localName, pkg, pkgname)
+				editMap, _, err := renameObjects(localName, pkg, pkgname)
 				if err != nil {
 					return err
 				}
@@ -1004,7 +1004,7 @@ func renameImports(ctx context.Context, snapshot Snapshot, m *Metadata, newPath
 // consequence of the requested renamings.
 //
 // It returns an error if the renaming would cause a conflict.
-func renameObjects(ctx context.Context, snapshot Snapshot, newName string, pkg Package, targets ...types.Object) (map[span.URI][]diff.Edit, map[types.Object]bool, error) {
+func renameObjects(newName string, pkg Package, targets ...types.Object) (map[protocol.DocumentURI][]diff.Edit, map[types.Object]bool, error) {
 	r := renamer{
 		pkg:          pkg,
 		objsToUpdate: make(map[types.Object]bool),
@@ -1048,8 +1048,8 @@ func renameObjects(ctx context.Context, snapshot Snapshot, newName string, pkg P
 }
 
 // Rename all references to the target objects.
-func (r *renamer) update() (map[span.URI][]diff.Edit, error) {
-	result := make(map[span.URI][]diff.Edit)
+func (r *renamer) update() (map[protocol.DocumentURI][]diff.Edit, error) {
+	result := make(map[protocol.DocumentURI][]diff.Edit)
 
 	// shouldUpdate reports whether obj is one of (or an
 	// instantiation of one of) the target objects.
@@ -1147,7 +1147,7 @@ func (r *renamer) update() (map[span.URI][]diff.Edit, error) {
 			lines := strings.Split(comment.Text, "\n")
 			tokFile := pgf.Tok
 			commentLine := safetoken.Line(tokFile, comment.Pos())
-			uri := span.URIFromPath(tokFile.Name())
+			uri := protocol.URIFromPath(tokFile.Name())
 			for i, line := range lines {
 				lineStart := comment.Pos()
 				if i > 0 {
@@ -1246,7 +1246,7 @@ func (r *renamer) updatePkgName(pgf *ParsedGoFile, pkgName *types.PkgName) (diff
 // whether the position ppos lies within it.
 //
 // Note: also used by references.
-func parsePackageNameDecl(ctx context.Context, snapshot Snapshot, fh FileHandle, ppos protocol.Position) (*ParsedGoFile, bool, error) {
+func parsePackageNameDecl(ctx context.Context, snapshot Snapshot, fh file.Handle, ppos protocol.Position) (*ParsedGoFile, bool, error) {
 	pgf, err := snapshot.ParseGo(ctx, fh, ParseHeader)
 	if err != nil {
 		return nil, false, err
