@@ -7,7 +7,6 @@ package source
 import (
 	"context"
 	"fmt"
-	"go/types"
 	"path"
 	"path/filepath"
 	"regexp"
@@ -17,7 +16,7 @@ import (
 	"unicode"
 
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
-	"golang.org/x/tools/gopls/internal/span"
+	"golang.org/x/tools/gopls/internal/settings"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/fuzzy"
 )
@@ -52,7 +51,7 @@ const maxSymbols = 100
 // with a different configured SymbolMatcher per View. Therefore we assume that
 // Session level configuration will define the SymbolMatcher to be used for the
 // WorkspaceSymbols method.
-func WorkspaceSymbols(ctx context.Context, matcher SymbolMatcher, style SymbolStyle, views []View, query string) ([]protocol.SymbolInformation, error) {
+func WorkspaceSymbols(ctx context.Context, matcher settings.SymbolMatcher, style settings.SymbolStyle, snapshots []Snapshot, query string) ([]protocol.SymbolInformation, error) {
 	ctx, done := event.Start(ctx, "source.WorkspaceSymbols")
 	defer done()
 	if query == "" {
@@ -61,17 +60,17 @@ func WorkspaceSymbols(ctx context.Context, matcher SymbolMatcher, style SymbolSt
 
 	var s symbolizer
 	switch style {
-	case DynamicSymbols:
+	case settings.DynamicSymbols:
 		s = dynamicSymbolMatch
-	case FullyQualifiedSymbols:
+	case settings.FullyQualifiedSymbols:
 		s = fullyQualifiedSymbolMatch
-	case PackageQualifiedSymbols:
+	case settings.PackageQualifiedSymbols:
 		s = packageSymbolMatch
 	default:
 		panic(fmt.Errorf("unknown symbol style: %v", style))
 	}
 
-	return collectSymbols(ctx, views, matcher, s, query)
+	return collectSymbols(ctx, snapshots, matcher, s, query)
 }
 
 // A matcherFunc returns the index and score of a symbol match.
@@ -158,17 +157,17 @@ func packageSymbolMatch(space []string, name string, pkg *Metadata, matcher matc
 	return nil, 0
 }
 
-func buildMatcher(matcher SymbolMatcher, query string) matcherFunc {
+func buildMatcher(matcher settings.SymbolMatcher, query string) matcherFunc {
 	switch matcher {
-	case SymbolFuzzy:
+	case settings.SymbolFuzzy:
 		return parseQuery(query, newFuzzyMatcher)
-	case SymbolFastFuzzy:
+	case settings.SymbolFastFuzzy:
 		return parseQuery(query, func(query string) matcherFunc {
 			return fuzzy.NewSymbolMatcher(query).Match
 		})
-	case SymbolCaseSensitive:
+	case settings.SymbolCaseSensitive:
 		return matchExact(query)
-	case SymbolCaseInsensitive:
+	case settings.SymbolCaseInsensitive:
 		q := strings.ToLower(query)
 		exact := matchExact(q)
 		wrapper := []string{""}
@@ -297,29 +296,24 @@ func (c comboMatcher) match(chunks []string) (int, float64) {
 //     of zero indicates no match.
 //   - A symbolizer determines how we extract the symbol for an object. This
 //     enables the 'symbolStyle' configuration option.
-func collectSymbols(ctx context.Context, views []View, matcherType SymbolMatcher, symbolizer symbolizer, query string) ([]protocol.SymbolInformation, error) {
+func collectSymbols(ctx context.Context, snapshots []Snapshot, matcherType settings.SymbolMatcher, symbolizer symbolizer, query string) ([]protocol.SymbolInformation, error) {
 	// Extract symbols from all files.
 	var work []symbolFile
 	var roots []string
-	seen := make(map[span.URI]bool)
+	seen := make(map[protocol.DocumentURI]bool)
 	// TODO(adonovan): opt: parallelize this loop? How often is len > 1?
-	for _, v := range views {
-		snapshot, release, err := v.Snapshot()
-		if err != nil {
-			continue // view is shut down; continue with others
-		}
-		defer release()
-
+	for _, snapshot := range snapshots {
 		// Use the root view URIs for determining (lexically)
 		// whether a URI is in any open workspace.
-		roots = append(roots, strings.TrimRight(string(v.Folder()), "/"))
+		folderURI := snapshot.Folder()
+		roots = append(roots, strings.TrimRight(string(folderURI), "/"))
 
-		filters := v.Options().DirectoryFilters
+		filters := snapshot.Options().DirectoryFilters
 		filterer := NewFilterer(filters)
-		folder := filepath.ToSlash(v.Folder().Filename())
+		folder := filepath.ToSlash(folderURI.Path())
 
 		workspaceOnly := true
-		if v.Options().SymbolScope == AllSymbolScope {
+		if snapshot.Options().SymbolScope == settings.AllSymbolScope {
 			workspaceOnly = false
 		}
 		symbols, err := snapshot.Symbols(ctx, workspaceOnly)
@@ -328,7 +322,7 @@ func collectSymbols(ctx context.Context, views []View, matcherType SymbolMatcher
 		}
 
 		for uri, syms := range symbols {
-			norm := filepath.ToSlash(uri.Filename())
+			norm := filepath.ToSlash(uri.Path())
 			nm := strings.TrimPrefix(norm, folder)
 			if filterer.Disallow(nm) {
 				continue
@@ -448,7 +442,7 @@ func convertFilterToRegexp(filter string) *regexp.Regexp {
 
 // symbolFile holds symbol information for a single file.
 type symbolFile struct {
-	uri  span.URI
+	uri  protocol.DocumentURI
 	md   *Metadata
 	syms []Symbol
 }
@@ -484,7 +478,10 @@ func matchFile(store *symbolStore, symbolizer symbolizer, matcher matcherFunc, r
 			// every field or method nesting level to access the field decreases
 			// the score by a factor of 1.0 - depth*depthFactor, up to a depth of
 			// 3.
-			depthFactor = 0.2
+			//
+			// Use a small constant here, as this exists mostly to break ties
+			// (e.g. given a type Foo and a field x.Foo, prefer Foo).
+			depthFactor = 0.01
 		)
 
 		startWord := true
@@ -582,33 +579,6 @@ func (sc *symbolStore) results() []protocol.SymbolInformation {
 	return res
 }
 
-func typeToKind(typ types.Type) protocol.SymbolKind {
-	switch typ := typ.Underlying().(type) {
-	case *types.Interface:
-		return protocol.Interface
-	case *types.Struct:
-		return protocol.Struct
-	case *types.Signature:
-		if typ.Recv() != nil {
-			return protocol.Method
-		}
-		return protocol.Function
-	case *types.Named:
-		return typeToKind(typ.Underlying())
-	case *types.Basic:
-		i := typ.Info()
-		switch {
-		case i&types.IsNumeric != 0:
-			return protocol.Number
-		case i&types.IsBoolean != 0:
-			return protocol.Boolean
-		case i&types.IsString != 0:
-			return protocol.String
-		}
-	}
-	return protocol.Variable
-}
-
 // symbolInformation is a cut-down version of protocol.SymbolInformation that
 // allows struct values of this type to be used as map keys.
 type symbolInformation struct {
@@ -616,7 +586,7 @@ type symbolInformation struct {
 	symbol    string
 	container string
 	kind      protocol.SymbolKind
-	uri       span.URI
+	uri       protocol.DocumentURI
 	rng       protocol.Range
 }
 
@@ -628,7 +598,7 @@ func (s symbolInformation) asProtocolSymbolInformation() protocol.SymbolInformat
 		Name: s.symbol,
 		Kind: s.kind,
 		Location: protocol.Location{
-			URI:   protocol.URIFromSpanURI(s.uri),
+			URI:   s.uri,
 			Range: s.rng,
 		},
 		ContainerName: s.container,
