@@ -6,6 +6,7 @@ package completion
 
 import (
 	"fmt"
+	"sort"
 	"strings"
 	"testing"
 	"time"
@@ -516,6 +517,7 @@ func main() {
 `
 	WithOptions(
 		WindowsLineEndings(),
+		Settings{"ui.completion.usePlaceholders": true},
 	).Run(t, src, func(t *testing.T, env *Env) {
 		// Trigger unimported completions for the mod.com package.
 		env.OpenFile("main.go")
@@ -552,18 +554,23 @@ package b
 
 func F0(a, b int, c float64) {}
 func F1(int, chan *string) {}
+func F2[K, V any](map[K]V, chan V) {} // missing type parameters was issue #60959
+func F3[K comparable, V any](map[K]V, chan V) {}
 `
 	WithOptions(
 		WindowsLineEndings(),
+		Settings{"ui.completion.usePlaceholders": true},
 	).Run(t, src, func(t *testing.T, env *Env) {
 		env.OpenFile("a/a.go")
 		env.Await(env.DoneWithOpen())
 
-		// The table lists the expected completions as they appear in Items.
+		// The table lists the expected completions of b.F as they appear in Items.
 		const common = "package a\r\n\r\nimport \"example.com/b\"\r\n\r\nvar _ = "
 		for i, want := range []string{
 			common + "b.F0(${1:a int}, ${2:b int}, ${3:c float64})\r\n",
 			common + "b.F1(${1:_ int}, ${2:_ chan *string})\r\n",
+			common + "b.F2[${1:K any}, ${2:V any}](${3:_ map[K]V}, ${4:_ chan V})\r\n",
+			common + "b.F3[${1:K comparable}, ${2:V any}](${3:_ map[K]V}, ${4:_ chan V})\r\n",
 		} {
 			loc := env.RegexpSearch("a/a.go", "b.F()")
 			completions := env.Completion(loc)
@@ -684,7 +691,6 @@ func _() {
 }
 
 func TestDefinition(t *testing.T) {
-	testenv.NeedsGo1Point(t, 17) // in go1.16, The FieldList in func x is not empty
 	files := `
 -- go.mod --
 module mod.com
@@ -733,10 +739,7 @@ package foo
 }
 
 // Test that completing a definition replaces source text when applied, golang/go#56852.
-// Note: With go <= 1.16 the completions does not add parameters and fails these tests.
 func TestDefinitionReplaceRange(t *testing.T) {
-	testenv.NeedsGo1Point(t, 17)
-
 	const mod = `
 -- go.mod --
 module mod.com
@@ -860,3 +863,137 @@ use ./dir/foobar/
 		}
 	})
 }
+
+func TestBuiltinCompletion(t *testing.T) {
+	const files = `
+-- go.mod --
+module mod.com
+
+go 1.18
+-- a.go --
+package a
+
+func _() {
+	// here
+}
+`
+
+	Run(t, files, func(t *testing.T, env *Env) {
+		env.OpenFile("a.go")
+		result := env.Completion(env.RegexpSearch("a.go", `// here`))
+		builtins := []string{
+			"any", "append", "bool", "byte", "cap", "close",
+			"comparable", "complex", "complex128", "complex64", "copy", "delete",
+			"error", "false", "float32", "float64", "imag", "int", "int16", "int32",
+			"int64", "int8", "len", "make", "new", "panic", "print", "println", "real",
+			"recover", "rune", "string", "true", "uint", "uint16", "uint32", "uint64",
+			"uint8", "uintptr", "nil",
+		}
+		if testenv.Go1Point() >= 21 {
+			builtins = append(builtins, "clear", "max", "min")
+		}
+		sort.Strings(builtins)
+		var got []string
+
+		for _, item := range result.Items {
+			// TODO(rfindley): for flexibility, ignore zero while it is being
+			// implemented. Remove this if/when zero lands.
+			if item.Label != "zero" {
+				got = append(got, item.Label)
+			}
+		}
+		sort.Strings(got)
+
+		if diff := cmp.Diff(builtins, got); diff != "" {
+			t.Errorf("Completion: unexpected mismatch (-want +got):\n%s", diff)
+		}
+	})
+}
+
+func TestOverlayCompletion(t *testing.T) {
+	const files = `
+-- go.mod --
+module foo.test
+
+go 1.18
+
+-- foo/foo.go --
+package foo
+
+type Foo struct{}
+`
+
+	Run(t, files, func(t *testing.T, env *Env) {
+		env.CreateBuffer("nodisk/nodisk.go", `
+package nodisk
+
+import (
+	"foo.test/foo"
+)
+
+func _() {
+	foo.Foo()
+}
+`)
+		list := env.Completion(env.RegexpSearch("nodisk/nodisk.go", "foo.(Foo)"))
+		want := []string{"Foo"}
+		var got []string
+		for _, item := range list.Items {
+			got = append(got, item.Label)
+		}
+		if diff := cmp.Diff(want, got); diff != "" {
+			t.Errorf("Completion: unexpected mismatch (-want +got):\n%s", diff)
+		}
+	})
+}
+
+// Fix for golang/go#60062: unimported completion included "golang.org/toolchain" results.
+func TestToolchainCompletions(t *testing.T) {
+	const files = `
+-- go.mod --
+module foo.test/foo
+
+go 1.21
+
+-- foo.go --
+package foo
+
+func _() {
+	os.Open
+}
+
+func _() {
+	strings
+}
+`
+
+	const proxy = `
+-- golang.org/toolchain@v0.0.1-go1.21.1.linux-amd64/go.mod --
+module golang.org/toolchain
+-- golang.org/toolchain@v0.0.1-go1.21.1.linux-amd64/src/os/os.go --
+package os
+
+func Open() {}
+-- golang.org/toolchain@v0.0.1-go1.21.1.linux-amd64/src/strings/strings.go --
+package strings
+
+func Join() {}
+`
+
+	WithOptions(
+		ProxyFiles(proxy),
+	).Run(t, files, func(t *testing.T, env *Env) {
+		env.RunGoCommand("mod", "download", "golang.org/toolchain@v0.0.1-go1.21.1.linux-amd64")
+		env.OpenFile("foo.go")
+
+		for _, pattern := range []string{"os.Open()", "string()"} {
+			loc := env.RegexpSearch("foo.go", pattern)
+			res := env.Completion(loc)
+			for _, item := range res.Items {
+				if strings.Contains(item.Detail, "golang.org/toolchain") {
+					t.Errorf("Completion(...) returned toolchain item %#v", item)
+				}
+			}
+		}
+	})
+}
