@@ -5,7 +5,6 @@
 package misc
 
 import (
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"strings"
@@ -15,7 +14,6 @@ import (
 	"golang.org/x/tools/gopls/internal/lsp/tests/compare"
 
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
-	"golang.org/x/tools/internal/testenv"
 )
 
 // Tests golang/go#38815.
@@ -59,7 +57,7 @@ func TestIssue59124(t *testing.T) {
 	const stuff = `
 -- go.mod --
 module foo
-go 1.29
+go 1.19
 -- a.go --
 //line foo.y:102
 package main
@@ -175,7 +173,7 @@ import "example.com/x"
 
 var _, _ = x.X, y.Y
 `
-	modcache, err := ioutil.TempDir("", "TestGOMODCACHE-modcache")
+	modcache, err := os.MkdirTemp("", "TestGOMODCACHE-modcache")
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -242,7 +240,6 @@ func TestA(t *testing.T) {
 
 // Test for golang/go#52784
 func TestGoWorkImports(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18)
 	const pkg = `
 -- go.work --
 go 1.19
