@@ -12,12 +12,11 @@ import (
 	"testing"
 
 	"golang.org/x/tools/gopls/internal/bug"
+	"golang.org/x/tools/gopls/internal/goversion"
 	"golang.org/x/tools/gopls/internal/hooks"
-	"golang.org/x/tools/gopls/internal/lsp"
 	"golang.org/x/tools/gopls/internal/lsp/fake"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/internal/gocommand"
-	"golang.org/x/tools/internal/testenv"
 
 	. "golang.org/x/tools/gopls/internal/lsp/regtest"
 )
@@ -172,40 +171,17 @@ func TestReloadOnlyOnce(t *testing.T) {
 		ProxyFiles(workspaceProxy),
 		WorkspaceFolders("pkg"),
 	).Run(t, workspaceModule, func(t *testing.T, env *Env) {
-		dir := env.Sandbox.Workdir.URI("goodbye").SpanURI().Filename()
+		dir := env.Sandbox.Workdir.URI("goodbye").Path()
 		goModWithReplace := fmt.Sprintf(`%s
 replace random.org => %s
 `, env.ReadWorkspaceFile("pkg/go.mod"), dir)
 		env.WriteWorkspaceFile("pkg/go.mod", goModWithReplace)
-		env.AfterChange(
+		env.Await(
 			LogMatching(protocol.Info, `packages\.Load #\d+\n`, 2, false),
 		)
 	})
 }
 
-// This test checks that gopls updates the set of files it watches when a
-// replace target is added to the go.mod.
-func TestWatchReplaceTargets(t *testing.T) {
-	t.Skipf("skipping known-flaky test: see https://go.dev/issue/50748")
-
-	WithOptions(
-		ProxyFiles(workspaceProxy),
-		WorkspaceFolders("pkg"),
-	).Run(t, workspaceModule, func(t *testing.T, env *Env) {
-		// Add a replace directive and expect the files that gopls is watching
-		// to change.
-		dir := env.Sandbox.Workdir.URI("goodbye").SpanURI().Filename()
-		goModWithReplace := fmt.Sprintf(`%s
-replace random.org => %s
-`, env.ReadWorkspaceFile("pkg/go.mod"), dir)
-		env.WriteWorkspaceFile("pkg/go.mod", goModWithReplace)
-		env.AfterChange(
-			UnregistrationMatching("didChangeWatchedFiles"),
-			RegistrationMatching("didChangeWatchedFiles"),
-		)
-	})
-}
-
 const workspaceModuleProxy = `
 -- example.com@v1.2.3/go.mod --
 module example.com
@@ -229,9 +205,7 @@ package b
 func Hello() {}
 `
 
-func TestAutomaticWorkspaceModule_Interdependent(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18) // uses go.work
-	const multiModule = `
+const multiModule = `
 -- moda/a/go.mod --
 module a.com
 
@@ -260,6 +234,8 @@ func Hello() int {
 	var x int
 }
 `
+
+func TestAutomaticWorkspaceModule_Interdependent(t *testing.T) {
 	WithOptions(
 		ProxyFiles(workspaceModuleProxy),
 	).Run(t, multiModule, func(t *testing.T, env *Env) {
@@ -302,7 +278,7 @@ module b.com
 
 go 1.12
 `
-	const multiModule = `
+	const files = `
 -- go.mod --
 module a.com
 
@@ -321,7 +297,7 @@ func main() {
 `
 	WithOptions(
 		ProxyFiles(proxy),
-	).Run(t, multiModule, func(t *testing.T, env *Env) {
+	).Run(t, files, func(t *testing.T, env *Env) {
 		env.OnceMet(
 			InitialWorkspaceLoad,
 			Diagnostics(env.AtRegexp("main.go", "x")),
@@ -335,7 +311,6 @@ func main() {
 // TODO(golang/go#55331): delete this placeholder along with experimental
 // workspace module.
 func TestDeleteModule_Interdependent(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18) // uses go.work
 	const multiModule = `
 -- go.work --
 go 1.18
@@ -402,7 +377,6 @@ func Hello() int {
 // Tests that the version of the module used changes after it has been added
 // to the workspace.
 func TestCreateModule_Interdependent(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18) // uses go.work
 	const multiModule = `
 -- go.work --
 go 1.18
@@ -467,7 +441,6 @@ func Hello() int {
 // This test confirms that a gopls workspace can recover from initialization
 // with one invalid module.
 func TestOneBrokenModule(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18) // uses go.work
 	const multiModule = `
 -- go.work --
 go 1.18
@@ -531,7 +504,6 @@ module example.com/bar
 }
 
 func TestUseGoWork(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18) // uses go.work
 	// This test validates certain functionality related to using a go.work
 	// file to specify workspace modules.
 	const multiModule = `
@@ -575,10 +547,18 @@ use (
 `
 	WithOptions(
 		ProxyFiles(workspaceModuleProxy),
+		Settings{
+			"subdirWatchPatterns": "on",
+		},
 	).Run(t, multiModule, func(t *testing.T, env *Env) {
-		// Initially, the go.work should cause only the a.com module to be
-		// loaded. Validate this by jumping to a definition in b.com and ensuring
-		// that we go to the module cache.
+		// Initially, the go.work should cause only the a.com module to be loaded,
+		// so we shouldn't get any file watches for modb. Further validate this by
+		// jumping to a definition in b.com and ensuring that we go to the module
+		// cache.
+		env.OnceMet(
+			InitialWorkspaceLoad,
+			NoFileWatchMatching("modb"),
+		)
 		env.OpenFile("moda/a/a.go")
 		env.Await(env.DoneWithOpen())
 
@@ -610,9 +590,13 @@ use (
 `)
 
 		// As of golang/go#54069, writing go.work to the workspace triggers a
-		// workspace reload.
+		// workspace reload, and new file watches.
 		env.AfterChange(
 			Diagnostics(env.AtRegexp("modb/b/b.go", "x")),
+			// TODO(golang/go#60340): we don't get a file watch yet, because
+			// updateWatchedDirectories runs before snapshot.load. Instead, we get it
+			// after the next change (the didOpen below).
+			// FileWatchMatching("modb"),
 		)
 
 		// Jumping to definition should now go to b.com in the workspace.
@@ -623,7 +607,13 @@ use (
 		// Now, let's modify the go.work *overlay* (not on disk), and verify that
 		// this change is only picked up once it is saved.
 		env.OpenFile("go.work")
-		env.AfterChange()
+		env.AfterChange(
+			// TODO(golang/go#60340): delete this expectation in favor of
+			// the commented-out expectation above, once we fix the evaluation order
+			// of file watches. We should not have to wait for a second change to get
+			// the correct watches.
+			FileWatchMatching("modb"),
+		)
 		env.SetBufferContent("go.work", `go 1.17
 
 use (
@@ -675,8 +665,6 @@ use (
 }
 
 func TestUseGoWorkDiagnosticMissingModule(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18) // uses go.work
-
 	const files = `
 -- go.work --
 go 1.18
@@ -708,7 +696,6 @@ module example.com/bar
 }
 
 func TestUseGoWorkDiagnosticSyntaxError(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18)
 	const files = `
 -- go.work --
 go 1.18
@@ -726,8 +713,6 @@ replace
 }
 
 func TestUseGoWorkHover(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18)
-
 	const files = `
 -- go.work --
 go 1.18
@@ -763,7 +748,6 @@ module example.com/bar/baz
 }
 
 func TestExpandToGoWork(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18)
 	const workspace = `
 -- moda/a/go.mod --
 module a.com
@@ -837,7 +821,6 @@ var _ = fmt.Printf
 }
 
 func TestGoWork_V2Module(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18) // uses go.work
 	// When using a go.work, we must have proxy content even if it is replaced.
 	const proxy = `
 -- b.com/v2@v2.1.9/go.mod --
@@ -928,7 +911,6 @@ func TestMultiModule_OneBrokenModule(t *testing.T) {
 	// missing go.sum with diagnostics. With go.work files, this doesn't work:
 	// the go.command will happily write go.work.sum.
 	t.Skip("golang/go#57509: go.mod diagnostics do not work in go.work mode")
-	testenv.NeedsGo1Point(t, 18) // uses go.work
 	const files = `
 -- go.work --
 go 1.18
@@ -1027,7 +1009,6 @@ package main
 }
 
 func TestAddAndRemoveGoWork(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18)
 	// Use a workspace with a module in the root directory to exercise the case
 	// where a go.work is added to the existing root directory. This verifies
 	// that we're detecting changes to the module source, not just the root
@@ -1059,7 +1040,7 @@ func main() {}
 		// package declaration.
 		env.AfterChange(
 			NoDiagnostics(ForFile("main.go")),
-			Diagnostics(AtPosition("b/main.go", 0, 0)),
+			Diagnostics(env.AtRegexp("b/main.go", "package (main)")),
 		)
 		env.WriteWorkspaceFile("go.work", `go 1.16
 
@@ -1085,7 +1066,7 @@ use (
 
 		env.AfterChange(
 			NoDiagnostics(ForFile("main.go")),
-			Diagnostics(AtPosition("b/main.go", 0, 0)),
+			Diagnostics(env.AtRegexp("b/main.go", "package (main)")),
 		)
 	})
 }
@@ -1137,8 +1118,6 @@ func (Server) Foo() {}
 
 // Test for golang/go#48929.
 func TestClearNonWorkspaceDiagnostics(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18) // uses go.work
-
 	const ws = `
 -- go.work --
 go 1.18
@@ -1194,7 +1173,7 @@ import (
 // supported.
 func TestOldGoNotification_SupportedVersion(t *testing.T) {
 	v := goVersion(t)
-	if v < lsp.OldestSupportedGoVersion() {
+	if v < goversion.OldestSupported() {
 		t.Skipf("go version 1.%d is unsupported", v)
 	}
 
@@ -1211,7 +1190,7 @@ func TestOldGoNotification_SupportedVersion(t *testing.T) {
 // legacy Go versions (see also TestOldGoNotification_Fake)
 func TestOldGoNotification_UnsupportedVersion(t *testing.T) {
 	v := goVersion(t)
-	if v >= lsp.OldestSupportedGoVersion() {
+	if v >= goversion.OldestSupported() {
 		t.Skipf("go version 1.%d is supported", v)
 	}
 
@@ -1231,15 +1210,15 @@ func TestOldGoNotification_Fake(t *testing.T) {
 	// oldest supported Go version here, we can at least ensure that the
 	// ShowMessage pop-up works.
 	ctx := context.Background()
-	goversion, err := gocommand.GoVersion(ctx, gocommand.Invocation{}, &gocommand.Runner{})
+	version, err := gocommand.GoVersion(ctx, gocommand.Invocation{}, &gocommand.Runner{})
 	if err != nil {
 		t.Fatal(err)
 	}
-	defer func(t []lsp.GoVersionSupport) {
-		lsp.GoVersionTable = t
-	}(lsp.GoVersionTable)
-	lsp.GoVersionTable = []lsp.GoVersionSupport{
-		{GoVersion: goversion, InstallGoplsVersion: "v1.0.0"},
+	defer func(t []goversion.Support) {
+		goversion.Supported = t
+	}(goversion.Supported)
+	goversion.Supported = []goversion.Support{
+		{GoVersion: version, InstallGoplsVersion: "v1.0.0"},
 	}
 
 	Run(t, "", func(t *testing.T, env *Env) {
@@ -1261,3 +1240,23 @@ func goVersion(t *testing.T) int {
 	}
 	return goversion
 }
+
+func TestGoworkMutation(t *testing.T) {
+	WithOptions(
+		ProxyFiles(workspaceModuleProxy),
+	).Run(t, multiModule, func(t *testing.T, env *Env) {
+		env.RunGoCommand("work", "init")
+		env.RunGoCommand("work", "use", "-r", ".")
+		env.AfterChange(
+			Diagnostics(env.AtRegexp("moda/a/a.go", "x")),
+			Diagnostics(env.AtRegexp("modb/b/b.go", "x")),
+			NoDiagnostics(env.AtRegexp("moda/a/a.go", `b\.Hello`)),
+		)
+		env.RunGoCommand("work", "edit", "-dropuse", "modb")
+		env.Await(
+			Diagnostics(env.AtRegexp("moda/a/a.go", "x")),
+			NoDiagnostics(env.AtRegexp("modb/b/b.go", "x")),
+			Diagnostics(env.AtRegexp("moda/a/a.go", `b\.Hello`)),
+		)
+	})
+}
